/* ************************************************************

   Program:  scan
   File:     CalcQTLlod.c
   Author:   Zachary Skrivanek
   Date:     February 4, 2000
   Purpose:  calculates lod score for QTL.
   
************************************************************* */
#include <stdio.h> /* scanf() printf() */
#include <stdlib.h> /* pow() */
#include <math.h> /* sqrt,pow,log10 */
#include "simple.h"   /* c commands */ 
/* #include <sys/ddi.h> I don't think I need this and besides
it causes problems on allele */

#define TRUE 1
#define FALSE 0 

#define ALPHA 0 
#define BETA 1 
#define SIGMA 2 

double  CalcQTLlod(double *line,
                   double ***IBDprob,
		   prs    **pairs,
                   int    *npairs,
		   int    npedigrees){

  int    ibd, ped, pair; /* counter */
  double slope, intercept, sigma_2;
  double lik, null_lik, null_mean, null_var, y, lik_tmp, null_lik_tmp;
  double sum_y, sum_y_2;
  int    n;
  double v[3];

  double  norm_pdf(double, double, double);

  v[0]=0.0;
  v[1]=0.5;
  v[2]=1.0;
		   

  slope = line[BETA];
  intercept = line[ALPHA];
  sigma_2 = line[SIGMA];
  lik=0.0;
  null_lik=0.0;
  sum_y=0.0;
  sum_y_2=0.0;
  null_mean=0.0;
  null_var=0.0;
  n=0;
  for(ped=0; ped<npedigrees; ped++)
    for(pair=0; pair<npairs[ped]; pair++){
      y=pairs[ped][pair].pheno;
      sum_y += y;
      sum_y_2 += pow(y,2.0);
      n++;
    }
    null_mean=sum_y/((double) n); 
    null_var=(sum_y_2-pow(sum_y,2.0)/((double) n));
    null_var /= ((double) n);

  for(ped=0; ped<npedigrees; ped++){
    for(pair=0; pair<npairs[ped]; pair++){
      y=pairs[ped][pair].pheno;
      lik_tmp=0.0;
      null_lik_tmp=0.0;
      for(ibd=0; ibd<3; ibd++){
        lik_tmp += norm_pdf(y,intercept+slope*v[ibd],sigma_2)*IBDprob[ped][pair][ibd];
        null_lik_tmp += norm_pdf(y,null_mean,null_var)*IBDprob[ped][pair][ibd];
      } /* ibd */
      lik += log10(lik_tmp);
      null_lik += log10(null_lik_tmp);
    } /* pair */
  } /* ped */
  return(lik-null_lik);        
}
 

