/* *************************************************************

   Program:   simple
   File:      ComplexCreateSpaces.c
   Author:    Zachary Skrivanek
   Date:      January 27, 2002 
   Purpose:   Dynamically allocates the storage space needed for
              peeling and simulation in a complex pedigree.  
	      Unlike createspaces.c (not yet) I put locus in the
	      first position of the shap and simhap array.  So j
	      far I leave ibd and ibdsim alone.
   Modified:  March 15, 2002.  Added some genotype elimination
	      for founders.  Introduced nualleles, and ualleles
	      to accomplish this.

************************************************************* */

#include <stdio.h>
#include "simple.h"

#define A 0
#define B 1

#define TRUE  1
#define FALSE 0

#define UNKNOWN 0 

#define PID 0

#define FOUNDER 0
#define PARENT  1
#define FINDEX  2
#define MINDEX  3
#define SID     4
#define SINDEX  5
#define SEX     6

#define MALE   1
#define FEMALE 0

#define F 0
#define M 1

void ComplexCreateSpaces(int     npedigrees,
		         int     *pedigreesize,
		         int     *npeels,
		         sindex  **cutset,
		         sindex  **peelset,
		         int     ***ancestry,
		         int     *nalleles,
                         int     nmarkers,
		         int     ****marker,
		         int     nloci,
                         int     *****ibdobs,
                         int     *****ibdsim,
		         int     **ibdF,
		         int     **ibdM,
		         int     *****simhap,
		         int     ****shap,
		         int     **hap,
			 int     ****spacesize,
		         int     *****cutspaces,
		         int     ****cutspace,
		         int     *****peelspaces,
		         int     ****peelspace,
			 double  *****wpp,
		         double  *****rfunction,
		         double  **simprob,
		         int     **whaps,
		         int     **whaps2,
			 int     ****nualleles,
			 int     *****ualleles)	
{

  int     ****tpeelspaces, ****tcutspaces, ***tpeelspace, ***tcutspace;
  int     ***tspacesize;
  double  ****twpp, ****trfunction;
  double  *tsimprob;
  int     *twhaps, *twhaps2;
  int     ****tibdobs;
  int     ****tibdsim;
  int     *tibdF;
  int     *tibdM;
  int     ****tsimhap;
  int     ***tshap;
  int     *thap;
  int     ***tnualleles, ****tualleles;
  int     mA, mB;
  int     fmA,fmB;
  int     mmA,mmB;
  int     fromF,fromM;
  int     findex,mindex;
  int     maxspace;

  count   ped, locus, peel, peel2, i, j, k, l;
  sindex  *c, *p;

  void    GE();

  /**************************************************/
  /* determine marker and disease allele spacesizes */
  /**************************************************/

  /* allocate space of spacesize */
  /* I'll later change the order so locus is first */
  tspacesize = (int ***) calloc(npedigrees,sizeof(int **));
  for(ped=0; ped < npedigrees; ped++) {
    tspacesize[ped] = (int **) calloc(pedigreesize[ped],sizeof(int *));
    for(i=0; i < pedigreesize[ped]; i++) 
      tspacesize[ped][i] = (int *) calloc(nloci,sizeof(int));
  }


  /* allocate space to tnualleles and tualleles */
  tnualleles= (int ***) calloc(nmarkers, sizeof(int **));
  tualleles= (int ****) calloc(nmarkers, sizeof(int ***));
  for(locus=0; locus < nmarkers; locus++) {
    tnualleles[locus] = (int **) calloc(npedigrees,sizeof(int *));
    tualleles[locus] = (int ***) calloc(npedigrees,sizeof(int **));
    for(ped=0; ped < npedigrees; ped++) {
      tnualleles[locus][ped] = (int *) calloc(pedigreesize[ped],sizeof(int));
      tualleles[locus][ped] = (int **) calloc(pedigreesize[ped],sizeof(int *));
      for(j=0; j < pedigreesize[ped]; j++)
        tualleles[locus][ped][j] = (int *) calloc(2,sizeof(int));
    }
  }


  /* first do peelspace */
  tpeelspaces = (int ****) calloc(nloci,sizeof(int ***));
  for(locus=0; locus < nmarkers; locus++) {
    tpeelspaces[locus] = (int ***) calloc(npedigrees,sizeof(int **));
    for(ped=0; ped < npedigrees; ped++) {
      tpeelspaces[locus][ped] = (int **) calloc(npeels[ped],sizeof(int *));
      for(peel=0; peel < npeels[ped]; peel++) {
	p = &peelset[ped][peel];
        tpeelspaces[locus][ped][peel] = (int *) calloc(p->n,sizeof(int));
        for(i=0; i < p->n; i++) {
          mA = marker[ped][p->ped[i]][locus][A];
	  mB = marker[ped][p->ped[i]][locus][B];
          if( mA == 0 && mB == 0) {
	    if( ancestry[ped][p->ped[i]][PARENT] )
            /* if we have a parent do some genotype elimination based on kids */
              GE(&ancestry[ped][0],  
                 pedigreesize[ped],
	         p->ped[i],
	         locus,
	         &marker[ped][0],
	         nalleles[locus],
	         tnualleles[locus][ped][p->ped[i]],
	         &tualleles[locus][ped][p->ped[i]][0],
	         &tpeelspaces[locus][ped][peel][i]);
            else
	      tpeelspaces[locus][ped][peel][i] = nalleles[locus] * nalleles[locus];
          }
          else if( mA == mB ) {
            tpeelspaces[locus][ped][peel][i] = 1;
	    tnualleles[locus][ped][p->ped[i]]=1;
	    tualleles[locus][ped][p->ped[i]][A]=mA;
	    tualleles[locus][ped][p->ped[i]][B]=mA;
          }
          else {
	    if( !ancestry[ped][p->ped[i]][FOUNDER] ) {
              findex = ancestry[ped][p->ped[i]][FINDEX];
              mindex = ancestry[ped][p->ped[i]][MINDEX];
              fmA = marker[ped][findex][locus][A];
              fmB = marker[ped][findex][locus][B];
              mmA = marker[ped][mindex][locus][A];
              mmB = marker[ped][mindex][locus][B];
              orderhaplotypes(i+1,mA,mB,fmA,fmB,mmA,mmB,&fromF,&fromM);
              if (fromF!=0 && fromM!=0){
		tpeelspaces[locus][ped][peel][i] = 1;
	        tnualleles[locus][ped][p->ped[i]]=2;
	        tualleles[locus][ped][p->ped[i]][F]=fromF;
	        tualleles[locus][ped][p->ped[i]][M]=fromM;
              }
              else{
                tpeelspaces[locus][ped][peel][i] = 2;
	        tnualleles[locus][ped][p->ped[i]]=2;
	        tualleles[locus][ped][p->ped[i]][F]=mA;
	        tualleles[locus][ped][p->ped[i]][M]=mB;
              }
            } 
            else{
              tpeelspaces[locus][ped][peel][i] = 2;
	      tnualleles[locus][ped][p->ped[i]]=2;
	      tualleles[locus][ped][p->ped[i]][F]=mA;
	      tualleles[locus][ped][p->ped[i]][M]=mB;
            }
          }
        }
      }
    }
  }
  /* allocate space of disease loci */
  for(locus=nmarkers; locus<nloci; locus++){
    tpeelspaces[locus] = (int ***) calloc(npedigrees,sizeof(int **));
    for(ped=0; ped < npedigrees; ped++) {
      tpeelspaces[locus][ped] = (int **) calloc(npeels[ped],sizeof(int *));
      for(peel=0; peel < npeels[ped]; peel++) {
	p = &peelset[ped][peel];
        tpeelspaces[locus][ped][peel] = (int *) calloc(p->n,sizeof(int));
        for(i=0; i < p->n; i++) 
          tpeelspaces[locus][ped][peel][i] = 4;
      }
    }
  }

  /* allocate spaces to spacesize */
  for(locus=0; locus < nloci; locus++) 
    for(ped=0; ped < npedigrees; ped++) 
      for(peel=0; peel < npeels[ped]; peel++) {
	p = &peelset[ped][peel];
        for(i=0; i < p->n; i++) 
          tspacesize[ped][p->ped[i]][locus]=tpeelspaces[locus][ped][peel][i];
      }

  /* now add up peelspaces to get peelspace */
  tpeelspace = (int ***) calloc(nloci,sizeof(int **));
  for(locus=0; locus < nloci; locus++) {
    tpeelspace[locus] = (int **) calloc(npedigrees,sizeof(int *));
    for(ped=0; ped < npedigrees; ped++) {
      tpeelspace[locus][ped] = (int *) calloc(npeels[ped],sizeof(int));
      for(peel=0; peel < npeels[ped]; peel++) {
	tpeelspace[locus][ped][peel]=1;
        for(i=0; i < peelset[ped][peel].n; i++) 
	  tpeelspace[locus][ped][peel] *= tpeelspaces[locus][ped][peel][i];
      }
    }
  }

  /* allocate cutspaces using peelspaces */
  tcutspaces = (int ****) calloc(nloci,sizeof(int ***));
  for(locus=0; locus < nloci; locus++) {
    tcutspaces[locus] = (int ***) calloc(npedigrees,sizeof(int **));
    for(ped=0; ped < npedigrees; ped++) {
      tcutspaces[locus][ped] = (int **) calloc(npeels[ped],sizeof(int *));
      for(peel=0; peel < npeels[ped]; peel++) {
	c = &cutset[ped][peel];
        tcutspaces[locus][ped][peel] = (int *) calloc(c->n,sizeof(int));
        for(i=0; i < c->n; i++) 
          for(peel2=0; peel2 < npeels[ped]; peel2++) {
	    p = &peelset[ped][peel2];
            for(j=0; j < p->n; j++) 
	      if(c->pid[i]==p->pid[j])
	        tcutspaces[locus][ped][peel][i]=tpeelspaces[locus][ped][peel2][j];
          }
      }
    }
  } 

  /* now add up cutspaces to get cutspace */
  tcutspace = (int ***) calloc(nloci,sizeof(int **));
  for(locus=0; locus < nloci; locus++) {
    tcutspace[locus] = (int **) calloc(npedigrees,sizeof(int *));
    for(ped=0; ped < npedigrees; ped++) {
    /* we don't allocate any space to the last peel */
      tcutspace[locus][ped] = (int *) calloc(npeels[ped],sizeof(int));
      for(peel=0; peel < npeels[ped]-1; peel++) {
	tcutspace[locus][ped][peel]=1;
        for(i=0; i < cutset[ped][peel].n; i++) 
	  tcutspace[locus][ped][peel] *= tcutspaces[locus][ped][peel][i];
      }
    /* for the last peel we give on space so we have some */
    /* place to store the last runction */
      tcutspace[locus][ped][npeels[ped]-1]=1;
    }
  }


  /* allocate maximum size to simprob */
  maxspace = tpeelspace[0][0][0]; 
  for(locus=0; locus < nloci; locus++) 
    for(ped=0; ped < npedigrees; ped++) 
      for(peel=1; peel < npeels[ped]; peel++) 
        if(tpeelspace[locus][ped][peel] > maxspace)
	  maxspace = tpeelspace[locus][ped][peel];

  tsimprob = (double *) calloc(maxspace, sizeof(double));


  /**************************************/
  /* create ibd spaces                  */
  /**************************************/

  tibdobs = (int ****) calloc(npedigrees,sizeof(int ***));
  tibdsim = (int ****) calloc(npedigrees,sizeof(int ***));
  for(i=0; i < npedigrees; i++) {
    tibdobs[i] = (int ***) calloc(pedigreesize[i],sizeof(int **));
    tibdsim[i] = (int ***) calloc(pedigreesize[i],sizeof(int **));
    for(j=0; j < pedigreesize[i]; j++) {
      tibdobs[i][j] = (int **) calloc(nloci,sizeof(int *));
      tibdsim[i][j] = (int **) calloc(nloci,sizeof(int *));
      for(k=0; k < nloci; k++) {
        tibdobs[i][j][k] = (int *) calloc(2,sizeof(int));
        tibdsim[i][j][k] = (int *) calloc(2,sizeof(int));
        for(l=0; l < 2; l++) {
          tibdobs[i][j][k][l] = 2;
          tibdsim[i][j][k][l] = 2;
        }
      }
    }
  }


  /**************************************/
  /* create simulation haplotype spaces */
  /**************************************/
/* implement this later 
  tsimhap = (int ****) calloc(nloci,sizeof(int ***));
  tshap = (int ***) calloc(nloci,sizeof(int **));
  for(locus=0; locus<nloci; locus++){
    tsimhap[locus] = (int ***) calloc(npedigrees,sizeof(int **));
    tshap[locus] = (int **) calloc(npedigrees,sizeof(int *));
    for(ped=0; ped<npedigrees; ped++){ 
      tsimhap[locus][ped] = (int **) calloc(pedigreesize[ped],sizeof(int *));
      tshap[locus][ped] = (int *) calloc(pedigreesize[ped],sizeof(int));
      for(i=0; i<pedigreesize[ped]; i++) 
        tsimhap[locus][ped][i] = (int *) calloc(2,sizeof(int));
    }
  }
OLD CODE
  tsimhap = (int ****) calloc(npedigrees,sizeof(int ***));
  tshap = (int ***) calloc(npedigrees,sizeof(int **));
    for(i=0; i<npedigrees; i++) {
      tsimhap[i] = (int ***) calloc(pedigreesize[i],sizeof(int **));
      tshap[i] = (int **) calloc(pedigreesize[i],sizeof(int *));
      for(j=0; j<pedigreesize[i]; j++) {
        tsimhap[i][j] = (int **) calloc(nloci,sizeof(int *));
        tshap[i][j] = (int *) calloc(nloci,sizeof(int));
        for(k=0; k<nloci; k++)
          tsimhap[i][j][k] = (int *) calloc(2,sizeof(int));
      }
    }
*/
/* i'll just change shap for now and later i'll change*/
/* simhap. But in order to cahnge simhap i have to change*/
/* updatesim */
  tsimhap = (int ****) calloc(npedigrees,sizeof(int ***));
    for(i=0; i<npedigrees; i++) {
      tsimhap[i] = (int ***) calloc(pedigreesize[i],sizeof(int **));
      for(j=0; j<pedigreesize[i]; j++) {
        tsimhap[i][j] = (int **) calloc(nloci,sizeof(int *));
        for(k=0; k<nloci; k++)
          tsimhap[i][j][k] = (int *) calloc(2,sizeof(int));
      }
    }

  tshap = (int ***) calloc(nloci,sizeof(int **));
  thap = (int *) calloc(nmarkers,sizeof(int));
  for(locus=0; locus<nloci; locus++){
    tshap[locus] = (int **) calloc(npedigrees,sizeof(int *));
    for(ped=0; ped<npedigrees; ped++)
      tshap[locus][ped] = (int *) calloc(pedigreesize[ped],sizeof(int));
  }

  /*****************************************
  *  create working ibd spaces for Peel()  *
  *****************************************/

  tibdF = (int *) calloc(nloci,sizeof(int));
  tibdM = (int *) calloc(nloci,sizeof(int));


  /********************************************
  *  create working spaces for cut/peel haps  *
  /*******************************************/

  for(ped=0; ped<npedigrees; ped++){
    maxspace = cutset[ped][0].n + peelset[ped][0].n;
    for(peel=1; peel<npeels[ped]; peel++){
      if(cutset[ped][peel].n + peelset[ped][peel].n > maxspace)
        maxspace = cutset[ped][peel].n;
    }
  }

  twhaps = (int *) calloc(maxspace, sizeof(int));
  twhaps2 = (int *) calloc(maxspace, sizeof(int));


  /********************************
  *  allocate space to rfunctions *
  ********************************/

  trfunction = (double ****) calloc(nloci, sizeof(double ***));
  for(locus=0; locus < nloci; locus++) {
    trfunction[locus] = (double ***) calloc(npedigrees, sizeof(double **));
    for(ped=0; ped < npedigrees; ped++) {
      trfunction[locus][ped] = (double **) calloc(npeels[ped], sizeof(double *));
      for(peel=0; peel < npeels[ped]; peel++) 
        trfunction[locus][ped][peel] = (double *) calloc(tcutspace[locus][ped][peel], sizeof(double));
    }
  }

  /***********************************************
  *  allocate space to working point potentials  *
  ***********************************************/

  twpp = (double ****) calloc(nloci, sizeof(double ***));
  for(locus=0; locus < nloci; locus++) {
    twpp[locus] = (double ***) calloc(npedigrees, sizeof(double **));
    for(ped=0; ped < npedigrees; ped++) {
      twpp[locus][ped] = (double **) calloc(npeels[ped], sizeof(double *));
      for(peel=0; peel < npeels[ped]; peel++) 
	twpp[locus][ped][peel] = (double *) calloc(tpeelspace[locus][ped][peel], sizeof(double));
    }
  }

  *spacesize   = tspacesize;
  *peelspaces  = tpeelspaces; 
  *cutspaces   = tcutspaces; 
  *peelspace   = tpeelspace; 
  *cutspace    = tcutspace; 
  *simprob     = tsimprob;
  *ibdobs      = tibdobs;
  *ibdsim      = tibdsim;
  *ibdF        = tibdF;
  *ibdM        = tibdM;
  *simhap      = tsimhap;
  *wpp         = twpp;
  *rfunction   = trfunction;
  *shap        = tshap;
  *hap         = thap;
  *whaps       = twhaps;
  *whaps2      = twhaps2;
  *nualleles   = tnualleles;
  *ualleles    = tualleles;
}

/* eliminate genotypes based on children */
void GE(int  **ancestry,  
        int  pedigreesize,
	int  PINDEX,
	int  locus,
	int  ***marker,
	int  nalleles,
	int  nualleles,
	int  *ualleles,
	int  *peelspaces){

  int   INDEX1, INDEX2;
  int   cA, cB, pA, pB;
  bool  UNIQUE=TRUE;

  void  nerror();

  count   i, j, k;

  if(ancestry[PINDEX][SEX]==MALE){
    INDEX1=FINDEX;
    INDEX2=MINDEX;
  }
  else{
    INDEX1=MINDEX;
    INDEX2=FINDEX;
  }

  /* find children */
  for(i = 0; i<pedigreesize; i++)
    if( ancestry[i][FOUNDER]==FALSE )
      if( ancestry[i][INDEX1]==PINDEX ){
        cA=marker[i][locus][A];
        cB=marker[i][locus][B];
        /* if we find one unknown child then continue */
        if( cA==UNKNOWN || cB==UNKNOWN ) continue;
        /* if child is homozygous then one is unique */
        else if( cA==cB ){
	  UNIQUE=TRUE;
	  for(j=0; j<nualleles; j++)
	    if(ualleles[j]==cA) UNIQUE=FALSE;
          if(UNIQUE==TRUE){
	    nualleles++;
	    ualleles[nualleles-1]=cA;
          }
        }
        else{ /* child heterozygous */
        /* get the spouse's genotype */
        /* if the child is hetero and the spouse is homo */
        /* then one of the alleles can be determined to come */
        /* from the parent and the other from the spouse.  */
	  pA=marker[ancestry[i][INDEX2]][locus][A];
	  pB=marker[ancestry[i][INDEX2]][locus][B];
	  if(pA!=UNKNOWN && pB!=UNKNOWN && pA==pB){
	    if(cA==pA){
	      UNIQUE=TRUE;
	      for(j=0; j<nualleles; j++)
	        if(ualleles[j]==cB) UNIQUE=FALSE;
              if(UNIQUE==TRUE){
	        nualleles++;
	        ualleles[nualleles-1]=cB;
              }
            }
	    else if(cB==pA){
	      UNIQUE=TRUE;
	      for(j=0; j<nualleles; j++)
	        if(ualleles[k]==cA) UNIQUE=FALSE;
              if(UNIQUE==TRUE){
	        nualleles++;
	        ualleles[nualleles-1]=cA;
              }
            }
          }
        }
      }

  switch(nualleles){
    case 0:
      *peelspaces = nalleles * nalleles;
      break;
    case 1:
      *peelspaces = 2 * nalleles - 1;
      break;
    case 2:
      *peelspaces = 2;
      break;
    default:
      nerror("Error in allocate spaces!");
  }
}



void DebugComplexCreateSpaces(int     npedigrees,
                              int     *npeels,
                              int     nmarkers,
                              int     ****marker,
                              sindex  **cutset,
			      sindex  **peelset,
			      int     ****cutspaces,
			      int     ****cutspace,
			      int     ****peelspaces,
			      int     ****peelspace)
			      
{
  char response[2];

  printf("\n\nDebug CreateSpaces()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) exit(0);
  if( strcmp(response,"y") == 0 ) {
    
    count   ped, locus, peel, peel2, i, j, k, l;
    sindex  *c, *p;

    printf("\n\n");
    for(ped=0; ped < npedigrees; ped++) {
      printf("Space Sizes for Pedigree #%2d\n",ped+1);
      printf("****************************\n");
      printf("\n\n");
      for(locus=0; locus < nmarkers; locus++) {
        printf("locus %2d\n",locus+1);
        printf("********\n");
        printf("peel: cutspaces (mA, mB) : cutspace : peelspaces (mA, mB) : peelspace\n");
        for(peel=0; peel < npeels[ped]; peel++) {
	  printf("%2d : ",peel+1);
	  c = &cutset[ped][peel];
          for(i=0; i < c->n; i++) 
            printf("%4d (%2d, %2d)  ",
		   cutspaces[locus][ped][peel][i],
                   marker[ped][c->ped[i]][locus][A],
                   marker[ped][c->ped[i]][locus][B]);
          if(peel<npeels[ped]-1)
	    printf(" : %4d : ", cutspace[locus][ped][peel]);
          else printf("NULL : NULL : ");
	  p = &peelset[ped][peel];
          for(i=0; i < p->n; i++) 
            printf("%4d (%2d, %2d)  ",
		   peelspaces[locus][ped][peel][i],
                   marker[ped][p->ped[i]][locus][A],
                   marker[ped][p->ped[i]][locus][B]);
          printf(": %4d\n", peelspace[locus][ped][peel]);
          printf("\n");
        } 
      } 
    }
  }
}
