/* *************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      gethaplabel.c
   Author:    Mark E. Irwin
   Date:      Sept 30, 1992
   Purpose:   Haplotype labels for the spaces used in peeling in
	      peel.c
   Modified:  1/27/02 disease alleles go from 1-2 instead of 0-1
   Modified:  March 15, 2002.  Added some genotype elimination
              for founders.  Introduced nualleles, and ualleles
              to accomplish this.

************************************************************* */
#include <stdio.h>

#define A 0
#define B 1

#define F 0
#define M 1

#define FINDEX 2
#define MINDEX 3

void ComplexGetHaplabel(int npedigrees,
	         int *pedigreesize,
	         int ***ancestry,
	         int *nalleles,
	         int ****marker,
		 int nmarkers,
	         int isdisease,
	         int nloci,
	         int ***spacesize,
		 int ***nualleles,
		 int ****ualleles,
	         int ******haplabel)

{
  int i,j,k,l,m;
  int mA,mB,fmA,fmB,mmA,mmB,fromF,fromM;
  int findex,mindex;
  int index;

  int expans[2];
  int *****thaplabel;

  void basen();
   
  void nerror(); 

  thaplabel = (int *****) calloc(npedigrees,sizeof(int ****));
  for(i=0; i<npedigrees; i++) {
    thaplabel[i] = (int ****) calloc(pedigreesize[i],sizeof(int ***));
    for(j=0; j<pedigreesize[i]; j++) {
      thaplabel[i][j] = (int ***) calloc(nloci,sizeof(int **));
      for(k=0; k<nloci; k++) {
        thaplabel[i][j][k] = (int **) calloc(spacesize[i][j][k],sizeof(int *));
        for(l=0; l<spacesize[i][j][k]; l++)
          thaplabel[i][j][k][l] = (int *) calloc(2,sizeof(int));
      }
    }
  }

  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++) {
      for(k=0; k<nmarkers; k++) {
	mA = marker[i][j][k][A];
        mB = marker[i][j][k][B];
        if(spacesize[i][j][k]==1) {
          if( mA == mB && mA != 0 && mB != 0 ) {
	    thaplabel[i][j][k][0][F] = mA;
	    thaplabel[i][j][k][0][M] = mA;
          } 
          else {
            thaplabel[i][j][k][0][F] = ualleles[k][i][j][F];
            thaplabel[i][j][k][0][M] = ualleles[k][i][j][M];
	  }
        }
        else if(spacesize[i][j][k]==2) {
          thaplabel[i][j][k][0][F] = ualleles[k][i][j][A];
          thaplabel[i][j][k][0][M] = ualleles[k][i][j][B];
          thaplabel[i][j][k][1][F] = ualleles[k][i][j][B];
          thaplabel[i][j][k][1][M] = ualleles[k][i][j][A];
        }
        else if(spacesize[i][j][k]==2*nalleles[k]-1) {
          for(l=0; l<nalleles[k]; l++){
            thaplabel[i][j][k][l][F] = ualleles[k][i][j][A];
            thaplabel[i][j][k][l][M] = l+1;
          }
          for(l=0; l<nalleles[k]; l++){
	    if(l+1==ualleles[k][i][j][A]) l++;
	    if(l>=ualleles[k][i][j][A]) index=nalleles[k]+l-1;
	    else index=nalleles[k]+l;
	    if(l==nalleles[k]) break;
            thaplabel[i][j][k][index][F] = l+1;
            thaplabel[i][j][k][index][M] = ualleles[k][i][j][A];
          }
        }
        else if(spacesize[i][j][k]==nalleles[k]*nalleles[k]) {
          for(l=0; l<nalleles[k]; l++)
            for(m=0; m<nalleles[k]; m++) {
              thaplabel[i][j][k][l*nalleles[k]+m][F] = l+1;
              thaplabel[i][j][k][l*nalleles[k]+m][M] = m+1;
            }
        }
        else nerror("Error in Get Haplabel!");
      }
      for(k=nmarkers; k<nloci; k++)
        for(l=0; l<4; l++) {
          basen(2,2,l,expans);
          thaplabel[i][j][k][l][F]=expans[0]+1;
          thaplabel[i][j][k][l][M]=expans[1]+1;
        }
  }
  *haplabel = thaplabel;
}
/*
void DebugGetHaplabel(int npedigrees,
		      int *pedigreesize,
		      int nloci,
		      int ***spacesize,
		      int *****haplabel)

{
  char response[2];

  printf("\n\nDebug GetHaplabel()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) exit(0);
  if( strcmp(response,"y") == 0 ) {

    int i,j,k,l;

    printf("\n");
    for(k=0; k<nloci; k++) {
      printf("Haplotype Labels for Loci %d \n\n",k+1);
      for(i=0; i<npedigrees; i++) {
	printf("Pedigree #%d\n\n",i+1);
	printf("Person ID Element FromF FromM\n");
	printf("--------- ------- ----- -----\n");
	for(j=0; j<pedigreesize[i]; j++) {
	  for(l=0; l<spacesize[i][j][k]; l++)
	    printf("%8d %7d %5d %5d\n",j+1,l,haplabel[i][j][k][l][F],
						haplabel[i][j][k][l][M]);
	  printf("\n");
        }
      }
    }
  }
}
*/
