/***************************************************************

  Program: SIMPLE 
  File:    ComplexPeel.c
  Author:  Zachary Skrivanek 
  Date:    January 26, 2002 
  Purpose: Peel complex pedigree.

***************************************************************/

#include "simple.h" /* c */
#include <stdio.h>

#define F 0 
#define M 1 
#define CHILD  0
#define MOM 1 
#define DAD 2 

count   ped, peel, locus, i, j, k;
sindex  *p, *q;


void  ComplexPeel(int      npedigrees,
		  int      **ntrans,
		  int      **nrinto,
		  tindex   ***tidx,
		  Rindex   ***ridx,
	          int      *****haplabel,
		  int      ****ibd,
		  int      *ibdF,
		  int      *ibdM,
		  double   *prob,
	          int      nloci,
		  int      locus,
	          int      *npeels,
	          sindex   **cutset,
	          int      **cutspace,
	          int      ***cutspaces,
	          sindex   **peelset,
	          int      **peelspace,
	          int      ***peelspaces,
	          int      *whaps,
	          int      *whaps2,
	          double   ***wpp,
		  double   ***rfunction){


  void    multi_base();
  double  trans(), rprob();
  double  jprob;
  Rindex  *r;

  for(ped=0; ped<npedigrees; ped++)
    for(peel=0; peel<npeels[ped]; peel++)
      for(i=0; i < cutspace[ped][peel]; i++){
        multi_base(cutset[ped][peel].n, &cutspaces[ped][peel][0],i,whaps);
      /* initialize to 0 */
	rfunction[ped][peel][i] = 0.0; 
	for(j=0; j < peelspace[ped][peel]; j++){
	  multi_base(peelset[ped][peel].n, &peelspaces[ped][peel][0],j,&whaps[cutset[ped][peel].n]);
	  jprob = 1.0;
	  for(k=0; k < ntrans[ped][peel]; k++)
	    jprob *= trans(&tidx[ped][peel][k],nloci,locus,ped, haplabel, whaps,prob,ibd,ibdF,ibdM);
	  for(k=0; k < nrinto[ped][peel]; k++){
	    r = &ridx[ped][peel][k]; 
	    jprob *= rprob(whaps, whaps2, r->set, cutset[ped][r->r].n, &cutspaces[ped][r->r][0], &rfunction[ped][r->r][0]);
          }
          jprob *= wpp[ped][peel][j];
	  rfunction[ped][peel][i] += jprob;
        }
      }
}


/* using tidx we extract the haplotypes for */
/* the child and parents and calculate the */
/* transition probability */

double  trans(tindex  *tidx,
	      int     nloci,
              int     locus,
              int     ped,
	      int     *****haplabel,
	      int     *whaps,
	      int     model,
	      double  **prob,
	      int     ****ibd,
	      int     *ibdF,
	      int     *ibdM)
{
  void   Getibd();

  int    idx, child[2], mom[2], dad[2];
  double cpF, cpM;

  double condprob();

  idx = whaps[tidx -> set[CHILD]];
  child[F] = haplabel[ped][tidx -> ped[CHILD]][locus][idx][F];
  child[M] = haplabel[ped][tidx -> ped[CHILD]][locus][idx][M];

  idx = whaps[tidx -> set[MOM]];
  mom[F] = haplabel[ped][tidx -> ped[MOM]][locus][idx][F];
  mom[M] = haplabel[ped][tidx -> ped[MOM]][locus][idx][M];

  idx = whaps[tidx -> set[DAD]];
  dad[F] = haplabel[ped][tidx -> ped[DAD]][locus][idx][F];
  dad[M] = haplabel[ped][tidx -> ped[DAD]][locus][idx][M];

  Getibd(ped,tidx -> ped[CHILD],F,nloci,ibd,ibdF);
  Getibd(ped,tidx -> ped[CHILD],M,nloci,ibd,ibdM);

  cpF = condprob(child[F],dad[F],dad[M],nloci,locus,ibdF,prob);
  cpM = condprob(child[M],mom[F],mom[M],nloci,locus,ibdM,prob);
  return( cpF * cpM);
}


/* using the 'set' index, first we extract */
/* the relevant haps from whaps and put them */
/* in whaps2, then we use whaps2 to form the */
/* appropriate index to elicit the correct */
/* rfunction */

double  rprob(int    *whaps,
	      int    *whaps2,
	      int    *set,
	      int    n,
	      int    *cutspaces,
	      double *rfunction)
{
  double  cpF, cpM;
  int     idx;

  int     unmulti_base();
  void    subset();

  subset(whaps, n, set, whaps2);

  idx = unmulti_base(n, cutspaces, whaps2);
  return(rfunction[idx]);
}

void  DebugComplexPeel(int     npedigrees,
		       int     *npeels, 	
		       int     **cutspace,
		       double  ***rfunction)
{
  char response[2];

  printf("\n\nDebug ComplexPeel()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0)
  {
    printf("\n\n");
    for(ped=0; ped < npedigrees; ped++){
      printf("ped %2d\n",ped+1);
      printf("******\n");
      printf("Peel: rfunction\n");
      printf("***************\n");
      for(peel=0; peel < npeels[ped]; peel++)
        for(i=0; i < cutspace[ped][peel]; i++)
          printf("%2d: %le\n",peel+1,rfunction[ped][peel][i]);
    }
  }
}


