/***************************************************************

  Program: SIMPLE 
  File:    ComplexSimulate.c
  Author:  Zachary Skrivanek 
  Date:    January 30, 2002 
  Purpose: Simulate complex pedigree.

***************************************************************/

#include "simple.h" /* c */

#define F 0 
#define M 1 


void  ComplexSimulate(int      npedigrees,
		      int      **ntrans,
		      int      **nrinto,
		      tindex   ***tidx,
		      Rindex   ***ridx,
	              int      *****haplabel,
		      int      ****ibd,
		      int      *ibdF,
		      int      *ibdM,
		      double   *prob,
	              int      nloci,
		      int      locus,
	              int      *npeels,
	              sindex   **cutset,
	              int      **cutspace,
	              int      ***cutspaces,
	              sindex   **peelset,
	              int      **peelspace,
	              int      ***peelspaces,
	              int      *whaps,
	              int      *whaps2,
	              double   ***wpp,
		      int      **shap,
		      int      ****simhap,
		      double   *simprob,
		      double   ***rfunction){


  double  marg;
  double  rand, drand;
  count   ped, peel, i, j, k;
  sindex  *p, *c;
  Rindex  *r;

  void    multi_base(), subset();
  double  trans(), rprob(), runif();

  for(ped=0; ped<npedigrees; ped++)
    for(peel=npeels[ped]-1; peel >= 0; peel--){
      c = &cutset[ped][peel];
      subset(&shap[ped][0],c->n,c->ped,whaps);
      marg = 0.0;
    /* derive joint and marginal prob */
    /* simprob contains the joint and */
    /* marg contains the marginal */
      for(i=0; i < peelspace[ped][peel]; i++){
        multi_base(peelset[ped][peel].n, &peelspaces[ped][peel][0], i, &whaps[c->n]);
	simprob[i] = 1.0;
	for(k=0; k < ntrans[ped][peel]; k++)
	  simprob[i] *= trans(&tidx[ped][peel][k],nloci,locus,ped, haplabel, whaps,prob,ibd,ibdF,ibdM);
	for(k=0; k < nrinto[ped][peel]; k++){
	  r = &ridx[ped][peel][k]; 
	  simprob[i] *= rprob(whaps, whaps2, r->set, cutset[ped][r->r].n, &cutspaces[ped][r->r][0], &rfunction[ped][r->r][0]);
        }
        simprob[i] *= wpp[ped][peel][i];
	marg += simprob[i];
      }
      rand = runif();
      drand = rand*marg;
    /* derive cumulative joint distribution */
      for(i=1; i < peelspace[ped][peel]; i++)
        simprob[i] += simprob[i-1];
    /* compare rand to cumulative conditional distribution */
    /* which is the same thing as comparing drand=rand*marg */ 
    /* to cumlative joint distribution */
      for(i=0; i < peelspace[ped][peel]; i++)
	if(drand < simprob[i]) break;
    /* update haplotypes */
      p = &peelset[ped][peel];
      multi_base(p->n, &peelspaces[ped][peel][0], i, whaps);
      for(i=0; i < p->n; i++){
	shap[ped][p->ped[i]] = whaps[i];
	simhap[ped][p->ped[i]][locus][F]=haplabel[ped][p->ped[i]][locus][whaps[i]][F];
	simhap[ped][p->ped[i]][locus][M]=haplabel[ped][p->ped[i]][locus][whaps[i]][M];
      }
    }
}

