/* ********************************************************************

    Program:   scan 
    File:      CreateDistances.noLR.c 
    Author:    Zachary Skrivanek
    Date:      May 23, 2000
    Purpose:   Creates array of distances for NPL.  This is very dif-
	       ferent from SIMPLE createdistances in that it simulates
	       at every point of interest vs using the reweighting
	       scheme.

********************************************************************* */

#include <stdlib.h>
#include <stdlib.h> /* exit */

void  CreateDistances_noLR(int     nmarkers, 
		           int     *nmodels,
                           int     nsteps,
		           double  *mdistances,
		           int     ***locusorder,
		           double  ***distances,
		           int     **numlik,
		           double  ****likdistances)

{

  int    i,j,k,l,model,step;
  int    nloci;
  int    ninter;
  int    **tlocusorder;
  int    *tnumlik;
  int    tnmodels;
  double cdist;
  double **tdistances;
  double ***tlikdistances;

  nloci = nmarkers + 1;
  ninter = nsteps-1;
  nsteps -= 1;  /* to conform to GH */
  tnmodels = (ninter+1)*(nmarkers-1)+2;

  /*******************
  ** allocate space **
  *******************/
  tlocusorder   = (int **) calloc(tnmodels,sizeof(int *));
  tnumlik       = (int *) calloc(tnmodels,sizeof(int));       
  tdistances    = (double **) calloc(tnmodels,sizeof(double *));
  tlikdistances = (double ***) calloc(tnmodels,sizeof(double **));


  for(i=0; i < tnmodels; i++)
  {
    /* we give the null case xtra space incase we want to
    convert to rec fractions and use mark's algorithm to 
    calculate ibd probs in which case we tack on a rec. frac
    of .5 for the null disease case (independent) */
    tlocusorder[i]   = (int *) calloc(nloci,sizeof(int));
    tdistances[i]    = (double *) calloc(nloci-1,sizeof(double));
    tnumlik[i] = 1;
    tlikdistances[i] = (double **) calloc(tnumlik[i],sizeof(double *));
    for(j=0; j < tnumlik[i]; j++)
      tlikdistances[i][j] = (double *) calloc(nloci-1,sizeof(double));
  }
  
  /***********************
  ** create tlocusorder ** 
  ***********************/

  for(i=0; i<nloci; i++)
    tlocusorder[0][i]=i+1;

  for(i=1; i <= nloci-2; i++)
    tlocusorder[tnmodels-1][i-1]=i;
  tlocusorder[tnmodels-1][nloci-2]=nloci;
  tlocusorder[tnmodels-1][nloci-1]=nloci-1;

  for(i=0; i<nmarkers-1; i++)  
    for(j=1; j<=ninter+1; j++){
      model = i*(ninter + 1) + j;
      for(k=1; k<=nloci;){
	if((k-1)>i)
	  tlocusorder[model][k-1]=k-1;
        else if((k-1)<i)
	  tlocusorder[model][k-1]=k;
        else if((k-1)==i){
	  tlocusorder[model][k-1]=k;
	  tlocusorder[model][k]=nloci;
	  k++;
        }
        k++;
      }
    }


  /*********************
  ** create distances **
  *********************/

  tdistances[1][0]=0.0;
  tlikdistances[1][0][0]=0.0;
  for(i=0; i<nmarkers-1; i++){
    tdistances[0][i]=mdistances[i];
    tlikdistances[0][0][i]=mdistances[i];
    tdistances[1][i+1]=mdistances[i];
    tlikdistances[1][0][i+1]=mdistances[i];
  }

  for(i=0; i<nmarkers-1; i++)
    for(step=1; step<=ninter+1; step++){
      model = i*(ninter + 1) + step+1;
      cdist = ((double) step )/((double) ninter+1)*mdistances[i];
      for(k=0; k < nloci-1; k++){
        if( k > i){
	  tdistances[model][k]=mdistances[k-1];
	  tlikdistances[model][0][k]=mdistances[k-1];
        }
        else if(k < i){
	  tdistances[model][k]=mdistances[k];
	  tlikdistances[model][0][k]=mdistances[k];
        }
        else if(k==i){
	  tdistances[model][k]=cdist;
	  tlikdistances[model][0][k]=cdist;
	  tdistances[model][k+1]=mdistances[i]-cdist; 
	  tlikdistances[model][0][k+1]=mdistances[i]-cdist; 
	  k++;
        }
      }
    }

  *locusorder   = tlocusorder;
  *distances    = tdistances;
  *numlik       = tnumlik;
  *likdistances = tlikdistances;
  *nmodels      = tnmodels;
}

void DebugCreateDistances_noLR(int     nmarkers,
                               int     nmodels,
                               int     **locusorder,
                               double  **distances,
                               int     *numlik,
                               double  ***likdistances)
{
  char response[2],c;
  int  nloci;

  nloci = nmarkers+1;

  printf("\n\nDebug CreateDistances()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) 
  {

    int    i,j,k;

    for(i=0; i<nmodels; i++) {
      for(j=0; j<nloci; j++)
        printf("%d ",locusorder[i][j]);
      printf("\n");
      for(j=0; j<nloci-1; j++)
        printf("%le ",distances[i][j]);
      printf("\n");
      for(j=0; j<numlik[i]; j++){
        for(k=0; k<nloci-1; k++)
          printf("%le ",likdistances[i][j][k]);
        printf("\n");
      }
    }
  }
}

