/* ********************************************************************

   Program:   simple (Sequential Imputation for Multi-Point Linkage
              Estimation)
   File:      DumpCommands.c 
   Author:    Zachary Skrivanek
   Date:      September 2, 2000
   Purpose:   Dump commands used in SIMPLE 
   Modified:  1/25/02- added npedigrees so I could process multiple
	      peel files

********************************************************************* */
#include "simple.h"  /* aff, cmd */
#include <stdio.h>  /* fscanf */

#define TRUE  1 
#define FALSE 0
#define bool  int

void  DumpCommands(cmd   c,
		   char  **argv,
		   int   argc,
		   int   npedigrees,
		   int   nloci)
{

  /* FILE  *gfopen(), *cmdfile; */
  int   i;

  /* cmdfile = gfopen("commands","w"); */

  printf("The following commands were used.\n");
  if(argc>1){
    printf("An '*' indicates that the command was found in the\n");
    printf("command file '%s' and may not necessarily be the\n",argv[1]);
    printf("default value.\n");
  }
  printf("-------------------------------------------------\n");
  printf("load %s",c.load.par);
  if(!c.load.def) printf("*");
  printf("\n");

  printf("scan %s",c.scan.par);
  if(!c.scan.def) printf("*");
  printf("\n");

  printf("analysis %s",c.analysis.par);
  if(!c.analysis.def) printf("*");
  printf("\n");

  if(c.peel.par){
    printf("peel");
    if(!c.peel.def) printf("*");
    for(i=0; i<npedigrees; i++)
      printf(" %s",c.peel.fname[i]);
    printf("\n");
  }

  printf("score %s",c.score.par);
  if(!c.score.def) printf("*");
  printf("\n");

  printf("units");
  if(!c.units.def) printf("*");
  else printf(" (AUTO) "); 
  printf("%s",c.units.par);
  printf("\n");

  printf("maxiter %d",c.maxiter.par);
  if(!c.maxiter.def) printf("*");
  printf("\n");

  printf("increment step %d",c.nsteps.par);
  if(!c.nsteps.def) printf("*");
  printf("\n");

  printf("seed %d",c.seed.par);
  if(!c.seed.def) printf("*");
  printf("\n");

/* the default is FALSE */
  if(!c.debug.def) {
    printf("debug");
    printf("*");
    printf("\n");
  }

  if(!c.dumpibd.def) {
    printf("dumpibd");
    printf("*");
    printf("\n");
  }

  if(!c.dumphap.def) {
    printf("dumphap");
    printf("*");
    printf("\n");
  }

  if(!c.dumplikrats.def) {
    printf("dumplikrats");
    printf("*");
    printf("\n");
  }

  if(c.reweight.def) 
    printf("NO reweighting used");
  else{
    printf("reweighting used");
    printf("*");
  }
  printf("\n");

  if(!c.wstar.def) {
    printf("wstar");
    printf("*");
    printf("\n");
  }

  if(!c.dumpdata.def) {
    printf("dumpdata");
    printf("*\n");
  }

/* the default is FALSE */
  if(!c.hasemanelston.def) {
    printf("haseman elston");
    printf("*");
    printf("\n");
  }

  if(!c.whichQT.def){
    printf("%d",c.whichQT.par);
    printf("*");
    printf("\n");
  }

  if(!c.missingQT.def){
    printf("missing QT %lf",c.missingQT.par);
    printf("*");
    printf("\n");
  }

  printf("process order ");
  if(c.debug.def) printf(" (AUTO) ");
  else printf("*");
  for(i=0; i<nloci; i++)
    printf("%d ",c.processorder.par[i]);
  printf("\n");

  printf("interference %d",c.interference.model);
  if(!c.interference.def) printf("*");
  if(c.interference.model==19) 
    printf(" m=%d",c.interference.par);
  printf("\n");

  if(!c.file.def){
    printf("file %s",c.file.par);
    printf("*\n");
    printf(" nmodels = %d",c.file.nmodels);
    printf("\n");
  } 

/* the default is FALSE */
  if(!c.sim.def) {
     printf("sim");
     printf("*");
     printf("\n");
  }

/* the default is FALSE */
  if(!c.total_stat.def) {
     printf("total stat");
     printf("*");
     printf("\n");
  }

  if(c.joint.par) {
    printf("joint");
    printf("* (simulate joint IVs)\n");
  }

  /* fclose(cmdfile); */
}
