/***************************************************************

  Program: scan 
  File:    FindPairs.c
  Author:  Zachary Skrivanek 
  Date:    February 1, 2001 
  Purpose: Find pairs of different type (sibs, avuncular, 
	   etc ...).  As of this moment the only type of pairs 
	   that are extracted are the FIRST pair of sibs from 
	   each sibship (just like GH) and all possible related
	   pairs (if dumppairs).  The sibs pairs always gets
	   put in the first index and the all pairs gets put in
	   the second index if sibs is also used ow get put in
	   second index.

***************************************************************/

#include <math.h> /* pow */
#include <stdlib.h> /* pow, free */
#include "simple.h"

#define TRUE 1
#define FALSE 0
#define sib1 0
#define sib2 1
#define PID 0
#define FID 1
#define MID 2
#define FOUNDER 0
#define FINDEX  2
#define MINDEX  3


int    ped, pair; /* index variables */

void  FindPairs(int    npedigrees,
		int    *pedigreesize,	
		int    ***pedigree,
		int    ***ancestry,
		prs    ****pairs,
		int    ***npairs,
		double ***qpheno,
		int    whichQT,
		double missingQT,
		int    type){

  bool   *peeled; /* peeled nuclear family by peeling parents */
  int    i,j; /* index variables */
  int    MAX_PAIRS;
  int    ntypes;

  prs    ***tpairs;  
  int    **tnpairs;

  void   nerror();

  if(type==3) ntypes=2;
  else ntypes=1;

  tnpairs = (int **) calloc(ntypes, sizeof(int *));
  tpairs = (prs ***) calloc(ntypes, sizeof(prs **));
  for(i=0; i<ntypes; i++){  
    tnpairs[i] = (int *) calloc(npedigrees, sizeof(int));
    tpairs[i] = (prs **) calloc(npedigrees, sizeof(prs *)); 
    for(ped=0; ped < npedigrees; ped++){
      MAX_PAIRS = pedigreesize[ped]*(pedigreesize[ped]+1)/2;
      tpairs[i][ped] = (prs *) calloc(MAX_PAIRS, sizeof(prs)); 
    }
  }

    for(ped=0; ped < npedigrees; ped++){
      peeled = (bool *) calloc(pedigreesize[ped], sizeof(bool));
      for(i=0; i<pedigreesize[ped]; i++)
	peeled = FALSE;
      for(i=0; i<pedigreesize[ped]; i++){
        if(type==1 || type==3){
	  /* get first pair of sibs from every sibship  */
	  /* first get a child whose sibs have not been */
	  /* used (so both parents are not peeled) and  */
	  /* who is not missing the required pheno0  */
	    if(!ancestry[ped][i][FOUNDER] &&
	       (!peeled[ancestry[ped][i][FINDEX]] &&
	       !peeled[ancestry[ped][i][MINDEX]]) &&
	       qpheno[ped][i][whichQT] != missingQT){
            /* mark the nuclear family as peeled */ 
	      peeled[ancestry[ped][i][FINDEX]]=TRUE; 
	      peeled[ancestry[ped][i][MINDEX]]=TRUE; 
              for(j=0; j<pedigreesize[ped]; j++){
	      /* find sib */
		if(pedigree[ped][j][MID]==pedigree[ped][i][MID] &&
		   pedigree[ped][j][FID]==pedigree[ped][i][FID] &&
		   i!=j &&
	           qpheno[ped][j][whichQT] != missingQT){
                  tpairs[0][ped][tnpairs[0][ped]].pid[sib1]=pedigree[ped][i][PID]; 
                  tpairs[0][ped][tnpairs[0][ped]].pid[sib2]=pedigree[ped][j][PID]; 
                  tpairs[0][ped][tnpairs[0][ped]].index[sib1]=i;
                  tpairs[0][ped][tnpairs[0][ped]].index[sib2]=j;
                  tpairs[0][ped][tnpairs[0][ped]].pheno=
		    pow(qpheno[ped][i][whichQT-1]-qpheno[ped][j][whichQT-1],2.0);
                  ++tnpairs[0][ped];
		  break; 
                } /* if sib2 */
              } /* j pedigreesize */
            } /* if sib1 */ 
          /* break; */
        }  /* if case==1 || case==3 */
        else if(type==2 || type==3){
	/* get all pairs which are not both founder */
          for(j=i+1; j<pedigreesize[ped]; j++)
	    if(!ancestry[ped][i][FOUNDER] ||
	       !ancestry[ped][j][FOUNDER]) {
              tpairs[ntypes-1][ped][tnpairs[ntypes-1][ped]].pid[sib1]=pedigree[ped][i][PID]; 
              tpairs[ntypes-1][ped][tnpairs[ntypes-1][ped]].pid[sib2]=pedigree[ped][j][PID]; 
              tpairs[ntypes-1][ped][tnpairs[ntypes-1][ped]].index[sib1]=i;
              tpairs[ntypes-1][ped][tnpairs[ntypes-1][ped]].index[sib2]=j;
              ++tnpairs[ntypes-1][ped];
            }
	}
	else nerror("Invalid type!");
      } /* i pedigreesize */
    }/* ped */ 
    free(peeled);
  /* }type */ 
  *npairs = tnpairs;
  *pairs = tpairs;
}

void  DebugFindPairs(int    npedigrees,
                     int    **npairs,
		     prs    ***pairs,
		     int    type){

  char response[2];

  printf("\n\nDebug FindPairs()? => ");
  scanf("%s", response);
  if(strcmp(response, "n") == 0) return;
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0) {

  int ntypes,i;
  char   *labels[] ={"sib"};

  if(type==3) ntypes=2;
  else ntypes=1;


    for(ped=0; ped<npedigrees; ped++){
      printf("\n\npedigree %d", ped+1);
      printf("\n\npair mem1 index mem2 index pheno    type\n"); 
      for(i=0; i<ntypes; i++)
        for(pair=0; pair<npairs[i][ped]; pair++)
          printf("%4d %4d %5d %4d %5d %8.6lf %4s\n",
		 pair+1, pairs[i][ped][pair].pid[sib1],
                 pairs[i][ped][pair].index[sib1],
                 pairs[i][ped][pair].pid[sib2],
                 pairs[i][ped][pair].index[sib2],
                 pairs[i][ped][pair].pheno,
		 labels[i]);
    }
  }
}

	         
