/* ********************************************************************

   Program:  NPL (Nonparametric Linkage) 
   File:     GetIBDStates.c
   Author:   Zachary Skrivanek
   Date:     March 23, 2000
   Purpose:  Creates the IBD states. 

********************************************************************* */
#include <stdio.h> /* fscanf(input,) printf() */
#include <stdlib.h> /* exit() */

#define TRUE  1
#define FALSE 0

#define PID 0
#define FID 1
#define MID 2

#define FOUNDER 0
#define PARENT  1
#define FINDEX  2
#define MINDEX  3
#define SID     4
#define SINDEX  5
#define SEX     6

#define MALE   1
#define FEMALE 0

#define F 0
#define M 1


void  GetIBDStates(int     ***states,
		   int     ***disIV, 
	           int     ***ancestry, 
	           int     npedigrees,
	           int     *pedigreesize,
		   int     ***peeled)
	           
{
  int    newstate=0;  /* give founder a unique state */
  int    i,j,k,l,z=0;
  int    parent;
  int    allpeeled;
  int    mf,mm;
  int    check;

  int    INDEX;
  int    IV;

  int    power(int, int);
  void   basen(int,int,int,int *);


  /*  We use disIV to determine the IBD states for the nonfounders. */
  for(i=0; i<npedigrees; i++)  {
    newstate=0;
    for(j=0; j<pedigreesize[i]; j++){
      if(ancestry[i][j][FOUNDER]){ /* find children of founders */
        if(ancestry[i][j][SEX]==MALE)
          INDEX=FINDEX;
        else 
          INDEX=MINDEX; 
        states[i][j][F] = ++newstate;
        states[i][j][M] = ++newstate;
        mf=states[i][j][F];	
        mm=states[i][j][M]; 
        for(parent=0; parent<2; parent++) 
          peeled[i][j][parent]=TRUE; /* founders stay peeled */
        for(k=0; k<pedigreesize[i]; k++)
          if(!ancestry[i][k][FOUNDER])
            if(ancestry[i][k][INDEX]==j){ /* found child */
              IV=disIV[i][k][INDEX-2];
	      states[i][k][INDEX-2]=mf*(1-IV)+mm*IV;
	      peeled[i][k][INDEX-2]=TRUE; 
            }	
      } /* if FOUNDER */ 
    } /* j pedigreesize loop */ 

    /* peel the rest */ 
    allpeeled=FALSE;
    while(allpeeled==FALSE){ 
      for(j=0; j<pedigreesize[i]; j++){
        if(!peeled[i][j][F]){
          INDEX=FINDEX;
          IV=disIV[i][j][F];
        }
        else if(!peeled[i][j][M]){
          INDEX=MINDEX;
          IV=disIV[i][j][M];
        } 
        else continue;
        for(k=0; k<pedigreesize[i]; k++)
          if(k==ancestry[i][j][INDEX] && peeled[i][k][IV]){ 
            mf=states[i][k][F];	
            mm=states[i][k][M]; 
            states[i][j][INDEX-2]= mf*(1-IV)+mm*IV; 
	    peeled[i][j][INDEX-2]=TRUE;
            break;
          }
      } /* j pedsize loop */
          
      /* check to see if all are peeled */ 
      check=0;
      for(j=0; j<pedigreesize[i]; j++)
        for(parent=0; parent<2; parent++)
	  check += peeled[i][j][parent];
      if(check==2*pedigreesize[i]) allpeeled=TRUE;
    } /* while all peeled loop */
  } /* i npedigrees loop */

}


void  DebugGetIBDStates(int     ***states,
                        int     ***pedigree,
                        int     ***ancestry,
                        int     *pedigreesize,
                        int     npedigrees)

{
  char response[2];

  printf("\n\nDebug GetIBDStates()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0){


    int    i,j,k,parent;
    int    INDEX;
    int    *peeled;

    int    power(int, int);

    /* print founders' states first */
    for(i=0; i<npedigrees; i++){
      printf("pedigree %d\n\n",i+1);
      peeled = (int *) calloc(pedigreesize[i], sizeof(int));
      for(j=0; j<pedigreesize[i]; j++){
        if(ancestry[i][j][FOUNDER]){
          printf("PID %d:  ",pedigree[i][j][PID]);
          for(parent=0; parent<2; parent++)
            printf("%d ",states[i][j][parent]);
          printf("\n");
          peeled[j]=TRUE;
        } 
      } 

      for(j=0; j<pedigreesize[i]; j++)
        if(ancestry[i][j][FOUNDER]){ /* find children of founders */
          if(ancestry[i][j][SEX]==MALE) INDEX=FINDEX;
          else INDEX=MINDEX;
          for(k=0; k<pedigreesize[i]; k++)
            if(ancestry[i][k][INDEX]==j && !peeled[k]){/* found child */
              printf("PID %d:  ",pedigree[i][k][PID]);
              for(parent=0; parent<2; parent++)
                printf("%d ",states[i][k][parent]);
              printf("\n");
              peeled[k]=TRUE;
            }
        }

      for(j=0; j<pedigreesize[i]; j++)
        if(!peeled[j]){
          printf("PID %d:  ",pedigree[i][j][PID]);
          for(parent=0; parent<2; parent++)
            printf("%d ",states[i][j][parent]);
          printf("\n");
        }
      printf("\n");
      free(peeled);
    } /* i npedigrees loop */
  } /* if */
} /* DebugGetIBDStates */
