/* *************************************************************

   Program:   simple
   File:      GetRIndex.c
   Author:    Zachary Skrivanek
   Date:      February 7, 2002 
   Purpose:   Get index used for aggregating R functions.  For 
	      every R function we find R* functions who have 
	      cutset members that are in R function.  If none
	      of the cutset members of the R* function were
	      peeled (ie this R* function was already ag-
	      greagated) then we add R* to R and we get the cor-
	      responding indices to do the peeling.

************************************************************* */

#include <stdio.h>
#include "simple.h"

#define FOUNDER 0 
#define FINDEX  2
#define MINDEX  3

#define TRUE  1
#define FALSE 0

#define CHILD  0
#define MOM 1 
#define DAD 2 

#define Not_Found -1

count   ped, peel, i, j;
sindex  *c, *p;

void GetRIndex(int     npedigrees,
	       int     *npeels,
	       sindex  **cutset,
	       sindex  **peelset,
	       int     ***nrinto,
	       Rindex  ****ridx)	
{

  int     **tnrinto;
  Rindex  ***tridx;

  count   peel2, peel3, k, l, m;
  bool    USED;
  int     n;
  sindex  *c2, *p2;

  tnrinto = (int **) calloc(npedigrees, sizeof(int *));
  tridx = (Rindex ***) calloc(npedigrees, sizeof(Rindex **));

  for(ped=0; ped < npedigrees; ped++) {
    tnrinto[ped] = (int *) calloc(npeels[ped], sizeof(int));
    tridx[ped] = (Rindex **) calloc(npeels[ped], sizeof(Rindex *));
    
    for(peel=npeels[ped]-1; peel > 0; peel--){
      p = &peelset[ped][peel];
      c = &cutset[ped][peel];
    /* the upper bound for the number of rfunctions is */
    /* the number of peels done prior to this peel */
      tridx[ped][peel] = (Rindex *) calloc(peel, sizeof(Rindex));
      for(i=0; i < p->n; i++)
        for(peel2=peel-1; peel2 >= 0; peel2--){
          c2 = &cutset[ped][peel2];
	  for(j=0; j < c2->n; j++)
	  /* found peel member in cutset */
	    if( c2->pid[j] == p->pid[i] ) {

	    /**************************************************
	    ** make sure this rfunction was not already used **
	    **************************************************/
 	      USED=FALSE;		
            /* make sure that multiple members in peel */
	    /* are not in the cutset of peel2 */
	      for(k=0; k<tnrinto[ped][peel]; k++)
	        if(tridx[ped][peel][k].r == peel2) USED=TRUE;
            /* make sure that this R function was not already */
	    /* added to a consequent R function */
	      for(k=0; k < c2->n; k++)
	        for(peel3=peel2+1; peel3 < peel; peel3++){
		  p2 = &peelset[ped][peel3];
	          for(m=0; m < p2->n; m++)
		    if(c2->pid[k] == p2->pid[m])
		      USED=TRUE;
                }
              if(USED) break;  
 
	      n=tnrinto[ped][peel];
	      tridx[ped][peel][n].set = 
		(int *) calloc(c2->n, sizeof(int));
	      tridx[ped][peel][n].pid= 
		(int *) calloc(c2->n, sizeof(int));

              tridx[ped][peel][n].r = peel2;
              tridx[ped][peel][n].n = c2->n;
            /* make sure that every member in c2 is found in the */
	    /* union of the peel and cut sets of the current R- */
	    /* function.  If so then add them else put out an */
	    /* error message */
	      for(k=0; k < c2->n; k++){
		USED=FALSE;
		for(l=0; l < c->n; l++)
		  if(c->pid[l]==c2->pid[k]){
		    USED=TRUE;
		    tridx[ped][peel][n].set[k] = l;
		    tridx[ped][peel][n].pid[k] = c->pid[l];
		    break;
                  }
		for(l=0; l < p->n; l++)
		  if(p->pid[l]==c2->pid[k]){
		    USED=TRUE;
		    tridx[ped][peel][n].set[k] = c->n + l;
		    tridx[ped][peel][n].pid[k] = p->pid[l];
		    break;
                  }
                if(!USED) nerror("Peel not valid!");   
              
              }
	      tnrinto[ped][peel]++;
	      break; /* peel2 */
            } /* if found someone */
        } /* peel2 */                       
    } /* peel */
  } /* ped */

  *nrinto= tnrinto;
  *ridx = tridx;
} /* program */
		    
        
void DebugGetRIndex(int     npedigrees,
		    int     *npeels,
		    int     **nrinto,
		    Rindex  ***ridx){

  char response[2];

  printf("\n\nDebug GetRIndex()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0) {

    Rindex  *r;
    printf("\n\n");

    printf("peel: R:  set index (pid)\n");
    printf("***** ***************\n");
    for(ped=0; ped < npedigrees; ped++) {
      printf("  xx ped %2d xx\n",ped+1);
      for(peel=0; peel < npeels[ped]; peel++) 
        for(i=0; i < nrinto[ped][peel]; i++) {
	  r = &ridx[ped][peel][i];
	  printf("%3d: ",peel+1);
	  printf("%3d: ",r->r+1);
	  for(j=0; j < r->n; j++)
	    printf("%3d(%d) ", r->set[j], r->pid[j]);
          printf("\n");
        }
     }
  }
}
