/* *************************************************************

   Program:   simple
   File:      GetTransIndex.c
   Author:    Zachary Skrivanek
   Date:      February 7, 2002 
   Purpose:   Get index used for transition probabilities

************************************************************* */

#include <stdio.h>
#include "simple.h"

#define FOUNDER 0 
#define FINDEX  2
#define MINDEX  3


#define CHILD  0
#define MOM 1 
#define DAD 2 

#define Not_Found -1

count   ped, peel, i, j;
sindex  *c, *p;

void GetTransIndex(int     npedigrees,
		   int     *npeels,
		   sindex  **cutset,
		   sindex  **peelset,
		   int     ***ancestry,
		   int     ***ntrans,
		   tindex  ****tidx)	
{

  int     **tntrans;
  tindex  ***ttidx;
  int     maxspace;
  int     set[3];
  int     child, mom, dad;


  tntrans = (int **) calloc(npedigrees, sizeof(int *));
  ttidx = (tindex ***) calloc(npedigrees, sizeof(tindex **));

  for(ped=0; ped < npedigrees; ped++) {
    tntrans[ped] = (int *) calloc(npeels[ped], sizeof(int));
    ttidx[ped] = (tindex **) calloc(npeels[ped], sizeof(tindex *));
    
    for(peel=0; peel < npeels[ped]; peel++) {
      c = &cutset[ped][peel];
      p = &peelset[ped][peel];
    /* the maximum number of transitions is the size of the union */
    /* of the cutset and peelset */
      maxspace = c->n + p->n;
      ttidx[ped][peel] = (tindex *) calloc(maxspace, sizeof(tindex));

    /******************************
    ** look for child in cut set **
    ******************************/
    /* for every child in cutset find if parents are in cut/peel sets */
      for(i=0; i < c->n; i++){
      /* found child */
	if( !ancestry[ped][c->ped[i]][FOUNDER] ) {
	  child = c->ped[i];
	  set[CHILD] = i;
	  mom = Not_Found;
	  dad = Not_Found;
        /* look for parents in cutset */
          for(j=0; j < c->n; j++)
	    if( c->ped[j] == ancestry[ped][child][FINDEX] ){
	      dad = c->ped[j];  /* get index within pedigree */
	      set[DAD] = j;  /* get index within union of cut and peel sets */
            }
	    else if( c->ped[j] == ancestry[ped][child][MINDEX] ){
	      mom = c->ped[j];  /* get index within pedigree */
	      set[MOM] = j;  /* get index within union of cut and peel sets */
            }
        /* if at least one of the parents are not in cut set then look */
	/* in peel set, ow abort since if all are in cut set we do not */
	/* use the transition probability */
          if(dad == Not_Found || mom == Not_Found) 
            for(j=0; j < p->n; j++)
	      if( p->ped[j] == ancestry[ped][child][FINDEX] ){
	        dad = p->ped[j]; /* get index within pedigree */
	        set[DAD] = c->n + j; /* get index within union of cut and peel sets */
              }
	      else if( p->ped[j] == ancestry[ped][child][MINDEX] ){
	        mom = p->ped[j]; /* get index within pedigree */
	        set[MOM] = c->n + j; /* get index within union of cut and peel sets */
              }
	  else break;
        /* if all are found and at least one parent is being peeled then we have a transition */
	  if( child != Not_Found && dad != Not_Found && mom != Not_Found ){
	  /* currently we just need to assign the ped index for the child */
	    ttidx[ped][peel][ tntrans[ped][peel] ].set[CHILD]=set[CHILD];
	    ttidx[ped][peel][ tntrans[ped][peel] ].ped[CHILD]=child;
	    ttidx[ped][peel][ tntrans[ped][peel] ].set[DAD]=set[DAD];
	    ttidx[ped][peel][ tntrans[ped][peel] ].ped[DAD]=dad;
	    ttidx[ped][peel][ tntrans[ped][peel] ].set[MOM]=set[MOM];
	    ttidx[ped][peel][ tntrans[ped][peel]++ ].ped[MOM]=mom;
          }
        }
      }

   /*********************************
   ** look for a child in peel set **
   *********************************/
      for(i=0; i < p->n; i++){
	if( !ancestry[ped][p->ped[i]][FOUNDER] ) {
	  child = p->ped[i];
	  set[CHILD] = c->n + i;
	  mom = Not_Found;
	  dad = Not_Found;
        /* look for parents in cutset */
          for(j=0; j < c->n; j++)
	    if( c->ped[j] == ancestry[ped][child][FINDEX] ){
	      dad = c->ped[j]; /* get index within pedigree */
	      set[DAD] = j;  /* get index within union of cut and peel sets */
            }
	    else if( c->ped[j] == ancestry[ped][child][MINDEX] ){
	      mom = c->ped[j]; /* get index within pedigree */
	      set[MOM] = j;  /* get index within union of cut and peel sets */
            }
        /* if at least one of the parents are not in cut set then look */
	/* in peel set */
          if(dad == Not_Found || mom == Not_Found) 
            for(j=0; j < p->n; j++)
	      if( p->ped[j] == ancestry[ped][child][FINDEX] ){
	        dad = p->ped[j]; /* get index within pedigree */
	        set[DAD] = c->n + j; /* get index within union of cut and peel sets */
              }
	      else if( p->ped[j] == ancestry[ped][child][MINDEX] ){
	        mom = p->ped[j]; /* get index within pedigree */
	        set[MOM] = c->n + j; /* get index within union of cut and peel sets */
              }
        /* if all are found and the child is being peeled then we have a transition */
	  if( child != Not_Found && dad != Not_Found && mom != Not_Found ){
	  /* currently we just need to assign the ped index for the child */
	    ttidx[ped][peel][ tntrans[ped][peel] ].set[CHILD]=set[CHILD];
	    ttidx[ped][peel][ tntrans[ped][peel] ].ped[CHILD]=child;
	    ttidx[ped][peel][ tntrans[ped][peel] ].set[DAD]=set[DAD];
	    ttidx[ped][peel][ tntrans[ped][peel] ].ped[DAD]=dad;
	    ttidx[ped][peel][ tntrans[ped][peel] ].set[MOM]=set[MOM];
	    ttidx[ped][peel][ tntrans[ped][peel]++ ].ped[MOM]=mom;
          }
        }
      }
    }
  }

  *ntrans = tntrans;
  *tidx = ttidx;
}
	  
void DebugGetTransIndex(int     npedigrees,
		        int     *npeels,
		   	int     **ntrans,
		   	tindex  ***tidx){

  char response[2];

  printf("\n\nDebug GetTransIndex()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0) {

    tindex  *t;
    printf("\n\n");

    printf("peel: child (ped index) dad (ped index) mom (ped index)\n");
    printf("***** ***************** *************** ***************\n");
    for(ped=0; ped < npedigrees; ped++) {
      printf("  xx ped %2d xx\n",ped+1);
      for(peel=0; peel < npeels[ped]; peel++) 
        for(i=0; i < ntrans[ped][peel]; i++) {
	  t = &tidx[ped][peel][i];
	  printf("%3d:  %3d(%3d)          %3d(%3d)        %3d(%3d)\n", peel+1, t->set[CHILD], t->ped[CHILD], t->set[DAD], t->ped[DAD],  t->set[MOM], t->ped[MOM]);
        }
    }
  }
}
