/***************************************************************

  Program: SIMPLE 
  File:    CreateWPP.c
  Author:  Zachary Skrivanek 
  Date:    January 26, 2002 
  Purpose: Get working point potentials for complex pedigree.

***************************************************************/

#include "simple.h" /* c */
#include <stdio.h>

#define F 0
#define M 1 

#define FOUNDER 0

#define UNKNOWN 0 
#define UNAFFECTED 1 
#define AFFECTED 2

count   ped, peel, locus, i, j;
sindex  *p, *q;

void    nerror();

void  GetWPP(int     npedigrees,
	     int     ***ancestry,
	     int     *****haplabel,
	     int     nloci,
	     bool    isdisease, 
	     aff     **aff,
	     double  **liability,
	     double  *daf,
	     double  **maf,
	     int     *npeels,
	     sindex  **peelset,
	     int     ***peelspace,
	     int     ****peelspaces,
	     int     *phaps,
	     double  ****wpp){


  void    multi_base();
  double  af(), pen();

  for(locus=0; locus<nloci; locus++)
    for(ped=0; ped < npedigrees; ped++)
      for(peel=0; peel<npeels[ped]; peel++)
        for(i=0; i < peelspace[locus][ped][peel]; i++){
	  wpp[locus][ped][peel][i] = 1.0;
	  p = &peelset[ped][peel];
	  multi_base(p->n, &peelspaces[locus][ped][peel][0],i,phaps);
	  for(j=0; j < p->n; j++){


          /* multiply by penetrances */
	  /* assume 1.0 for markers */
	    if(locus >= (nloci-1) && isdisease)
	      wpp[locus][ped][peel][i] *= pen(phaps[j],&aff[ped][p->ped[j]],liability);
	    if(ancestry[ped][p->ped[j]][FOUNDER]){

	    /* multiply by allele freqs */
	      if(locus == (nloci-1) && isdisease)
	        wpp[locus][ped][peel][i] *= af(&haplabel[ped][p->ped[j]][locus][phaps[j]][0],daf);
              else wpp[locus][ped][peel][i] *= af(&haplabel[ped][p->ped[j]][locus][phaps[j]][0],&maf[locus][0]);

	    }
          } 
      } 
}


double af(int    *haplabel,
	  double *af)
{
  int  mF, mM;

  mF = haplabel[F];
  mM = haplabel[M];
  return(af[mF-1]*af[mM-1]);
}


double pen(int     hap,
	   aff     *aff,
	   double  **liability)
{
  int  idx;

/* 3 liabilities but 4 haplotypes for disease */
  if(hap >= 2) idx=hap-1;
  else idx=hap;

  if(aff->status == AFFECTED)
    return(liability[aff->class-1][idx]);
  else if(aff->status == UNAFFECTED)
    return(1-liability[aff->class-1][idx]);
  else if(aff->status == UNKNOWN)
    return(1.0);
  else
    nerror("Invalid affected status!");
} 


void  DebugGetWPP(int     npedigrees,
		  int     nloci,
		  int     *npeels,
		  int     ***peelspace,
		  double  ****wpp)
{
  char response[2];

  printf("\n\nDebug GetWPP()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0)
  {
    printf("\n\n");
    for(ped=0; ped < npedigrees; ped++){
      printf("ped %2d\n",ped+1);
      printf("******\n");
      for(locus=0; locus<nloci; locus++){
        printf("locus %2d\n",locus+1);
        printf("********\n");
        for(peel=0; peel<npeels[ped]; peel++)
          for(i=0; i < peelspace[locus][ped][peel]; i++)
	    printf("%2d: %lf\n",peel+1,wpp[locus][ped][peel][i]);
      }
    }
  }
}
    
