/* ************************************************************

   Program:   simple (Sequential Imputation for Multi-Point Linkage
              Estimation)
   File:      ParseCommands.c 
   Author:    Zachary Skrivanek
   Date:      September 2, 2000
   Purpose:   Read in commands for SIMPLE 
   Modified:  November 21, 2001 added c.fixaffected.par
   Modified:  1/25/02- added npedigrees to input so I could 
	      process npedigrees peel files

************************************************************* */
#include "simple.h"  /* aff, cmd */
#include <stdio.h>  /* fscanf */

#define TRUE  1 
#define FALSE 0
#define MAX_NAME 20 

void  ParseCommands(cmd   *c,
		    int   nargs,
                    FILE  *runfile,
		    int   *nloci,
		    int   nQT, 
		    int   *nalleles,
		    int   npedigrees,
		    int   isdisease)
		    
{

  cmd   tc;
  void  sort();
  int   nmarkers;
  int   i;
  char  command[MAX_NAME],command2[MAX_NAME],buffer[MAX_NAME];

  nmarkers = *nloci - isdisease;

/**************************
** assign default values **
**************************/

/* process loci in descending order by nalleles */
  tc.processorder.par = (int *) calloc(*nloci, sizeof(int));

  for(i=0; i<nmarkers; i++)
      tc.processorder.par[i]=i+1;

  sort(&tc.processorder.par[0], nalleles, nmarkers);
  if(isdisease) tc.processorder.par[*nloci-1]=*nloci;/* process disease last */

/* default LociName, PedName again since we are passing this
through a pointer */
  tc.load.par = (char *) calloc(MAX_NAME, sizeof(char));
  tc.load.def = TRUE; 

  tc.load2.par = (char *) calloc(MAX_NAME, sizeof(char));
  tc.load2.def = TRUE; 

  tc.pen.par = (char *) calloc(MAX_NAME, sizeof(char));
  tc.pen.def = TRUE; 

  tc.scan.par = (char *) calloc(MAX_NAME, sizeof(char));
  strcpy(tc.scan.par,"ped.dat");
  tc.scan.def = TRUE;

  tc.analysis.par = (char *) calloc(MAX_NAME, sizeof(char));
  strcpy(tc.analysis.par,"BOTH"); 
  tc.analysis.def=TRUE;

  tc.peel.par=FALSE; /* the default is not to use the peeling files */
  tc.peel.def=TRUE;

  tc.score.par = (char *) calloc(MAX_NAME, sizeof(char));
  strcpy(tc.score.par,"PAIRS"); 
  tc.score.def=TRUE;

  tc.units.par = (char *) calloc(MAX_NAME, sizeof(char));
  strcpy(tc.units.par,"rec-frac");
  tc.units.def = TRUE;

  tc.whichQT.par=1;
  tc.whichQT.def=TRUE;

  tc.diseaseloc.par=1;
  tc.diseaseloc.def=TRUE;

  tc.maxiter.par=5000;
  tc.maxiter.def = TRUE; 

  tc.nsteps.par=5;
  tc.nsteps.def = TRUE;

  tc.dumpibd.par=FALSE;
  tc.dumpibd.def = TRUE;

  tc.dumphap.par=FALSE;
  tc.dumphap.def = TRUE;

  tc.dumppairs.par=FALSE;
  tc.dumppairs.def = TRUE;

  tc.dumpprobs.par=FALSE;
  tc.dumpprobs.def = TRUE;

  tc.hasemanelston.par=FALSE;
  tc.hasemanelston.def = TRUE;

  tc.seed.par=123456789;
  tc.seed.def=TRUE;

  tc.missingQT.par=0.0;
  tc.missingQT.def=TRUE;

  tc.debug.par=FALSE;
  tc.debug.def=TRUE;

  tc.fixaffected.par=FALSE;
  tc.fixaffected.def=TRUE;

  tc.wstar.par=FALSE;
  tc.wstar.def=TRUE;

  tc.dumpdata.par=FALSE;
  tc.dumpdata.def=TRUE;

  tc.dumplikrats.par=FALSE;
  tc.dumplikrats.def=TRUE;

  tc.dumpscores.par=FALSE;
  tc.dumpscores.def=TRUE;

  tc.reweight.par=FALSE;
  tc.reweight.def=TRUE;

  tc.interference.model=19;    
  tc.interference.def=TRUE;
  tc.interference.par=0;      

  tc.file.par = (char *) calloc(MAX_NAME, sizeof(char));
  tc.file.nmodels=0;
  tc.file.def=TRUE;         

  tc.total_stat.par=FALSE;  
  tc.total_stat.def=TRUE;  

  tc.type.par=0;  
  tc.type.def=TRUE;  

  tc.sim.par=FALSE;  /* used in calclik only */
  tc.sim.def=TRUE;  /* used in calclik only */

  tc.joint.par=FALSE;
  tc.joint.def=TRUE;

  if(nargs==2){
    rewind(runfile);
    while(fscanf(runfile,"%s",command)==1){
      if(strcmp(command,"load")==0){
        if(fscanf(runfile,"%s",tc.load.par)!=1) /* redundant */
          nerror("expecting a file name after command 'load'");
	else tc.load.def=FALSE;
      }
      else if(strcmp(command,"load2")==0){
        if(fscanf(runfile,"%s",tc.load2.par)!=1) /* redundant */
          nerror("expecting a file name after command 'load2'");
	else tc.load2.def=FALSE;
      }
      else if(strcmp(command,"pen")==0){
        if(fscanf(runfile,"%s",tc.pen.par)!=1) 
          nerror("expecting a file name after command 'pen'");
	else tc.pen.def=FALSE;
      }
      else if(strcmp(command,"scan")==0){
        if(fscanf(runfile,"%s",tc.scan.par)!=1) /* redundant */
          nerror("expecting a file name after command 'scan'");
	else tc.scan.def=FALSE; 
      }
      else if(strcmp(command,"analysis")==0){
        if(fscanf(runfile,"%s",tc.analysis.par)!=1) 
          nerror("expecting a 'BOTH' or 'LOD' or 'NPL' after command 'analysis'");
        if((strcmp(tc.analysis.par,"BOTH")!=0) &&
           (strcmp(tc.analysis.par,"LOD")!=0) &&
           (strcmp(tc.analysis.par,"NPL")!=0) &&
           (strcmp(tc.analysis.par,"NONE")!=0)) 
          nerror("expecting a 'BOTH' or 'LOD' or 'NPL' or 'NONE' after command 'analysis'");
	/* if(((strcmp(tc.analysis.par,"NPL")==0)
         || (strcmp(tc.analysis.par,"BOTH")==0)) && !isdisease)
          nerror("You can't do NPL analysis without disease info!"); */
        /* if(isdisease && (strcmp(tc.analysis.par,"NPL")==0 || 
          (strcmp(tc.analysis.par,"NONE")==0))) 
          *nloci -= 1; */
	tc.analysis.def=FALSE; 
      } 
      else if(strcmp(command,"peel")==0){
	tc.peel.fname = (char **) calloc(npedigrees, sizeof(char *));
	for(i=0; i<npedigrees; i++){
	  tc.peel.fname[i] = (char *) calloc(MAX_NAME, sizeof(char));
          if(fscanf(runfile,"%s",tc.peel.fname[i])!=1) 
            nerror("expecting a file names after peel command!");
        }
	tc.peel.par=TRUE;
	tc.peel.def=FALSE;
      }
      else if(strcmp(command,"score")==0){
        if(fscanf(runfile,"%s",tc.score.par)!=1) 
          nerror("expecting a 'ALL' or 'PAIRS' after command 'score'");
        if((strcmp(tc.score.par,"ALL")!=0) &&
           (strcmp(tc.score.par,"PAIRS")!=0))
          nerror("expecting a 'ALL' or 'PAIRS' after command 'score'");
	else tc.score.def=FALSE; 
      } 
      else if(strcmp(command,"units")==0){
        if(fscanf(runfile,"%s",tc.units.par)!=1) 
          nerror("expecting a 'cM' or 'rec-frac' after command 'units'");
        if((strcmp(tc.units.par,"cM")!=0) &&
           (strcmp(tc.units.par,"rec-frac")!=0))
          nerror("expecting a 'cM' or 'rec-frac' after command 'units'");
	else tc.units.def=FALSE; 
      } 
      else if(strcmp(command,"debug")==0){
        tc.debug.par=TRUE;
	tc.debug.def=FALSE; 
      }
      else if(strcmp(command,"dumpibd")==0){
        tc.dumpibd.par=TRUE;
	tc.dumpibd.def=FALSE; 
      }
      else if(strcmp(command,"dumphap")==0){
        tc.dumphap.par=TRUE;
	tc.dumphap.def=FALSE; 
      }
      else if(strcmp(command,"dumppairs")==0){
        tc.dumppairs.par=TRUE;
	tc.dumppairs.def=FALSE; 
	if(tc.hasemanelston.par==TRUE) tc.type.par=3;
	else tc.type.par=2;
	tc.type.def=FALSE;
      }
      else if(strcmp(command,"dumpprobs")==0){
        tc.dumpprobs.par=TRUE;
	tc.dumpprobs.def=FALSE; 
	if(tc.hasemanelston.par==TRUE) tc.type.par=3;
	else tc.type.par=2;
	tc.type.def=FALSE;
      }
      else if(strcmp(command,"fix")==0){
        fscanf(runfile,"%s",command2);
	if(strcmp(command2,"affecteds")!=0)
          nerror("%s %s is not a valid command!",command,command2);
        tc.fixaffected.par=TRUE;
	tc.fixaffected.def=FALSE; 
      }
      else if(strcmp(command,"wstar")==0){
        tc.wstar.par=TRUE;
	tc.wstar.def=FALSE; 
      }
      else if(strcmp(command,"dumplikrats")==0){
        tc.dumplikrats.par=TRUE;
	tc.dumplikrats.def=FALSE; 
      }
      else if(strcmp(command,"dumpscores")==0){
        tc.dumpscores.par=TRUE;
	tc.dumpscores.def=FALSE; 
      }
      else if(strcmp(command,"reweight")==0){
        tc.reweight.par=TRUE;
	tc.reweight.def=FALSE; 
      }
      else if(strcmp(command,"haseman")==0){
        fscanf(runfile,"%s",command2);
	if(strcmp(command2,"elston")!=0){
          fprintf(stderr,"%s %s is not a valid command!",command,command2);
	  exit(1);
        }
        else {
	  tc.hasemanelston.def = FALSE;
	  tc.hasemanelston.par= TRUE;
	  /* if dumppairs then ntypes = 2 */
	  if(tc.dumppairs.par==TRUE) tc.type.par=3;
	  else tc.type.par=1;
	  if(nQT>1)
	    if(fscanf(runfile,"%d",&tc.whichQT.par)!=1)
	      nerror("There is more than one phenotype.  You must specifiy which one you wish to analyze with the haseman-elston statistc.");
            else tc.whichQT.def=FALSE;	
        }
      }
      else if(strcmp(command,"interference")==0){
        if(fscanf(runfile,"%d ",&tc.interference.model)!=1)
          nerror("expecting a '0' or '19' after command 'interference'");
        else tc.interference.def=FALSE;
        if(tc.interference.model == 19) if(fscanf(runfile,"%d ",&tc.interference.par)!=1)
          nerror("expecting an intensity paramater for chi-squared model");
      }
      else if(strcmp(command,"file")==0){
        if(fscanf(runfile,"%s",tc.file.par)!=1) /* redundant */
          nerror("expecting a file name for distances after command 'file'");
        tc.file.def = FALSE; 
        if(fscanf(runfile,"%d",&tc.file.nmodels)!=1)
          nerror("expecting nmodels after command 'file <filename> '");
      }
      else if(strcmp(command,"increment")==0){
        fscanf(runfile,"%s",command2);
	if(strcmp(command2,"step")!=0)
          nerror("%s %s is not a valid command!",command,command2);
        if(fscanf(runfile,"%d",&tc.nsteps.par)!=1)
          nerror("expecting an integer after command 'increment step'");
        else 
	  tc.nsteps.def = FALSE;
      }
      else if(strcmp(command,"maxiter")==0){
        if(fscanf(runfile,"%d",&tc.maxiter.par)!=1)
          nerror("expecting an integer after command 'maxiter'");
	else tc.maxiter.def=FALSE; 
      }
      else if(strcmp(command,"diseaseloc")==0){
        if(fscanf(runfile,"%d",&tc.diseaseloc.par)!=1)
          nerror("expecting an integer after command 'diseaseloc'");
	else tc.diseaseloc.def=FALSE; 
	if(tc.diseaseloc.par==1)
          if(fscanf(runfile,"%lf",&tc.dis)!=1)
            nerror("expecting a double after diseaseloc 1"); 
      }
      else if(strcmp(command,"diseaseloc2")==0){
        if(fscanf(runfile,"%d",&tc.diseaseloc2.par)!=1)
          nerror("expecting an integer after command 'diseaseloc'");
	else tc.diseaseloc2.def=FALSE; 
	if(tc.diseaseloc2.par==1)
          if(fscanf(runfile,"%lf",&tc.dis)!=1)
            nerror("expecting a double after diseaseloc2 1"); 
      }
      else if(strcmp(command,"dumpdata")==0){
        tc.dumpdata.par=TRUE;
	tc.dumpdata.def=FALSE; 
      }
      else if(strcmp(command,"seed")==0){
        if(fscanf(runfile,"%d",&tc.seed.par)!=1)
          nerror("expecting an integer after command 'seed'");
	else tc.seed.def=FALSE;
      }
      else if(strcmp(command,"missingQT")==0){
        if(fscanf(runfile,"%lf",&tc.missingQT.par)!=1)
          nerror("expecting a real number after command 'missingQT'");
	else tc.missingQT.def=FALSE;
      }
      else if(strcmp(command,"process")==0){
	fscanf(runfile,"%s",command2);
	if(strcmp(command2,"order")!=0)
	  nerror("%s %s is not a valid command!", command, command2);
        for(i=0; i<*nloci; i++)
          if(fscanf(runfile,"%d",&tc.processorder.par[i])!=1)
          {
            fprintf(stderr,"Expecting nloci=%d integers after command 'process order'.\n",*nloci);
            nerror("If you used 'analysis NPL', this command should come before the command 'process order'\nto adjust nloci accordingly.");
          }
	  else tc.processorder.def=FALSE;
      }
      else if(strcmp(command,"sim")==0){
        tc.sim.par=TRUE;
	tc.sim.def=FALSE;
      }
      else if(strcmp(command,"total")==0){
        fscanf(runfile,"%s",command2);
	if(strcmp(command2,"stat")!=0)
	  nerror("%s %s is not a valid command!",command, command2);
	tc.total_stat.par=TRUE;
	tc.total_stat.def=FALSE;
      }
      /* skip if commented out */
      else if(command[0]=='#' || (command[0]=='/' && command[1]=='*') || (command[0]=='<' && command[1]=='<')) 
	fgets(buffer, MAX_NAME-1, runfile);
         
      else if(strcmp(command,"joint")==0){
        tc.joint.par=TRUE;
	tc.joint.def=FALSE; 
      }
      else{
        fprintf(stderr,"%s:  ",command);
        nerror("Not a valid command!");
      }
    }
    fclose(runfile);
  }

  /* if(isdisease) put this inside the command=anlaysis part
    if(strcmp(tc.analysis.par,"NPL")==0 || 
      (strcmp(tc.analysis.par,"NONE")==0)) 
      *nloci -= 1;*/

  if(!isdisease)
    if(strcmp(tc.analysis.par,"NONE")!=0) nerror("There is no disease info to process!");
    else if(tc.hasemanelston.par==TRUE) nerror("There is no disease info to process!");

  *c = tc; 
}
