/* ************************************************************

   Program:  scan
   File:     scan.c
   Author:   Zachary Skrivanek
   Date:     November 13, 2000
   Purpose:  Calculates Whittemore & Halpern's Multi-Locus 
             Non-Parametric Statistics (ALL and PAIRS) and/or LOD 
             Zscoress using output from SIMPLE. This is a modified
	     version of PrintScores.c.  The difference is that
	     it can deal with a various number of interior points
	     instead of a fixed number.  It used only the scores
	     at the markers to estimate their scores but assumes
	     that reweighted scores are generated at S[i][0][ped]
	     and S[i][numlik[i]-1][ped] 
   
************************************************************* */
#include <stdio.h> /* scanf() printf() */
#include <stdlib.h> /* pow() */
#include <math.h> /* sqrt,pow,log10 */
#include "simple.h"   /* c commands */ 
/* #include <sys/ddi.h> I don't think I need this and besides
it causes problems on allele */

#define TRUE 1
#define FALSE 0 

#define ALPHA 0 
#define BETA 1 
#define SIGMA 2 

void   PrintScores(double ***S,
                   double ***Z_var,
                   double ***p_value,
                   double ***Z,
                   double *null_mn,
                   double *null_var,
                   double **lik,
                   double **lod_var,
		   double ****line,
		   double ******IBDprob,
                   double **position,
                   int    nmarkers,
                   cmd    c,
                   int    *numlik,
                   int    npedigrees,
		   prs    ***pairs,
		   int    **npairs){
		   

  int    i,j,k,ped,numlik_tmp,type;
  double null_lik,lod,lod2,lod3,var,nvar,pos,w1,w2,w3,e=0.0001;
  double slope, intercept, sigma_2;
  double Zscores, Zscores2,scores,scores2, var2, p_value_tmp, p_value_tmp2;
  char   *labels[] ={"sib"};

  double CalcQTLlod();
  double max(double, double);
  
  if((strcmp(c.analysis.par,"BOTH")==0) || (strcmp(c.analysis.par,"NPL")==0))
  { 
    if(!c.sim.par){
      printf("Null Scores\n");
      printf("ped     mean        var\n");
      printf("*** ********* ********* \n");
      Zscores = 0.0;
      scores = 0.0;
      var = 0.0;
      for(ped=0; ped<npedigrees; ped++){
        if(c.total_stat.par)  {
          Zscores += null_mn[ped];
          nvar += null_var[ped];
        }
        else
          printf("%3d %9.6lf %9.6lf\n",ped+1,null_mn[ped],null_var[ped]);
      }
      if(c.total_stat.par){
        Zscores /= sqrt(npedigrees);
        var /= npedigrees;
        printf("all %9.6lf %9.6lf %9.6lf \n ",scores,Zscores,sqrt(nvar));
      }
      printf("\n");
  
      printf("ped   pos         S         Z +/-     SE(Z) p-valu\n");
      printf("*** ***** ********* ********* *** ********* *******\n");
      /* print the first marker's scores */ 
      for(ped=0; ped<npedigrees; ped++)
        printf("%3d %5.2lf %9.6lf %9.6lf +/- %9.6lf %6.4lf \n", ped+1, 100*position[1][0], S[0][0][ped],Z[0][0][ped], sqrt(Z_var[0][0][ped]/((double) c.maxiter.par) * null_var[ped]),p_value[0][0][ped]);
      Zscores = 0.0;
      scores = 0.0;
      var = 0.0;
      for(i=2; i< c.file.nmodels; i++){
        for(j=0; j<numlik[i]; j++){
          Zscores = 0.0;
          scores = 0.0;
          var = 0.0;
          p_value_tmp = 0.0;
          p_value_tmp2 = 0.0;
          for(ped=0; ped<npedigrees; ped++){
            Zscores = Z[i-1][j][ped];
            scores = S[i-1][j][ped];
            var = Z_var[i-1][j][ped];
            p_value_tmp = p_value[i-1][j][ped];
            pos = position[i][j];
            if(c.total_stat.par){
              Zscores += Z[i-1][j][ped];
              scores += S[i-1][j][ped];
              var += Z_var[i-1][j][ped];
              p_value_tmp2 = max(p_value_tmp,p_value_tmp2);
            }
            else
              printf("%3d %5.2lf %9.6lf %9.6lf +/- %9.6lf %6.4lf \n", ped+1, 100*pos,scores,Zscores, sqrt(var/((double) c.maxiter.par) * null_var[ped]),p_value_tmp);
          } /* ped */
        } /* numlik */ 
        if(c.total_stat.par){
          Zscores /= sqrt(npedigrees);
          var /= npedigrees;
	  var /= nvar;
          printf("%3d %5.2lf %9.6lf %9.6lf +/- %9.6lf %6.4lf \n", ped+1, 100*pos,Zscores,scores, sqrt(var/((double) c.maxiter.par) * null_var[ped]),p_value_tmp2);
        }
      } /* nmodels */ 
      printf("\n");
    }
    else { /* if sim */
      for(ped=0; ped<npedigrees; ped++)
        printf("%3d %9.6le %9.6le\n",ped+1,null_mn[ped],null_var[ped]);
      for(i=1; i<c.file.nmodels; i++)
	for(j=0; j<numlik[i]; j++)
          for(ped=0; ped<npedigrees; ped++)
            printf("%9.6le ", Z[i-1][j][ped]);
      printf("\n");
      for(i=1; i<c.file.nmodels; i++)
	for(j=0; j<numlik[i]; j++)
          for(ped=0; ped<npedigrees; ped++)
            printf("%9.6le ", S[i-1][j][ped]);
      printf("\n");
      /* just print out the variances vs SE */
      for(i=1; i<c.file.nmodels; i++)
	for(j=0; j<numlik[i]; j++)
          for(ped=0; ped<npedigrees; ped++)
            printf("%9.6le ", Z_var[i-1][j][ped]);
      printf("\n");
    } 
  
  } 

  if((strcmp(c.analysis.par,"BOTH")==0) || (strcmp(c.analysis.par,"LOD")==0))
  {
    null_lik=lik[0][0];
    if(!c.sim.par){
      printf("\nLOD Scores\n");
      printf("pos   LOD       SE(LOD)\n");
      printf("***** *****************\n");
    }

        if(lik[1][0] > 0.0)
          lod = log10(lik[1][0]/null_lik);
        else
          lod=-10000;
        var = lod_var[1][0];
	pos=position[1][0];
        if(c.sim.par)
          printf("%10.6lf %6.4lf ", lod,sqrt(var));
        else
          printf("%5.2lf %10.6lf %6.4lf\n", 100*pos,lod,sqrt(var));

/* assuming we simulate in the model of every interval */
/* and we do reweighting at markers and all interior points */ 
    for(i=1; i < nmarkers; i++){
      for(j=1; j < c.nsteps.par+1; j++){
        if(lik[i][j] > 0.0)
          lod = log10(lik[i][j]/null_lik);
        else
          lod=-10000;
        var = lod_var[i][j];


        if(j == c.nsteps.par){
          if(i < nmarkers - 2){
            w1 = 1/(var + e);
            w2 = 1/(lod_var[i+1][0] + e);
            if(lik[i+1][0] > 0.0)
              lod = (lod * w1 + log10(lik[i+1][0]/null_lik) * w1)/(w1+w2);
            else
              lod=-10000;
            var = (var*pow(w1,2.0) + lod_var[i+1][0]*pow(w2,2.0))/pow(w1+w2,2.0);
          }
/* else just print the end point and don't average */
        }
	/* hack */
	pos=position[(i-1)*(c.nsteps.par)+j+1][0];
        if(c.sim.par)
          printf("%10.6lf %6.4lf ", lod,sqrt(var));
        else
          printf("%5.2lf %10.6lf %6.4lf\n", 100*pos,lod,sqrt(var));
      }
    }
    printf("\n");
  }

/*
    while(i < c.file.nmodels){
      numlik_tmp=numlik[i];
      while(j < numlik_tmp){
        if(lik[i-nmarkers][j] > 0.0)
          lod = log10(lik[i-nmarkers][j]/null_lik);
        else
          lod=-10000;
        var = lod_var[i-nmarkers][j];
        pos = position[i][j];
        if(j == numlik[i] -1){
          if(i < c.file.nmodels - 1){
            w1 = 1/(var + e);
            w2 = 1/(lod_var[i+1-nmarkers][0] + e);
            if(lik[i+1-nmarkers][0] > 0.0)
              lod = (lod * w1 + log10(lik[i+1-nmarkers][0]/null_lik) * w1)/(w1+w2);
            else
              lod=-10000;
            var = (var*pow(w1,2.0) + lod_var[i+1-nmarkers][0]*pow(w2,2.0))/pow(w1+w2,2.0);
            j=1;
          }
          else ++j;
          ++i;
        }
        else ++j;
        if(c.sim.par)
          printf("%10.6lf %6.4lf ", lod,sqrt(var));
        else
          printf("%5.2lf %10.6lf %6.4lf\n", 100*pos,lod,sqrt(var));
      }
    }
    printf("\n");
  }
*/

  if(c.hasemanelston.par==TRUE)
  { 
    if(!c.sim.par){
      printf("ML estimator of haseman-elston statistic\n");
      printf("pos    type       LOD     slope\n");
      printf("****** ************************\n");
    }
    for(i=2; i< c.file.nmodels; i++)
      for(j=0; j<numlik[i]; j++){
        pos = position[i-1][j];
  /*      for(type=0; type<ntypes; type++){ 7/18/02 */
        type=0;
          lod = CalcQTLlod(&line[i-1][j][type][0],
			   &IBDprob[i-1][j][type][0],
			   &pairs[type][0],
			   &npairs[type][0],
			   npedigrees);

          slope=line[i-1][j][type][BETA];

          if(c.sim.par)
            printf("%9.6lf %9.6lf ",lod, slope);
          else
            printf("%6.3lf %4s %9.6lf %9.6lf \n", 100*pos,labels[type],lod, slope);
        /* } 7/18/02 */
      } /* numlik */
  } /* if he */
}
 

