/* **********************************************************************

    Program:   SIMPLE, SCAN
    File:      ReadDistances.c
    Author:    Zachary Skrivanek
    Date:      August 9, 2001
    Purpose:   Reads in the distances desired to be analyzed using 
               reweighting method.

************************************************************************* */
#include <stdio.h>
#include "simple.h" /* c */

void ReadDistances(int    nloci,
		   int    ***locusorder,
		   double ***distances,
		   int    **numlik,
		   double ****likdistances,
		   cfile  file)

{
  FILE *fp;
  FILE *fopen();

  int i,j,k;
  int **tlocusorder;
  double **tdistances;
  int *tnumlik;
  double ***tlikdistances;

  if( (fp = fopen(file.par,"r")) == NULL)
     nerror("%s not available for reading",file.par);

  tlocusorder = (int **) calloc(file.nmodels,sizeof(int *));
  tnumlik = (int *) calloc(file.nmodels,sizeof(int));
  tdistances = (double **) calloc(file.nmodels,sizeof(double *));
  tlikdistances = (double ***) calloc(file.nmodels,sizeof(double **));
  for(i=0; i<file.nmodels; i++) {
    tdistances[i] = (double *) calloc(nloci-1,sizeof(double));
    tlocusorder[i] = (int *) calloc(nloci,sizeof(int));
  }

  for(i=0; i<file.nmodels; i++) {
    for(j=0; j<nloci; j++)
      fscanf(fp,"%d",&tlocusorder[i][j]);
    for(j=0; j<nloci-1; j++)
      fscanf(fp,"%le ",&tdistances[i][j]);
    fscanf(fp,"%d",&tnumlik[i]);
    tlikdistances[i] = (double **) calloc(tnumlik[i],sizeof(double *));
    for(j=0; j<tnumlik[i]; j++) {
      tlikdistances[i][j] = (double *) calloc(nloci-1,sizeof(double));
      for(k=0; k<nloci-1; k++)
        fscanf(fp,"%le",&tlikdistances[i][j][k]);
    }
  }

  *locusorder = tlocusorder;
  *distances = tdistances;
  *numlik = tnumlik;
  *likdistances = tlikdistances;
}

void DebugReadDistances(int     nmarkers,
                          int     nmodels,
                          int     **locusorder,
                          double  **distances,
                          int     *numlik,
                          double  ***likdistances)
{
  char response[2],c;
  int  nloci;

  nloci = nmarkers+1;

  printf("\n\nDebug CreateDistances()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) 
  {

    int    i,j,k;

    for(i=0; i<nmodels; i++) {
      for(j=0; j<nloci; j++)
        printf("%d ",locusorder[i][j]);
      printf("\n");
      for(j=0; j<nloci-1; j++)
        printf("%le ",distances[i][j]);
      printf("\n");
      for(j=0; j<numlik[i]; j++){
        for(k=0; k<nloci-1; k++)
          printf("%le ",likdistances[i][j][k]);
        printf("\n");
      }
    }
  }
}


/*
void DebugReadRecombFracs(int nloci,
			  int nmodels,
			  int **locusorder,
			  double **distances,
			  int *numlik,
			  double ***likdistances)
{
  char response[2];

  printf("\n\nDebug ReadRecombFracs ()? =>");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) {

    int i,j,k;

    printf("\n\n");
    for(i=0; i<nmodels; i++) {
      printf("The locus order for model %d is:",i);
      for(j=0; j<nloci; j++)
        printf(" %d",locusorder[i][j]);
      printf("\n\n");
      printf("Recombination Probabilities for Model %d\n\n",i);
      printf(" Loci Prob[Recombination]\n");
      for(j=0; j<nloci-1; j++)
        printf("%2d %2d    %7.5lf \n",j+1,j+2,distances[i][j]);
      for(k=0; k<numlik[i]; k++) {
        printf("\nRecombination Probabilities for model %d, submodel %d are:\n"
								,i,k);
        printf(" Loci Prob[Recombination]\n");
        for(j=0; j<nloci-1; j++)
          printf("%2d %2d    %7.5lf \n",j+1,j+2,likdistances[i][k][j]);
      }
    }
  }

}*/

