/***************************************************************

  Program:   simple 
  File:      ReadLinkageFile.c
  Author:    Zachary Skrivanek 
  Date:      Aug 31, 2000 
  Purpose:   Read GENEHUNTER linkage file.  If there is a disease
	     we assume it is placed first in the file and is given
	     1 in the locus order.  We also assume that the 
	     command lines #3 (order) is followed by a comment 
	     character.
  Modified:  May 25, 2002  Modified the algorithm to determine 
	     number of loci.  Now there is no limit.

***************************************************************/
#include "simple.h"  /* aff */
#include <stdio.h> /*fclose */
#include <math.h>
#include <string.h> /* strcmp */

#define MAX 256
#define MAX_ALLELES 30
#define D 1  /* disease allele */
#define d 0  /* normal allele */
#define TRUE 1
#define FALSE 0


int    i,j,k;

void  ReadLinkageFile(int    *nloci,
		      double **mdistances,
		      int    **markerorder, 
                      int    **nalleles,
		      double ***maf,
		      int    *isdisease,
		      int    *nQT,
		      double *disaf,
		      double ***liability,
		      int    *no_liab_class,
		      char   *LociName){

  int    tnloci, npheno;
  double *tmdistances;
  int    *tnalleles;
  double **tmaf;
  int    tisdisease=FALSE;
  int    tnQT=0,ttnQT=0;
  double **tliability;
  int    tinterference;   /* we don't actually use this */ 
  int    tm;              /* later on we'll use this */ 
  int    tno_liab_class;


  FILE   *gfopen(), *loci;
  double tmp1, tmp2;
  int    tmp; 
  char   buffer[MAX], *ctl_str;
  int    locus_type;
  int    locus;    /* for indexing only */
  int    nmarkers;
  int    *tmarkerorder;
  char   *q;

/* open loci file for read */
  loci = gfopen(LociName,"r");

/* get npheno = nmarkers+nQT+isdisease */
  fscanf(loci,"%d",&npheno);

/* skip the rest of line 1 and the next line */
/* and we'll use the 3rd line to figure out nloci */
  for(i=0; i<3; i++)
    fgets(buffer, MAX-1,loci);

/*
  ctl_str="%d%d%d%d%d%d%d%d%d%d%d%d%d%d%d%d";
  tnloci=sscanf(buffer,ctl_str,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp,&tmp);

  if(tnloci > 15) nerror("SIMPLE can not process more than 15 markers");
*/
  tnloci=0;
  for(q=buffer; *q != '\0'; )
    if(isdigit(*q)){
      while(isdigit(*q))
        ++q;
      ++tnloci;
    }
    else ++q;
  

  ttnQT=npheno-tnloci;
  nmarkers = tnloci; /* we subtract 1 if we find a disease */

  tmarkerorder = (int *) calloc(tnloci, sizeof(int));
  rewind(loci);
  fgets(buffer, MAX-1, loci);
  fgets(buffer, MAX-1, loci);
  for(locus=0; locus<tnloci; locus++){
    fscanf(loci,"%d",&tmarkerorder[locus]);
  }


/* read in locus order there are less than tnloci marker usually
unless there is no disease and no qtl 
  tmarkerorder = (int *) calloc(nloci,sizeof(int));

  while(fscanf(loci,"%d",&tmarkerorder[locus])==1)
    locus++;
*/
/* the loci order is given only for markers and disease locus 
   (not QT) if there is a disease we will subtract 1 from 
   nmarkers */

  /* nmarkers=locus;

  ttnQT = tnloci-nmarkers; */

/* for(locus=0; locus<tnloci; locus++)
    if(fscanf(loci,"%d",&tmarkerorder[locus])!=1){
       fprintf(stderr,"locus order for locus # %d is not an int!\n",locus+1);
       nerror("Linkage File format error!"); 
    }
*/


/* if there is a disease we subtract one for marker order since
   we assume the disease is placed as 1 in the physical order */
  fgets(buffer,MAX-1,loci); /* discard the rest of the line */

  tmaf=(double**) calloc(tnloci, sizeof(double*)); 
  tnalleles=(int*) calloc(tnloci, sizeof(int));
  for(locus=0; locus<tnloci; locus++)
    tmaf[locus]= (double*) calloc(MAX_ALLELES,sizeof(double));
/* if there is a disease then we allocated too much space.  
   So what! */

/* read in locus data */
  for(locus=0; locus<npheno; locus++){
    if(fscanf(loci,"%d",&locus_type)!=1) 
      nerror("Linkage File format error:  locus type not an integer!");
    switch(locus_type){

      case 0:

        fgets(buffer,MAX-1,loci); /* discard the rest of the line */
        for(i=0; i<5; i++)
          fgets(buffer, MAX-1, loci);
        tnQT++;
      break;

      case 1:

      /* get disease locus data */
      /*  assume that the disease is first */
        tisdisease = TRUE;
        for(i=0; i<tnloci; i++)
          tmarkerorder[i] -= 1;

/*	diseaseloc=locus+1-tnQT;  */

      /* get gene frequency */
        nmarkers -= 1;
        fgets(buffer,MAX-1,loci); /* discard the rest of the line */
		  /* assume two alleles */
        if(fscanf(loci, "%le %le", &disaf[d], &disaf[D])!=2)
          nerror("Linkage File format error:  dis freq not real!");
        fgets(buffer,MAX-1,loci); /* discard the rest of the line */
          
      /*get number of liability classes */
        fgets(buffer, MAX-1,loci);
        if(sscanf(buffer, "%d", &tno_liab_class)!=1)
          nerror("Linkage File format error:  no_liab_class not int!");
      
      /* get penetrances */
        tliability =(double**) calloc(tno_liab_class, sizeof(double*));
        for(i=0;i<tno_liab_class; i++)
           tliability[i]=(double*)calloc(3, sizeof(double));  
      
        for(i=0; i<tno_liab_class; i++){
          for(j=0; j<3; j++)
            if(fscanf(loci, "%le", &tliability[i][j])!=1){
              fprintf(stderr,"penetrance %d for liab class %d not real!\n",i+1,j+1);
              nerror("Linkage File format error!"); 
            }
          fgets(buffer, MAX-1, loci);
        }
      break;

      case 3:

      /* get marker data */
        fgets(buffer, MAX-1, loci);
        if(sscanf(buffer, "%d", &tnalleles[tmarkerorder[locus]-1])!=1){
          fprintf(stderr,"nalleles for marker # %d is not an int!\n",locus-tisdisease+1);
          nerror("Linkage File format error!"); 
        }
	
      /* read in marker freqs */
        for(j=0; j < tnalleles[tmarkerorder[locus]-1]; j++)
          if(fscanf(loci, "%le", &tmaf[tmarkerorder[locus]-1][j])!=1){
            fprintf(stderr,"maf for marker # %d allele # %d is not real!\n",locus-tisdisease+1,j+1);
            nerror("Linkage File format error!"); 
          } 
        fgets(buffer, MAX-1, loci);
      break;

      default:
	fprintf(stderr,"locus type = %d",locus_type);
        nerror("locus type is not 0, 1 or 3!");
    }
  } /* read in loci data */


  if(ttnQT!=tnQT) {
    fprintf(stderr,"There are %d quantitative phenotypes listed but there are %d phenotypes (1st number of line one) and the loci order is given for %d loci implying that there should be %d-%d=%d phenotypes\n",tnQT,npheno, tnloci, npheno, tnloci, ttnQT);
     nerror("Linkage File format error!"); 
  }
  if(fscanf(loci, "%d %d",&tmp, &tinterference)!=2)
    nerror("Linkage File format error: interference model not int!"); 


  if(tinterference > 0)
    fscanf(loci,"%d",&tm);
  fgets(buffer, MAX-1, loci);

  tmdistances = (double*) calloc(tnloci-tisdisease-1, sizeof(double));
  /* tdistances = (double*) calloc(tnloci-1, sizeof(double)); */
/* discard first distance if there is a disease */
  /* if(tisdisease) fscanf(loci, "%lf", &tdistances[0]); */
  if(tisdisease) fscanf(loci, "%lf", &tmp1); 
  for(i=0; i<nmarkers-1; i++){
     if(fscanf(loci, "%lf", &tmdistances[i])!=1)
       nerror("Linkage File format error: distance not real!"); 
     /* tdistances[i+1]=tmdistances[i]; */
  }
  fclose(loci);

  /* tnloci=nmarkers+tisdisease; */

  *nloci=tnloci;
  *mdistances=tmdistances;
  /* *distances=tdistances; */
  /* *locusorder=tlocusorder; */
  *markerorder=tmarkerorder;
  *nalleles=tnalleles;
  *maf=tmaf;
  *isdisease=tisdisease;
  /* *diseaseloc=tdiseaseloc; */
  *nQT=tnQT;
  *liability=tliability;
  /* *interference=tinterference;
  *m=tm; */
  *no_liab_class=tno_liab_class;

} /* end of ReadLinkageFile */ 


void DebugReadRun(int     debug,
                  int     interference,
                  int     m,
                  int     nmarkers,
                  cfile   *file,
                  double  *mdistances,
                  int     nsteps,
                  int     isdisease,
                  int     *processorder,
                  int     maxiter,
                  int     seed)
{
  char response[2];

  printf("\n\nDebug ReadRun()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0)
  {


    printf("\n\n");
    if(debug)
      printf("Debug mode is ON");
    else
      printf("Debug mode is OFF");
    printf("\n\n");
    if(interference == 0)
      printf("NO interference model (Haldane) \n\n");
    else if(interference == 19)
    {
      printf("The interference model is:  CHS \n");
      printf("The intensity parameter is: %d \n\n",m);
    }
    printf("There are %d markers",nmarkers);
    printf("\n\n");
    if(!(file->def)){
      printf("A file will be provided to do the analysis\n");
      printf("with nmodels=%d.\n",file->nmodels);
    }
    else 
    {
      printf("The genetic distances are: \n");
      printf("Loci:    ");
      for(i=0; i< nmarkers; i++)
          printf("%-2d      ",i+1);
      printf("\n         ");
      for(i=0; i<(nmarkers-1); i++)
        printf("--------");
      printf("--\n");
      printf("cMorgs:   ");
      for(i=0; i < (nmarkers-1); i++)
        printf("  %-5.3lf ", mdistances[i]);
      printf("\n\n");
      printf("%d steps per interval will be simulated\n\n",nsteps);
    }
    if( isdisease )
        printf("There is a disease locus");
    else
      printf("There is no disease locus");
    printf("\n\n");
    for(i=0; i < nmarkers + isdisease; i++)
        printf("Locus # %d is peeled in step # %d\n",
               processorder[i],i+1);
    printf("\n\n");
    printf("Number of iterations = %d\n\n",maxiter);
    printf("The seed is %d\n",seed);

  }
}
