/***************************************************************

  Program: SIMPLE 
  File:    ReadPedFile.c
  Author:  Zachary Skrivanek 
  Date:    September 1, 2000 
  Purpose: Read in GH ped.dat file.

***************************************************************/

#include <stdio.h> /*fclose */
#include <math.h>
#include <string.h> /* strcmp */
/* #include "simple.h" affstatus, qpheno, c */
/* #ifdef HapFreqs
#include "/home/zachary/genetics/Haplotype/src/simple.h" 
#endif

#ifdef SIMPLE
#include "simple.h"
#endif */

#include "simple.h"

#define MAX 256
#define MAX_NAME 20
#define MAX_PEDS 30
#define TRUE 1
#define FALSE 0 
#define MA 0
#define MB 1
#define PID 0
#define FID 1
#define MID 2

int    i,j,k,l;

void  ReadPedFile(int    *npedigrees,
		  char   ***pednames,
	          int    **pedigreesize,
	          int    ****pedigree,
	          int    *****marker,
	          int    nmarkers,
	          int    nQTL,
		  double ****qpheno,
		  int    isdisease,
		  aff    ***affected,
	          int    no_liab_class,
	          cmd    c){
  
  int    tnpedigrees;
  int    *tpedigreesize;
  int    ***tpedigree;
  int    ****tmarker;
  double ***tqpheno;
  aff    **taffected;
  char   **tpednames;

  FILE   *ped, *gfopen();
  char   buffer[MAX], ped1[MAX_NAME], ped2[MAX_NAME];

/* open ped file for read */
  ped = gfopen(c.scan.par,"r");

  tpedigreesize = (int *) calloc(MAX_PEDS, sizeof(int));
  tpednames = (char **) calloc(MAX_PEDS, sizeof(char *));
  for(i=0; i<MAX_PEDS; i++)
    tpednames[i] = (char *) calloc(MAX_NAME, sizeof(char));

/* count number of pedigrees and how many in each */
  tnpedigrees=0;
  fgets(buffer,MAX-1,ped);
  sscanf(buffer,"%s", ped2);
  while(fgets(buffer,MAX-1, ped)!=NULL) {
    tpedigreesize[tnpedigrees]++;
    strcpy(ped1, ped2);
    sscanf(buffer, "%s", ped2);
    while(strcmp(ped1, ped2)==0)
    {
      tpedigreesize[tnpedigrees]++;
      fgets(buffer, MAX-1, ped);
      if(feof(ped)) break;
      sscanf(buffer, "%s", ped2);
    }
    tnpedigrees++;
  }

  rewind(ped);
  tpedigree=(int***)calloc(tnpedigrees, sizeof(int**));
  tmarker=(int****)calloc(tnpedigrees, sizeof(int***));
  tqpheno=(double***)calloc(tnpedigrees, sizeof(double**));
  taffected=(aff**)calloc(tnpedigrees, sizeof(aff*));
  for(i=0; i<tnpedigrees; i++){
    tpedigree[i]=(int**)calloc(tpedigreesize[i], sizeof(int*));
    tmarker[i]=(int***)calloc(tpedigreesize[i], sizeof(int**));
    tqpheno[i]=(double**)calloc(tpedigreesize[i], sizeof(double*));
    taffected[i]=(aff*)calloc(tpedigreesize[i], sizeof(aff));
    for(j=0; j<tpedigreesize[i]; j++){
      tpedigree[i][j]=(int*)calloc(3, sizeof(int));
      tmarker[i][j]=(int**)calloc(nmarkers, sizeof(int*));
      tqpheno[i][j]=(double*)calloc(nQTL, sizeof(double));
      for(k=0; k<nmarkers; k++)
	tmarker[i][j][k]=(int*)calloc(2,sizeof(int));
      if(feof(ped)){
	fprintf(stderr,"ped: %d ind: %d \n",i+1,j+1);
        nerror("Number of individuals does not match the pedigree file!\n");
      }
      fscanf(ped, "%s", buffer);   
      if(j==0) strcpy(&tpednames[i][0],buffer);
      for(k=0; k<3; k++)
        fscanf(ped,"%d", &tpedigree[i][j][k]);
      fscanf(ped, "%d", &k); /* we don't need sex for simple */
      /* set the default to unaffected */
      taffected[i][j].status=1;
      if(isdisease){
	fscanf(ped,"%d",&taffected[i][j].status);
        taffected[i][j].class = 1; /* the default class is 1 */
        if(no_liab_class>1) fscanf(ped,"%d",&taffected[i][j].class);
      }
      for(k=0; k<nmarkers; k++)
	for(l=0; l<2; l++)
	  fscanf(ped,"%d",&tmarker[i][j][k][l]);
      for(k=0; k<nQTL; k++)
      /* if non-numeric then we assume it is missing */
      /* and of course if the number corresponds to */
      /* c.missingQT.par then we also assume it is */
      /* missing */
	if(fscanf(ped, "%lf", &tqpheno[i][j][k])!=1){
	  tqpheno[i][j][k]==c.missingQT.par;
	  /* skip the missing value */
	  fscanf(ped, "%s", buffer);
        }
    }
  }
  fclose(ped);

  *npedigrees = tnpedigrees;
  *pednames = tpednames;
  *pedigreesize = tpedigreesize;
  *pedigree = tpedigree;
  *marker = tmarker;
  *qpheno = tqpheno;
  *affected = taffected;

} /* end of ReadPedFile.c */


void DebugReadPedigree(int npedigrees,
                       int *pedigreesize,
                       int ***pedigree)
{
  char response[2];

  printf("\n\nDebug ReadPedigree()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) {


    printf("\n\n");
    printf("Number of Pedigrees:  %d",npedigrees);
    printf("\n");
    for(i=0; i < npedigrees; i++) {
      printf("\n\n");
      printf("\t*** Pedigree #%d  (%d Persons) ***", i+1, pedigreesize[i]);
      printf("\n\n");
      printf("PersonID  FatherID  MotherID");
      printf("\n");
      printf("--------  --------  --------");
      for(j=0; j < pedigreesize[i]; j++) {
        printf("\n");
        printf("%8d  %8d  %8d",
               pedigree[i][j][PID], pedigree[i][j][FID], pedigree[i][j][MID]);
      }
    }
    printf("\n\n");
  }
}



void DebugReadMarker(int    npedigrees,
                     int    *pedigreesize,
                     int    ***pedigree,
                     int    nmarkers,
                     int    *nalleles,
                     int    ****marker,
                     double **maf,
		     int    nQTL,
		     double ***qpheno,
		     double missingQT)
{
  char response[2];


  printf("\n\nDebug Marker & QTL Info()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {

    int nmarker,qtl;


    printf("\n\n");
    printf("There are %d markers",nmarkers);
    printf("\n");
    for(nmarker=0; nmarker < nmarkers; nmarker++) {

      printf("\n");
      printf("Marker #%d has %d alleles",nmarker+1,nalleles[nmarker]);
      printf("\n\n");
      printf("Marker Allele Frequencies:");
      printf("\n\n");
      printf("Marker  Frequency");
      printf("\n");
      printf("------  ---------");
      printf("\n");
      for(i=0; i < nalleles[nmarker]; i++) {
        printf("%6d  %9.3le",i+1,maf[nmarker][i]);
        printf("\n");
      }
      printf("\n\n");
      for(i=0; i < npedigrees; i++) {
        printf("\nMarker Data for Pedigree #%d",i+1);
        printf("\n\n");
        printf("PersonID  Marker A  Marker B");
        printf("\n");
        printf("--------  --------  --------");
        for(j=0; j < pedigreesize[i]; j++) {
          printf("\n");
          printf("%8d  %8d  %8d",
                 pedigree[i][j][PID],
                 marker[i][j][nmarker][MA],marker[i][j][nmarker][MB]);
        }
      }
      printf("\n\n");
    }
    printf("There are %d QTLs\n\n",nQTL);
    for(qtl=0; qtl<nQTL; qtl++){
      printf("QTL %d\n",qtl+1);
      for(i=0; i < npedigrees; i++) {
        printf("\n\n");
        printf("QTL Data for Pedigree #%d",i+1);
        printf("\n\n");
        printf("PersonID  Phenotype");
        printf("\n");
        printf("--------  ---------\n");
        for(j=0; j < pedigreesize[i]; j++)
	  if(qpheno[i][j][qtl] == missingQT)
            printf("%8d  missing\n",pedigree[i][j][PID]);
          else
            printf("%8d  %lf\n",pedigree[i][j][PID], qpheno[i][j][qtl]);
      }
    }
  }
}
