/* ********************************************************************

   Program:  NPL (Non-Parametric Linkage)
   File:     ReadSIMPLE.c
   Author:   Zachary Skrivanek
   Date:     March 23, 2000
   Purpose:  read in the IBD data produced by SIMPLE. 

********************************************************************* */
#include <stdio.h> /* fscanf */
#include <stdlib.h> /* exit */
#include "simple.h"

#define  FOUNDER 0
#define  F 0
#define  M 1
#define  PID 0
#define  TRUE 1

void ReadSIMPLE(int    ***IBDsim,
              double *weight,
	      int    nweight,
	      double **likrats,
	      int    nmarkers,
	      int    *numlik,
	      int    ***ancestry,
	      int    *pedigreesize,
	      int    npedigrees,
              cmd    c,
	      int    iter)

{
  int    i,j,tempint,dum=0;

  char   line[100]; 

 
  /**************************
  ** read in SIMPLE output **
  **************************/

  scanf("%d",&tempint);
  for(i=0; i<nweight; i++)
    if(scanf("%le",&weight[i])!=1) {
      fprintf(stderr,"line %d iter %d\n",iter+1,tempint);
      nerror("Expecting a double for weight");
    }
  if((strcmp(c.analysis.par,"BOTH")==0) ||
     (strcmp(c.analysis.par,"NPL")==0) ||
     c.hasemanelston.par==TRUE)
  {
    for(i=0; i<npedigrees; i++)
      for(j=0; j<pedigreesize[i]; j++)
        if(!ancestry[i][j][FOUNDER]){ 
	  dum++;
          if(scanf("%d",&IBDsim[i][j][F])!=1) {
	    fprintf(stderr,"line %d iter %d nonfounder # %d \n",iter+1,tempint,dum);
	    /* nerror("expecting an integer for paternal IBD vector");*/
          } 
        }
    dum=0;
    for(i=0; i<npedigrees; i++)
      for(j=0; j<pedigreesize[i]; j++)
        if(!ancestry[i][j][FOUNDER]){ 
	  dum++;
          if(scanf("%d",&IBDsim[i][j][M])!=1){
	    fprintf(stderr,"line %d iter %d nonfounder # %d \n",iter,tempint,dum);
	    /* nerror("expecting an integer for maternal IBD vector"); */
          }
        } 
  }
  /* if(isdisease){ */
  if((strcmp(c.analysis.par,"BOTH")==0) ||
     (strcmp(c.analysis.par,"LOD")==0))
  {
    scanf("%le",&likrats[0][0]); /* null case */
    /* for(i=1; i<nmarkers; i++)
      for(j=0; j<numlik[i+nmarkers]; j++) */
/* assuming we simulate in the model of every interval */
/* and we do reweighting at markers and all interior points */
   if(nmarkers==1)
      scanf("%le",&likrats[0][0]); /* 6/11/02 garbage */
    for(i=1; i<nmarkers; i++)
      for(j=0; j<c.nsteps.par+1; j++) 
        if(scanf("%le",&likrats[i][j])!=1) nerror("Expecting a double for likelihood ratio");
  }
  else fgets(line,100,stdin);

}


void DebugReadSIMPLE(int    ***IBDsim,
                   double *weight,
		   int    nweight,
	           double **likrats,
	           int    nmarkers,
	           int    *numlik,
	           int    ***ancestry,
	           int    ***pedigree,
	           int    *pedigreesize,
	           int    npedigrees,
                   cmd    c)
{
  
  char response[2];
  int  i,j;

  /* printf("\n\nDebug ReadSIMPLE()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0)*/ {

    printf("\n\n");
    printf("weights:");
    for(i=0; i<nweight; i++)
      printf("%le ",weight[i]);
    printf("\n");

    if((strcmp(c.analysis.par,"BOTH")==0) ||
       (strcmp(c.analysis.par,"NPL")==0) ||
	c.hasemanelston.par==TRUE)
    {
      for(i=0; i<npedigrees; i++){
        printf("pedigree:  %d\n",i+1);
        for(j=0; j<pedigreesize[i]; j++)
          if(!ancestry[i][j][FOUNDER]) 
	    printf("PID %d: %d %d \n",pedigree[i][j][PID],IBDsim[i][j][F],IBDsim[i][j][M]);
        printf("\n");
      }
    }
    /* if(isdisease){ */
    if((strcmp(c.analysis.par,"BOTH")==0) ||
       (strcmp(c.analysis.par,"LOD")==0))
    {
      printf("likrats:  ");
      printf("%le ",likrats[0][0]);
      for(i=1; i<nmarkers; i++)
        for(j=0; j<c.nsteps.par+1;j++)
          printf("%le ",likrats[i][j]);
      printf("\n"); 
    } 
  }

}
