/* ********************************************************************

   Program:  NPL (Non-Parametric Linkage)
   File:     SimIBDs.c
   Author:   Zachary Skrivanek
   Date:     March 23, 2000
   Purpose:  Simulate missing IBD data produced by SIMPLE.

********************************************************************* */
#include <stdlib.h> /* exit */
#include <stdio.h> /* printf, scanf */

#define UNKNOWN 2
#define PID 0
#define FOUNDER 0


void SimJointIBDs(int    ***IBDsim,
             int    ***disIV,
             double *prob,
	     int    ***ancestry,
	     int    *pedigreesize,
	     int    npedigrees,
	     int    nloci)

{

  int    i,j,k,l;
  double drnd, dtemp;
  int    IBD,*IV;

  int    power(int, int);
  double rnd();
  void   basen(int, int, int, int *);
  int    unbase(int, int, int *);

  IV = (int *) calloc(nloci, sizeof(int));
  

  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++)
      if(!ancestry[i][j][FOUNDER])
        for(k=0; k<2; k++){
	  IBD=IBDsim[i][j][k] + 2*power(3,nloci-1);
	  basen(nloci,3,IBD,IV);
	  for(l=0; l<nloci; l++)
       	    if(IV[l]==UNKNOWN){
	      drnd=rnd();
	      dtemp=drnd*prob[IBD];
	      if(dtemp < prob[IBD-2*power(3,l)]){
		IV[l]=0;
	        IBD=IBD-2*power(3,l);
              }
              else{
		IV[l]=1;
	        IBD=IBD-power(3,l); 
              }
            }
	  disIV[i][j][k] = IV[nloci-1]; 
	  IBDsim[i][j][k] = IBD - IV[nloci-1]*power(3,nloci-1);
	}

  free(IV);

}


/*
void DebugSimIBDs(int    ***IBDsim,
		  int    ***disIV,
		  int    ***ancestry,
                  int    ***pedigree,
		  int    *pedigreesize,
		  int    npedigrees,
		  int    *locusorder,
		  int    nloci)

{

  char response[2];

  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0){

    int    i,j,k,l;
    int    *IV;

    void   basen(int, int, int, int *);


    IV = (int *) calloc(nloci-1, sizeof(int));

    for(i=0; i<npedigrees; i++)
      for(j=0; j<pedigreesize[i]; j++)
        if(!ancestry[i][j][FOUNDER]){
          printf("PID %d:\n",pedigree[i][j][PID]);
          for(k=0; k<2; k++){
            printf("%d:  ",IBDsim[i][j][k]);
            basen(nloci-1,3,IBDsim[i][j][k],IV);
            for(l=0; l<nloci-1; l++)
	    {
	      if(locusorder[l]==nloci) printf("'%d ", disIV[i][j][k]); 
              printf("%d ",IV[l]);
            }
            printf("\n");
          }
          printf("\n");
        } 
  }
}

*/
