/* ***************************************************************************

   Program:   simple (multi-locus linkage analysis)
   File:      calcblank.c
   Author:    Mark E. Irwin
   Date:      July 26, 1999
   Purpose:   Determines members of the pedigree who don't provide
	      useful information about recombination and can be blanked
	      from the dataset for each locus.

*************************************************************************** */

#include <stdio.h>

#define PID 0
#define FALSE 0
#define TRUE 1

#define M 1
#define F 0

#define FOUNDER 0
#define PARENT  1
#define FINDEX  2
#define MINDEX  3
#define SID     4
#define SINDEX  5
#define SEX     6

void CalcBlank(int npedigrees,
	       int *pedigreesize,
	       int ***ancestry,
	       int nmarkers,
	       int nloci,
               int ****marker,
	       double ****ppotential,
	       int ****blank)

{

  int i,j,k;
  int ***tblank;
  int anyblank,anyblankb,anyblankt;

  tblank = (int ***) calloc(npedigrees,sizeof(int **));
  for(i=0; i<npedigrees; i++) {
    tblank[i] = (int **) calloc(pedigreesize[i],sizeof(int *));
    for(j=0; j<pedigreesize[i]; j++)
      tblank[i][j] = (int *) calloc(nloci,sizeof(int));
  }

  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++)
      for(k=0; k<nloci; k++)
        tblank[i][j][k] = FALSE;

  for(k=0; k<nloci; k++)
    {
      for(i=0; i<npedigrees; i++)
        for(j=0; j<pedigreesize[i]; j++)
          tblank[i][j][k] = FALSE;

      for(i=0; i<npedigrees; i++)
        {
          anyblank = TRUE;
          while(anyblank) /* check from bottom and top */
            {
              anyblankb = TRUE;
              while(anyblankb)  /* check for bottom up blanking */
                {
                  anyblankb = FALSE;
                    for(j=pedigreesize[i]-1; j>=0; j--)
                      {
                        if(!tblank[i][j][k] && 
                             nodata(i,j,k,nmarkers,marker,ppotential))
                          {
                            if(!ancestry[i][j][PARENT])
                              {
                                tblank[i][j][k] = TRUE;
                                anyblankb = TRUE;
                              }
                            else if(checkdescendents(i,j,pedigreesize[i],k,
                                                       tblank,ancestry))
                              {
                                tblank[i][j][k] = TRUE;
                                anyblankb = TRUE;
                              }
                          }
                      }
                } 
              anyblankt = TRUE;
              while(anyblankt)  /* check for top down blanking */
                {
                  anyblankt = FALSE;
/*  Need to modify this to make sure sibs are uniformative as well (Need
    everyone above this person to be uniformative, not just the parents).
    Modification probably not important as top down check probably won't
    pick anyone up as probably only founders would be picked up by this
    and they are effectively blanked anyways as their inheritance
    vectors can never be determined.

                    for(j=0; j<pedigreesize[i]; j++)
                      {
                        if(!tblank[i][j][k] && 
                             nodata(i,j,k,nmarkers,marker,ppotential))
                          {
                            if(ancestry[i][j][FOUNDER])
                              {
                                tblank[i][j][k] = TRUE;
                                anyblankt = TRUE;
                              }
                            else if(tblank[i][ancestry[i][j][FINDEX]][k] &&
                                      tblank[i][ancestry[i][j][MINDEX]][k])
                              {
                                tblank[i][j][k] = TRUE;
                                anyblankt = TRUE;
                              }
                          }
                      }
*/
                }
              anyblank = anyblankb || anyblankt;
            }
        }
    }
    *blank = tblank;
  }

int nodata(int pednum,
           int pindex,
           int locus,
           int nmarkers,
           int ****marker,
           double ****ppotential)

{
  int temp1, temp2;

  if(locus<nmarkers)  /* locus of interest is a marker locus */
    return((marker[pednum][pindex][locus][F] == 0) &&
                (marker[pednum][pindex][locus][M] == 0));
  else if (locus == nmarkers) /* locus of interest is a disease locus */
    {
      temp1 = (ppotential[pednum][pindex][locus][0] ==
                  ppotential[pednum][pindex][locus][1]);
      temp2 = (ppotential[pednum][pindex][locus][1] ==
                  ppotential[pednum][pindex][locus][3]);
      return( temp1 && temp2 );
    }
}

int checkdescendents(int pednum,
                     int pindex,
                     int pedsize,
                     int locus,
                     int ***blank,
		     int ***ancestry)

{
  int j, tempblank;

  tempblank = TRUE;
  if(ancestry[pednum][pindex][SEX])  /* subject is male */
    {
      for(j=0; j<pedsize; j++)
        if(!ancestry[pednum][j][FOUNDER])
          if(ancestry[pednum][j][FINDEX] == pindex)   /* found offspring */
            if(!blank[pednum][j][locus])       /* offspring isn't blanked */
              tempblank = FALSE;
    }
  else  /* subject is female */
    {
      for(j=0; j<pedsize; j++)
        if(!ancestry[pednum][j][FOUNDER])
          if(ancestry[pednum][j][MINDEX] == pindex)   /* found offspring */
            if(!blank[pednum][j][locus])        /* offspring isn't blanked */
              tempblank = FALSE;
    }
  return(tempblank);
}

void DebugCalcBlank(int npedigrees,
                    int *pedigreesize,
                    int ***pedigree,
                    int nloci,
                    int ***blank)
{
  char response[2];

  printf("\n\nDebug CalcBlank()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {

    int i,j,k;

    printf("\n\n");
    for(k=0; k<nloci; k++) {
      printf("People not to have ibds simulated for locus %d\n",k+1);
      printf("\nPedigree PersonID SimStatus\n");
      printf("------- -------- ---------\n\n");
      for(i=0; i<npedigrees; i++) {
        for(j=0; j<pedigreesize[i]; j++)
          printf("%6d   %6d   %6d\n",i+1,pedigree[i][j][PID],
                                                blank[i][j][k]);
        printf("\n");
      }
    }
  }
}

