/* *****************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      calcibdprobs.c
   Author:    Mark E. Irwin
   Date:      Aug 5, 1992
   Purpose:   Given a set of recombination fraction, calculates the
	      joint distribution of the ibds in the non-interference case

*************************************************************************** */
#include <stdio.h>

void Calcibdprobs(int nloci,
		  int nmodels,
		  int **locusorder,
		  double **theta,
		  double ***prob,
		  int *numlik,
		  double ***liktheta,
		  double ****likprob)
{
  void basen();
  int unbase();
  int power();
  double recprob();
  double reclikprob();

  int i,j,k,l,m;
  int *c2;
  double **tprob;
  double ***tlikprob;

  c2 = (int *) calloc(nloci,sizeof(int));
  tprob = (double **) calloc(nmodels,sizeof(double *));
  tlikprob = (double ***) calloc(nmodels,sizeof(double **));
  for(i=0; i<nmodels; i++) {
    tprob[i] = (double *) calloc(power(3,nloci),sizeof(double));
    tlikprob[i] = (double **) calloc(numlik[i],sizeof(double *));
    for(j=0; j<numlik[i]; j++)
      tlikprob[i][j] = (double *) calloc(power(3,nloci),sizeof(double));
  }

  for(k=0; k<nmodels; k++)
    for(i=0; i<power(2,nloci); i++) {
      basen(nloci,2,i,c2);
      m = unbase(nloci,3,c2);
      tprob[k][m] = 0.5;
      for(l=0; l<numlik[k]; l++)
        tlikprob[k][l][m] = 0.5;
      for(j=1; j<nloci; j++) {
        tprob[k][m] *= recprob(j,c2[locusorder[k][j-1]-1],
					c2[locusorder[k][j]-1],k,theta);
        for(l=0; l<numlik[k]; l++)
          tlikprob[k][l][m] *= reclikprob(j,c2[locusorder[k][j-1]-1],
					c2[locusorder[k][j]-1],k,l,liktheta);
  
      }
    }
  *prob = tprob;
  *likprob = tlikprob;
}

double recprob(int t,
	  int a,
	  int b,
	  int model,
	  double **theta)
{
  if (a==b) return(1.0 - theta[model][t-1]);
  else return(theta[model][t-1]);
}

double reclikprob(int t,
	  	  int a,
	  	  int b,
	 	  int model,
		  int sub,
		  double ***liktheta)
{
  if (a==b) return(1.0 - liktheta[model][sub][t-1]);
  else return(liktheta[model][sub][t-1]);
}
