/* ***************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      calcmarginprobs.c
   Author:    Mark E. Irwin
   Date:      July 17, 1992
   Purpose:   Calculates all marginal probabilities of the ibds given
	      the joint distribution.

*************************************************************************** */

#include<stdio.h>

void Calcmarginprobs(int nloci,
		     int nmodels,
		     double **prob,
		     int *numlik,
		     double ***likprob)
{
  void basen();
  int firstm();
  int power();

  int i,j,count,t;
  int *c3;

  c3 = (int *) calloc(nloci,sizeof(int));

  for(count=2;count<power(3,nloci);count++) {
    basen(nloci,3,count,c3);
    t = firstm(nloci,2,c3);
    if (t<nloci)
      for(i=0; i<nmodels; i++) {
	prob[i][count] = prob[i][count-2*power(3,t)]+prob[i][count-power(3,t)];
        for(j=0; j<numlik[i]; j++)
          likprob[i][j][count] = likprob[i][j][count-2*power(3,t)]+
					likprob[i][j][count-power(3,t)];
      }
  }
}

void DebugCalcmarginprobs(int nloci,
			  int nmodels,
			  double **prob,
			  int *numlik,
			  double ***likprob)
{
  char response[2];

  printf("\n\nDebug Calcmarginprobs ()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) {

    int i;

    printf("\n\n");
    for(i=0; i<nmodels; i++) 
    printf("Model %d: Sum of P[Z1,...,Znloci] = %le\n",
		i,prob[i][power(3,nloci)-1]);
  }
}

void DebugProbtable(int nloci,
		    int nmodels,
                    double **prob,
		    int *numlik,
		    double ***likprob)
{
  char response[2];

  printf("\n\nDebug Probtable ()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) {

    void basen();
    int power();

    int i,j,k;
    int *c3;

    c3 = (int *) calloc(nloci,sizeof(int));

    printf("\n\n");

    for(k=0; k<nmodels; k++) {
      printf("Model %d\n\n",k);
      for(i=0; i<power(3,nloci); i++) {
        printf("%5d",i);
        basen(nloci,3,i,c3);
        for(j=0; j<nloci; j++)
          printf(" %d",c3[j]);
        printf(" %8.6le",prob[k][i]);
        for(j=0; j<numlik[k]; j++)
          printf(" %8.6le",likprob[k][j][i]);
        printf("\n");
      }
    }

  }
}

