/* ***************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      calcpeelingorder.c
   Author:    Mark E. Irwin
   Date:      August 3, 1992
   Purpose:   Determines a peeling order for pedigreees with no loops.
	      This is Mike Frigge's algorithm.
   Modified:  August 4, 2002.  Don't do type III for a person whose
	      spouse has unpeeled parents.  If caught in the "npeeled"
	      loop, break out of it with an error message.

*************************************************************************** */

#include <stdio.h>

#define PID 0
#define FID 1
#define MID 2

#define TRUE 1
#define FALSE 0

#define FOUNDER 0
#define PARENT  1
#define FINDEX  2
#define MINDEX  3
#define SID     4
#define SINDEX  5
#define SEX     6

void CalcPeelingOrder(int npedigrees,
		      int *pedigreesize,
		      int ***ancestry,
		      int ***peelorder,
		      int ***peeltype,
		      int ***peelinto)
{
  int   i;
  int   j;
  int   k;
  int   anypeels;
  int   findex;
  int   mindex;
  int   npeeled,tnpeeled;
  int   sid;
  int **tpeelorder;
  int **tpeeltype;
  int **tpeelinto;

  void  nerror();

  tpeelorder = (int **) calloc(npedigrees,sizeof(int *));
  tpeeltype  = (int **) calloc(npedigrees,sizeof(int *));
  tpeelinto     = (int **) calloc(npedigrees,sizeof(int *));
  for(i=0; i < npedigrees; i++) {
    tpeelorder[i] = (int *) calloc(pedigreesize[i],sizeof(int));
    tpeeltype[i]  = (int *) calloc(pedigreesize[i],sizeof(int));
    tpeelinto[i]     = (int *) calloc(pedigreesize[i],sizeof(int));
  }

  for(i=0; i < npedigrees; i++) {
    npeeled = 0;
    while(npeeled < pedigreesize[i]-1) {
      anypeels = TRUE;
      tnpeeled=npeeled;
      while(anypeels) {
	anypeels = FALSE;
	for(j=pedigreesize[i]-1; j >= 0; j--) {
	  if(tpeelorder[i][j] == 0) 
{
	    if(Type1(i,j,pedigreesize[i],ancestry,tpeelorder)) {
	      npeeled++;
	      tpeelorder[i][j] = npeeled;
	      tpeeltype[i][j]  = 1;
	      anypeels = TRUE;
	    }
}
	}
	for(j=pedigreesize[i]-1; j >= 0; j--) {
	  if(tpeelorder[i][j] == 0) 
{
	    if(Type2(i,j,pedigreesize[i],ancestry,tpeelorder)) {
	      npeeled++;
	      tpeelorder[i][j] = npeeled;
	      tpeeltype[i][j]  = 2;
	      anypeels = TRUE;
	    }
}
	}
      }
      anypeels = TRUE;
      while(anypeels) {
	anypeels = FALSE;
	for(j=pedigreesize[i]-1; j >= 0; j--) {
	  if(tpeelorder[i][j] == 0) 
{
	    if(Type3(i,j,pedigreesize[i],ancestry,tpeelorder)) {
	      npeeled++;
	      tpeelorder[i][j] = npeeled;
	      tpeeltype[i][j]  = 3;
	      anypeels = TRUE;
	    }
}
	}
	for(j=pedigreesize[i]-1; j >= 0; j--) {
	  if(tpeelorder[i][j] == 0) 
{
	    if(Type4(i,j,pedigreesize[i],ancestry,tpeelorder)) {
	      npeeled++;
	      tpeelorder[i][j] = npeeled;
	      tpeeltype[i][j]  = 4;
	      anypeels = TRUE;
	    }
}
	}
      }  
      if(tnpeeled==npeeled) nerror("Can not find valid peeling order!"); 
    }
  }

  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++)
      if( tpeeltype[i][j] == 0 ) {
	tpeelorder[i][j] = pedigreesize[i];
	break;
      }
	
  for(i=0; i < npedigrees; i++) 
    for(j=0; j < pedigreesize[i]; j++) 
      switch( tpeeltype[i][j] ) {
      case 0:
	tpeelinto[i][j] = pedigreesize[i];
	break;
      case 1:
	if( tpeelorder[i][ancestry[i][j][FINDEX]] < 
	    tpeelorder[i][ancestry[i][j][MINDEX]] ) {
	  tpeelinto[i][j] = ancestry[i][j][FINDEX];
	} else {
	  tpeelinto[i][j] = ancestry[i][j][MINDEX];
	}
	break;
      case 2:
      case 3:
                 /* Find spouse peeled later */
        if(ancestry[i][j][SEX]) {
          for(k=0; k < pedigreesize[i]; k++)
            if(!ancestry[i][k][FOUNDER])
              if((ancestry[i][k][FINDEX] == j) &&
                 (tpeelorder[i][ancestry[i][k][MINDEX]] > tpeelorder[i][j])) {
                tpeelinto[i][j] = ancestry[i][k][MINDEX];
                break;
              }
          }
        else {
          for(k=0; k < pedigreesize[i]; k++)
            if(!ancestry[i][k][FOUNDER])
              if((ancestry[i][k][MINDEX] == j) &&
                 (tpeelorder[i][ancestry[i][k][FINDEX]] > tpeelorder[i][j])) {
                tpeelinto[i][j] = ancestry[i][k][FINDEX];
                break;
              }
          }
	break;
      case 4:
	if( ancestry[i][j][SEX] ) {
	  for(k=0; k < pedigreesize[i]; k++) 
	    if( (ancestry[i][k][FINDEX] == j) &&
	        (tpeelorder[i][k] > tpeelorder[i][j]) ) {
	      tpeelinto[i][j] = k;
	      break;
	    }
	} else {
	  for(k=0; k < pedigreesize[i]; k++) 
	    if( (ancestry[i][k][MINDEX] == j) && 
	        (tpeelorder[i][k] > tpeelorder[i][j]) ) {
	      tpeelinto[i][j] = k;
	      break;
	    }
	}
	break;
      }
	
	
  *peelorder = tpeelorder;
  *peeltype  = tpeeltype;
  *peelinto  = tpeelinto;

}

/* Peels child into parents */

int Type1(int pnum,
	  int pindex,
	  int pedigreesize,
	  int ***ancestry,
	  int **peelorder)
{
  int i;
  int spouseunpeeled();

  /* is person founder */
  if( ancestry[pnum][pindex][FOUNDER] )
    return(FALSE);

  /* is father peeled */
  if( peelorder[pnum][ancestry[pnum][pindex][FINDEX]] != 0 )
    return(FALSE);

  /* is mother peeled */
  if( peelorder[pnum][ancestry[pnum][pindex][MINDEX]] != 0 )
    return(FALSE);

  /* does person have no children (bottom of pedigree branch) */
  if( !ancestry[pnum][pindex][PARENT] ) 
    return(TRUE);

  /* is there an unpeeled spouse */
  if(spouseunpeeled(pnum,pindex,pedigreesize,ancestry,peelorder))
    return(FALSE);

  /* is there an unpeeled child */
  for(i=0; i < pedigreesize; i++)
    if ( !ancestry[pnum][i][FOUNDER] )
      if( ancestry[pnum][pindex][SEX] ) {
	if( (ancestry[pnum][i][FINDEX] == pindex) &&
	    (peelorder[pnum][i] == 0) )
	  return(FALSE);
      } else {
	if( (ancestry[pnum][i][MINDEX] == pindex) &&
	    (peelorder[pnum][i] == 0) )
	  return(FALSE);
      }
  
  return(TRUE);
}

/* Peels spouse into spouse (all children already peeled) */

int Type2(int pnum,
	  int pindex,
	  int pedigreesize,
	  int ***ancestry,
	  int **peelorder)
{
  int i;
  int allspousepeeled();
  int numspouseunpeeled();
  

  /* does person have no children (bottom of pedigree branch) */
  if( !ancestry[pnum][pindex][PARENT] )
    return(FALSE);

  /* are all spouses peeled */
  if(allspousepeeled(pnum,pindex,pedigreesize,ancestry,peelorder))
    return(FALSE);

  /* are parents peeled */
  if(!ancestry[pnum][pindex][FOUNDER]) {
    if(peelorder[pnum][ancestry[pnum][pindex][FINDEX]] == 0)
      return(FALSE);

    if(peelorder[pnum][ancestry[pnum][pindex][MINDEX]] == 0) 
      return(FALSE);
  }

  /* is there an unpeeled child */
  for(i=0; i < pedigreesize; i++)
    if ( !ancestry[pnum][i][FOUNDER] )
      if( ancestry[pnum][pindex][SEX] ) {
	if( (ancestry[pnum][i][FINDEX] == pindex) &&
	    (peelorder[pnum][i] == 0) )
	  return(FALSE);
      } else {
	if( (ancestry[pnum][i][MINDEX] == pindex) &&
	    (peelorder[pnum][i] == 0) )
	  return(FALSE);
      }

 /* is there exactly 1 unpeeled spouse */
  if(numspouseunpeeled(pnum,pindex,pedigreesize,ancestry,peelorder) == 1)
    return(TRUE);
  else
    return(FALSE);
}

/* peels spouse into spouse and child */

int Type3(int pnum,
	  int pindex,
	  int pedigreesize,
	  int ***ancestry,
	  int **peelorder)
{
  int i;
  int noffspring;
  int allspousepeeled();
  int numspouseunpeeled();
  int GetUnpeeledSpouseIdx();
  
  /* does person have no children (bottom of pedigree branch) */
  if( !ancestry[pnum][pindex][PARENT] )
    return(FALSE);

  /* does person have unpeeled parent */
  if( !ancestry[pnum][pindex][FOUNDER] ) {
    if(peelorder[pnum][ancestry[pnum][pindex][FINDEX]] == 0) 
      return(FALSE);
    if(peelorder[pnum][ancestry[pnum][pindex][MINDEX]] == 0) 
      return(FALSE);
  }

  /* are all spouses peeled */
  if(allspousepeeled(pnum,pindex,pedigreesize,ancestry,peelorder))
    return(FALSE);

  /* get number of unpeeled children */
  noffspring = 0;
  for(i=0; i < pedigreesize; i++)
    if( !ancestry[pnum][i][FOUNDER] ) 
      if( ancestry[pnum][pindex][SEX] ) {
	if( ancestry[pnum][i][FINDEX] == pindex )
	  if( peelorder[pnum][i] == 0 )
	    noffspring++;
      } else {
	if( ancestry[pnum][i][MINDEX] == pindex )
	  if( peelorder[pnum][i] == 0 )
	    noffspring++;
      }

  /* is there exactly 1 unpeeled spouse and 1 unpeeled offspring */
  if((noffspring == 1) &&
       (numspouseunpeeled(pnum,pindex,pedigreesize,ancestry,peelorder)== 1)){
    i=GetUnpeeledSpouseIdx(pnum,pindex,pedigreesize,ancestry,peelorder);
    /* does spouse have unpeeled parent */
    if( !ancestry[pnum][i][FOUNDER] ) {
      if(peelorder[pnum][ancestry[pnum][i][FINDEX]] == 0) 
        return(FALSE);
      if(peelorder[pnum][ancestry[pnum][i][MINDEX]] == 0) 
        return(FALSE);
    }
    return(TRUE);
  }
  else
    return(FALSE);

}

/* peels parent into child */

int Type4(int pnum,
	  int pindex,
	  int pedigreesize,
	  int ***ancestry,
	  int **peelorder)
{
  int i;
  int noffspring;
  int spouseunpeeled();
  

  /* does person have no children (bottom of pedigree branch) */
  if( !ancestry[pnum][pindex][PARENT] )
    return(FALSE);

  /* is there an unpeeled parent */
  if( !ancestry[pnum][pindex][FOUNDER] ) {
    if(peelorder[pnum][ancestry[pnum][pindex][FINDEX]] == 0)
      return(FALSE);
    if(peelorder[pnum][ancestry[pnum][pindex][MINDEX]] == 0)
      return(FALSE);
  }

  /* is there an unpeeled spouse */
  if(spouseunpeeled(pnum,pindex,pedigreesize,ancestry,peelorder))
    return(FALSE);

  /* get number of unpeeled children */
  noffspring = 0;
  for(i=0; i < pedigreesize; i++)
    if( !ancestry[pnum][i][FOUNDER] ) 
      if( ancestry[pnum][pindex][SEX] ) {
	if( ancestry[pnum][i][FINDEX] == pindex )
	  if( peelorder[pnum][i] == 0 )
	    noffspring++;
      } else {
	if( ancestry[pnum][i][MINDEX] == pindex )
	  if( peelorder[pnum][i] == 0 )
	    noffspring++;
      }

  if(noffspring == 1)
    return(TRUE);
  else
    return(FALSE);

}

int allspousepeeled(int pednum,
		    int pindex,
		    int pedigreesize,
		    int ***ancestry,
		    int **peelorder)

{
  int i,j;
  int peeled;

  peeled = TRUE;

  if(ancestry[pednum][pindex][SEX]) /* pindex is a man */
    {
      for(j=0; j<pedigreesize; j++)
        {
          if(!ancestry[pednum][j][FOUNDER])
            if(ancestry[pednum][j][FINDEX] == pindex)
              if(peelorder[pednum][ancestry[pednum][j][MINDEX]] == 0)
                peeled = FALSE;
        }
    }
  else
    {
      for(j=0; j<pedigreesize; j++)
        {
          if(!ancestry[pednum][j][FOUNDER])
            if(ancestry[pednum][j][MINDEX] == pindex)
              if(peelorder[pednum][ancestry[pednum][j][FINDEX]] == 0)
                peeled = FALSE;
        }
    }
  return(peeled);
}

int spouseunpeeled(int pednum,
                   int pindex,
                   int pedigreesize,
                   int ***ancestry,
                   int **peelorder)

{
  int j;
  int unpeeled;

  unpeeled = FALSE;

  if(ancestry[pednum][pindex][SEX]) /* pindex is a man */
    {
      for(j=0; j<pedigreesize; j++)
        {
          if(!ancestry[pednum][j][FOUNDER])
            if(ancestry[pednum][j][FINDEX] == pindex)
              if(peelorder[pednum][ancestry[pednum][j][MINDEX]] == 0)
                unpeeled = TRUE;
        }
    }
  else
    {
      for(j=0; j<pedigreesize; j++)
        {
          if(!ancestry[pednum][j][FOUNDER])
            if(ancestry[pednum][j][MINDEX] == pindex)
              if(peelorder[pednum][ancestry[pednum][j][FINDEX]] == 0)
                unpeeled = TRUE;
        }
    }
  return(unpeeled);
}

int numspouseunpeeled(int pednum,
                      int pindex,
                      int pedigreesize,
                      int ***ancestry,
                      int **peelorder)

/* determines whether number of unpeeled spouses is 0, 1, or 2 or more */

{

  int j;
  int firstspouse;
  int startpoint;
  int numspouse;

  numspouse = 0;
  if(ancestry[pednum][pindex][SEX]) /* pindex is a man */
    {
      for(j=0; j<pedigreesize; j++)
        {
          if(!ancestry[pednum][j][FOUNDER])
            if(ancestry[pednum][j][FINDEX] == pindex)
              if(peelorder[pednum][ancestry[pednum][j][MINDEX]] == 0)
                {
                  firstspouse = ancestry[pednum][j][MINDEX];
                  numspouse++;
	          startpoint = j+1;
                  break;
                }
        }
      for(j=startpoint; j<pedigreesize; j++)
        {
          if(!ancestry[pednum][j][FOUNDER])
            if(ancestry[pednum][j][FINDEX] == pindex)
              if(peelorder[pednum][ancestry[pednum][j][MINDEX]] == 0)
                {
                  if(ancestry[pednum][j][MINDEX] != firstspouse)
                    {
	  	      numspouse++;
                      break;
                    }
                }
        }
    }
  else /* pindex is a woman */
    {
      for(j=0; j<pedigreesize; j++)
        {
          if(!ancestry[pednum][j][FOUNDER])
            if(ancestry[pednum][j][MINDEX] == pindex)
              if(peelorder[pednum][ancestry[pednum][j][FINDEX]] == 0)
                {
                  firstspouse = ancestry[pednum][j][FINDEX];
                  numspouse++;
	          startpoint = j+1;
                  break;
                }
        }
      for(j=startpoint; j<pedigreesize; j++)
        {
          if(ancestry[pednum][j][MINDEX] == pindex)
            if(!ancestry[pednum][j][FOUNDER])
              if(peelorder[pednum][ancestry[pednum][j][FINDEX]] == 0)
                {
                  if(ancestry[pednum][j][FINDEX] != firstspouse)
                    {
	  	      numspouse++;
                      break;
                    }
                }
        }
    }
  return(numspouse);
}


int GetUnpeeledSpouseIdx(int pednum,
                         int pindex,
                         int pedigreesize,
                         int ***ancestry,
                         int **peelorder)

/* determines index of first unpeeled spouse in pedigree */

{

  int j;

  if(ancestry[pednum][pindex][SEX]) { /* pindex is a man */
    for(j=0; j<pedigreesize; j++)
      if(!ancestry[pednum][j][FOUNDER])
        if(ancestry[pednum][j][FINDEX] == pindex)
          if(peelorder[pednum][ancestry[pednum][j][MINDEX]] == 0)
	    return(ancestry[pednum][j][MINDEX]);
  }
  else /* pindex is a woman */{
    for(j=0; j<pedigreesize; j++)
      if(!ancestry[pednum][j][FOUNDER])
        if(ancestry[pednum][j][MINDEX] == pindex)
          if(peelorder[pednum][ancestry[pednum][j][FINDEX]] == 0)
	    return(ancestry[pednum][j][FINDEX]);
  }
}

void DebugCalcPeelingOrder(int npedigrees,
			   int *pedigreesize,
			   int ***pedigree,
			   int **peelorder,
			   int **peeltype,
			   int **peelinto)
{
  char response[2];

  printf("\n\nDebug DeterminePeelingOrder()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0) {
      
    int i;
    int j;
    int k;

    for(i=0; i < npedigrees; i++) {
      printf("\n\n");
      printf("Pedigree #%d", i+1);
      printf("\n\n");
      printf("Order   PersonID   Type   IntoID");
      printf("\n");
      printf("-----   --------   ----   ------");
      for(j=0; j < pedigreesize[i]-1; j++)
	for(k=0; k < pedigreesize[i]; k++)
	  if( peelorder[i][k] == j+1 )
	    printf("\n%5d   %8d   %4d   %6d",
		   j+1,
		   pedigree[i][k][PID],
		   peeltype[i][k],
		   pedigree[i][peelinto[i][k]][PID]);
      for(j=0; j < pedigreesize[i]; j++)
	if( peelorder[i][j] == pedigreesize[i] )
	  printf("\n%5d   %8d   %4d     ROOT ",
		 pedigreesize[i],
		 pedigree[i][j][PID],
		 peeltype[i][j]);
    }
    printf("\n\n");
  }
}

