/* *********************************************************************

   Program:   SIMPLE
   File:      chs.c
   Author:    Zachary Skrivanek
   Date:      April 25, 1999
   Purpose:   calculates IBD probabilities using chi-square model  
	      modified to deal with NPL on one locus.

********************************************************************* */

#include <stdio.h>

void CHS(int    nloci,
         int    nmodels,
         int    isdisease,
	 int    **locusorder,
         double **distances,
	 int    *numlik,
         double ***likdistances,
         double ***prob,
         double ****likprob,
         int    p)           /* p is the intensity parameter */

{
  int     i,j,k,l,m,n;
  int     max, unlinked;
  int     *c2;
  int	  *r2;
  int     *s2;
  double  *tempp;       /* tempp is used to extract a vector */
  double  **tprob;      /* of fractions from a matrix of fractions */
  double  ***tlikprob;  /* tprob is used to store the vector of probs */

  void   torecomb(int, int *, int *);
  void   tostorageorder(int, int *, int *, int *);
  void   basen();
  int    unbase();

  int    power(int, int);


  /******************* 
  ** allocate space **
  *******************/

  c2 = (int *) calloc(nloci,sizeof(int));
  r2 = (int *) calloc(nloci-1,sizeof(int));
  s2 = (int *) calloc(nloci,sizeof(int));
  tempp = (double *) calloc(power(2,nloci-1),sizeof(double));
  tprob = (double **) calloc(nmodels,sizeof(double *));
  tlikprob = (double ***) calloc(nmodels,sizeof(double **));
  
  for(i=0; i<nmodels; i++) 
  {
    tprob[i] = (double *) calloc(power(3,nloci),sizeof(double));
    tlikprob[i] = (double **) calloc(numlik[i],sizeof(double *));
    for(j=0; j < numlik[i]; j++)
       tlikprob[i][j] = (double *) calloc(power(3,nloci),sizeof(double));
  }

  /****************************
  ** calculate probabilities **
  ****************************/

  /* calculate null probs for disease case */
  if(isdisease)
  {
    basen(nloci,2,power(2,nloci)-1,c2);
    max = unbase(nloci,3,c2);        /* figure out highest index */

    basen(nloci,2,power(2,nloci-1),c2);
    unlinked = unbase(nloci,3,c2);   /* figure out shift */
    recomb_pp(p,&distances[0][0],tempp,nloci-1) ; 
    for(i=0; i<power(2,nloci-2); i++) 
    {
      basen(nloci-1,2,i,c2);
      torecomb(nloci-1,c2,r2);
      n = unbase(nloci-2,2,r2);
      tostorageorder(nloci-1,&locusorder[0][0],c2,s2);
      m = unbase(nloci-1,3,s2);
      tprob[0][m] = tempp[n] / 4.0;  /* divide by 2 so it sums to 1 */
      tprob[0][max-m] = tempp[n] / 4.0; /* divide by 2 for unlinked */ 
      tprob[0][m+unlinked] = tempp[n] / 4.0; 
      tprob[0][max-unlinked-m] = tempp[n] / 4.0;  
    }
    recomb_pp(p,&likdistances[0][0][0],tempp,nloci-1) ; 
    for(i=0; i<power(2,nloci-2); i++) 
    {
      basen(nloci-1,2,i,c2);
      torecomb(nloci-1,c2,r2);
      n = unbase(nloci-2,2,r2);
      tostorageorder(nloci-1,&locusorder[0][0],c2,s2);
      m = unbase(nloci-1,3,s2);
      tlikprob[0][0][m]=tempp[n] / 4.0; /*divide by 2 so it sums to 1 */
      tlikprob[0][0][max-m]=tempp[n] / 4.0;/*divide by 2 for unlinked */
      tlikprob[0][0][m+unlinked] = tempp[n] / 4.0; 
      tlikprob[0][0][max-unlinked-m] = tempp[n] / 4.0;  
    }
  }


  /* calculate the rest of the probs */
  basen(nloci,2,power(2,nloci)-1,c2);
  max = unbase(nloci,3,c2);        /* figure out highest index */
  
  for(k=isdisease; k<nmodels; k++)
  {  
    recomb_pp(p,&distances[k][0],tempp,nloci) ; 
    for(i=0; i<power(2,nloci-1); i++) 
    {
      basen(nloci,2,i,c2);
      torecomb(nloci,c2,r2);
      n = unbase(nloci-1,2,r2);
      tostorageorder(nloci,&locusorder[k][0],c2,s2);
      m = unbase(nloci,3,s2);
      tprob[k][m] = tempp[n] / 2.0;  /* divide by 2 so it sums to 1 */
      tprob[k][max-m] = tempp[n] / 2.0;  
    }
    for(j=0; j<numlik[k]; j++)
    {
      recomb_pp(p,&likdistances[k][j][0],tempp,nloci);
      for(i=0; i<power(2,nloci-1); i++) 
      {
        basen(nloci,2,i,c2);
        torecomb(nloci,c2,r2);
        n = unbase(nloci-1,2,r2);
        tostorageorder(nloci,&locusorder[k][0],c2,s2);
        m = unbase(nloci,3,s2);
        tlikprob[k][j][m] = tempp[n] / 2.0;         
        tlikprob[k][j][max-m] = tempp[n] / 2.0;         
      }
    }  
  }

  /* if we are just doing NPL on one marker */
  /* the chs won't work b/c it only deals with */
  /* recombination patterns.  Here we just need */
  /* to address whether the marker was inherited */
  /* paternally or maternally or unkown */
  if(nloci == 1 && !isdisease){
    tprob[0][0]=0.5;
    tprob[0][1]=0.5;
    tprob[0][2]=1.0;
    tlikprob[0][0][0]=0.5;
    tlikprob[0][0][1]=0.5;
    tlikprob[0][0][2]=1.0;
  }
 
  *prob=tprob;
  *likprob=tlikprob; 
  free(tempp);
}

/* converts inheritance vector to recombination vector */

void torecomb(int nloci,
	      int *c2,
	      int *r2)

{
  int i;
  int abs();

  for(i=0; i<(nloci-1); i++) 
  {
    if (c2[i] == c2[i+1]) r2[i] = 0;
    else r2[i] = 1;
  }
}

/* reorders inheritance vector from physical order to internal storage
order */

void tostorageorder(int nloci,
		    int *order,
		    int *c2,
		    int *s2)

{
  int i;
 
  for(i=0; i<nloci; i++)
    s2[order[i]-1] = c2[i];
}
