/* ***************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      creatediseaseppots.c
   Author:    Mark E. Irwin
   Date:      August 19, 1992
   Purpose:   Updates the disease prepoint potentials with 
	      allele frequency information.
   Modified:  1/27/02- subtract one from haplabel for disease
	      since we changed allele labels form 0-1 to 1-2

*************************************************************************** */

#include <stdio.h>

#define F 0
#define M 1

#define PID 0

#define FOUNDER 0

void CreateDiseasePPots(int npedigrees,
		        int *pedigreesize,
		        int ***ancestry,
		        int *****haplabel,
			int nloci,
		        double *disaf,
		        double ****ppotential)

{
  int   i;
  int   j;
  int   k;

  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++)
      if( ancestry[i][j][FOUNDER] )
        for(k=0; k<4; k++)
	  ppotential[i][j][nloci-1][k] *=
	    (disaf[haplabel[i][j][nloci-1][k][F]-1] *
     	     disaf[haplabel[i][j][nloci-1][k][M]-1]);
}
 

void DebugCreateDiseasePPots(int npedigrees,
			     int *pedigreesize,
			     int ***pedigree,
			     int *****haplabel,
			     int nloci,
			     double ****ppotential)
{
  char response[2];

  printf("\n\nDebug CreateDiseasePPots()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {
    
    int i;
    int j;
    int k;

    printf("\n\n");
    printf("Point Potentials for Disease Locus");
    printf("\n\n");
    for(i=0; i < npedigrees; i++) {
      printf("       Pedigree #%d",i+1);
      printf("\n\n");
      printf("PersonID  Haplotype  h(haplotype)");
      printf("\n");
      printf("--------  ---------  ------------");
      for(j=0; j < pedigreesize[i]; j++) {
	for(k=0; k<4; k++)
	  printf("\n%8d      %1d  %1d    %11.5le",
	          pedigree[i][j][PID],
		  haplabel[i][j][nloci-1][k][F],haplabel[i][j][nloci-1][k][M],
		  ppotential[i][j][nloci-1][k]);
	printf("\n");
      }
    }
    printf("\n\n");
  }
  printf("\n\n");
}
