/* ********************************************************************

    Program:   simple 
    File:      createlikdistances	 
    Author:    Zachary Skrivanek
    Date:      August 22, 1999
    Purpose:   creates array of distances for simulation

********************************************************************* */

#include <stdlib.h>

void  CreateDistances(int     nloci, 
		      int     nmodels,
		      int     isdisease,
                      int     nsteps,
		      double  *mdistances,
		      int     ***locusorder,
		      double  ***distances,
		      int     **numlik,
		      double  ****likdistances)

{

  int    i,j,k,l;
  int    **tlocusorder;
  int    *tnumlik;
  int    nmarkers;
  double **tdistances;
  double ***tlikdistances;

  nsteps -= 1;  /* to conform to GH */

  nmarkers=nloci-isdisease;
/*******************
** allocate space **
*******************/

  tlocusorder   = (int **) calloc(nmodels,sizeof(int *));
  tnumlik       = (int *) calloc(nmodels,sizeof(int));       
  tdistances    = (double **) calloc(nmodels,sizeof(double *));
  tlikdistances = (double ***) calloc(nmodels,sizeof(double **));


/* if there is just one marker then we have to make the appropriate
 modificantions, that is way there are 'if' statements with nloci==2*/
/* if there is no disease */      
/* tnumlik[0] = 0; */
  for(i=0; i < nmodels; i++)
  {
  /* we give the null case xtra space in case we want to
  convert to rec fractions and use mark's algorithm to 
  calculate ibd probs in which case we tack on a rec. frac
  of .5 for the null disease case (independent) */
  tlocusorder[i]   = (int *) calloc(nloci,sizeof(int));
  tdistances[i]    = (double *) calloc(nloci-1,sizeof(double));
  /* if(isdisease && i==0) tnumlik[i] = 1; */ 
    if(i==0 || nmarkers==1) tnumlik[i] = 1; 
  /* else if(isdisease) tnumlik[i] = (nsteps + 2); */
    else tnumlik[i] = (nsteps + 2);
    tlikdistances[i] = (double **) calloc(tnumlik[i],sizeof(double *));
    for(j=0; j < tnumlik[i] ; j++)
      tlikdistances[i][j] = (double *) calloc(nloci-1,sizeof(double));
  }

  /***********************
  ** create tlocusorder ** 
  ***********************/

 /* for(j=0; j < (nloci-1); j++)
    tlocusorder[0][j] = (j+1); */
 /*  for(j=0; j < (nloci-isdisease); j++)
       tlocusorder[0][j] = (j+1);*/
 /* I edited the above out on 4/13/00 since we need a locusorder
for the disease even though we don't need it for the ibdprob calcs
we do need it later for dumping the ibd vectors */
  for(j=0; j < nloci; j++)
    tlocusorder[0][j] = (j+1);

  for(i=1; i<nmodels; i++)
    for(j=0; j < nloci; j++)
    {
      if(j<i) l=j;
      else if(j==i) {tlocusorder[i][j]=nloci;  continue;}
      else l=(j-1);
      tlocusorder[i][j]=(l+1);
    }

  /********************************
  ** take care of the null model **
  *********************************/
  
  /* this is everything if there is no disease */
  for(i=0; i<(nmarkers-1); i++)
  {
    tdistances[0][i] = mdistances[i];
  /* if(isdisease) tlikdistances[0][0][i] = mdistances[i]; */
    tlikdistances[0][0][i] = mdistances[i];
  }

  /**************************************
  ** create distances for disease case **
  **************************************/
  
  if(nmarkers > 1)  
  {
    for(i=1; i < nmodels; i++)
    {
    /* create tdistances */
      for(j=0; j < (nloci - 1); j++)
      {
        if(j < (i-1) ) l=j;
        else if(j == (i-1) ) 
        {
	  tdistances[i][j]   = mdistances[j] * 0.5;
	  tdistances[i][j+1] = mdistances[j] * 0.5;
	  continue;
        }
        else l=(j+1);
        tdistances[i][l] = mdistances[j];
      }
    /* create tlikdistances */
      for(j=0; j < tnumlik[i]; j++)
        for(k=0; k< (nloci - 2); k++)
        {
          if(k < (i-1) ) l=k;
	  else if(k == (i-1) )
	  {
	    tlikdistances[i][j][k]=(mdistances[k] *  
	      ( (double) (j%tnumlik[i]) / ( (double) (tnumlik[i]-1) )));
	    tlikdistances[i][j][k+1]=(mdistances[k] -
	      tlikdistances[i][j][k]);
            continue;
          }
	  else l=(k+1);
	  tlikdistances[i][j][l]=mdistances[k];
        }
    }
  }
  *locusorder   = tlocusorder;
  *distances    = tdistances;
  *numlik       = tnumlik;
  *likdistances = tlikdistances;
}


void DebugCreateDistances(int     nloci,
		          int     nmodels,
		          int     isdisease,
		          int     **locusorder,
		          double  **distances,
		          int     *numlik,
		          double  ***likdistances)
{
  char response[2],c;
/* this does not debug correctly if there is a diseas and just
one locus since we only have 1 model for one marker and in a sense
we really have two models but in the locus case we are simulating right 
on top of  it */
  printf("\n\nDebug CreateDistances()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0) 
  {

    int    i,j,k;
    int    start,count;
    div_t  nloops;

    /* calculate number of rows needed to print out distances */
    nloops = div(nloci, 7);

    for(i=0; i<nmodels; i++) 
    {
      printf("\n\n");
      printf("\n    s\n    u\n    b\nm   m\no   o\nd   d\ne   e\n");
      printf("l   l    Locus Order/ Genetic Distances\n");
      printf("*** *** ******************************************"); 
      printf("***********************\n\n");
      /* print locus order */
      /* start loop is used so we don't print off screen */
      for(start=0; start <= nloops.quot; start++) 
      {
	count=0;
	if( (nloci - (start * 7)) != 1)
	{
          for(j=(start * 7); j<nloci ;j++)
	  {
	    if(j==(start * 7)) printf("order:  ");
	    if(i==0 && isdisease && j==(nloci-1))
	      printf("dis");
	    else if(locusorder[i][j]==nloci && isdisease) 
	      printf("dis      ");
            else printf("%-3d      ",locusorder[i][j]);
            count++;
	    if( count%8 == 0 ) break; 
          }
	  printf("\n");

	  /* print underline */
	  count=0;
          for(j=(start * 7); j<(nloci-1);j++ )
	  {
	    if(j==(start * 7)) printf("        ");
            printf("=========");
	    count++;
	    if( count%8 == 0 ) break;
          }
	  printf("\n");
        } /* if statement */
 
        /* print distances */ 
        for(j=(start * 7); j<(nloci-1);)
	{
          if(j==(start * 7)) printf("%-3d     ",i); 
          if(i==0 && isdisease && j==(nloci-2) ) 
	    printf("   infinity");
          else printf("   %6.4lf",distances[i][j]);
	  j++;
	  if( j%7 == 0 )
	  {
	    printf("\n");
	    break;
          }
        }
        if(j==(nloci-1) && j%7 > 0)
	  printf("\n");
      }
      printf("--- ---    ------\n");

      for(k=0; k < numlik[i]; k++) 
      {
        for(start=0; start <= nloops.quot; start++)
        {
          for(j=(start * 7); j<(nloci-1);)
	  {
            if(j==(start * 7)) printf("%-3d %-3d ",i,k); 
            if(i==0 && isdisease && j==(nloci-2)) 
	      printf("   infinity");
            else printf("   %-6.4lf",likdistances[i][k][j]);
	    j++;
	    if( j%7 == 0 )
	    {
	      printf("\n");
	      break;
            }
          }
          if(j==(nloci-1) && j%7 > 0)
	    printf("\n");
        }
      }
      printf("\npress 'return' to continue");
      scanf("%c",&c);
      printf("\n");
    }
  }
}

