/* ***************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      createmarkerppots.c
   Author:    Mark E. Irwin
   Date:      July 29, 1992
   Purpose:   Creates the marker point potentials in preparation for peeling.

*************************************************************************** */

#include <stdio.h>

#define F 0
#define M 1

#define PID 0

#define FOUNDER 0

void CreateMarkerPPots(int npedigrees,
		       int *pedigreesize,
		       int ***ancestry,
		       int nmarkers,
		       int *nalleles,
		       int *****haplabel,
		       int ***spacesize,
		       double **maf,
		       double ****ppotential)

{
  int   i;
  int   j;
  int   k;
  int   nmarker;



  /*******************************/
  /* Create the point potentials */
  /*******************************/
    
  for(nmarker=0; nmarker < nmarkers; nmarker++) {
    for(i=0; i < npedigrees; i++)
      for(j=0; j < pedigreesize[i]; j++)
        for(k=0; k<spacesize[i][j][nmarker]; k++)
	  ppotential[i][j][nmarker][k] = 1.0;

    for(i=0; i < npedigrees; i++)
      for(j=0; j < pedigreesize[i]; j++)
	if( ancestry[i][j][FOUNDER] )
          for(k=0; k<spacesize[i][j][nmarker]; k++)
	    ppotential[i][j][nmarker][k] *=
	      (maf[nmarker][haplabel[i][j][nmarker][k][F]-1] *
	       maf[nmarker][haplabel[i][j][nmarker][k][M]-1]);

  }
}
 

void DebugCreateMarkerPPots(int npedigrees,
			    int *pedigreesize,
			    int ***pedigree,
			    int nmarkers,
			    int *nalleles,
			    int *****haplabel,
			    int ***spacesize,
			    double ****ppotential)
{
  char response[2];

  printf("\n\nDebug CreateMarkerPPots()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {
    
    int i;
    int index;
    int j;
    int k;
    int l;
    int nmarker;

    printf("\n\n");
    for(nmarker=0; nmarker < nmarkers; nmarker++) {
      printf("Point Potentials for Marker #%d",nmarker+1);
      printf("\n\n");
      for(i=0; i < npedigrees; i++) {
	printf("       Pedigree #%d",i+1);
	printf("\n\n");
	printf("PersonID  Haplotype  h(haplotype)");
	printf("\n");
	printf("--------  ---------  ------------");
	for(j=0; j < pedigreesize[i]; j++) {
	  for(k=0; k<spacesize[i][j][nmarker]; k++)
	    printf("\n%8d      %1d  %1d    %11.5le",
		   pedigree[i][j][PID],
		   haplabel[i][j][nmarker][k][F],haplabel[i][j][nmarker][k][M],
		   ppotential[i][j][nmarker][k]);
	  printf("\n");
	}
      }
      printf("\n\n");
    }
    printf("\n\n");
  }
}
