/* **************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      createspaces.c
   Author:    Mark E. Irwin
   Date:      Sept 30, 1992
   Purpose:   Dynamically allocates the storage space needed for peeling 
	      and simulation.

*************************************************************************** */

#include <stdio.h>

#define A 0
#define B 1

#define PID 0

#define UNKNOWN 2

#define TRUE 1

#define FOUNDER 0
#define FINDEX 2
#define MINDEX 3

void CreateSpaces(int npedigrees,
		  int *pedigreesize,
		  int ***ancestry,
		  int *nalleles,
                  int nmarkers,
		  int ****marker,
                  int isdisease,
		  int nloci,
		  int **peeltype,
		  int **peelinto,
                  int ****spacesize,
                  int *****ibdobs,
                  int *****ibdsim,
		  int **ibdF,
		  int **ibdM,
		  int ***jpotflag,
		  double *****ppotential,
		  double *****wppotential,
		  double ******jpotential,
		  double *****simwppotential,
		  double ******simjpotential,
		  int *****simhap,
		  int ****shap,	
		  int **hap)	
{
  int     i;
  int     j;
  int     k;
  int     l;
  int     sizeA;
  int     sizeB;
  int  ***tspacesize;
  int ****tibdobs;
  int ****tibdsim;
  int    *tibdF;
  int    *tibdM;
  int   **tjpotflag;
  int ****tsimhap;
  int  ***tshap;
  int  *thap;
  int mA,mB;
  int fmA,fmB;
  int mmA,mmB;
  int fromF,fromM;
  int findex,mindex;

  double  ****tppotential;
  double  ****twppotential;
  double *****tjpotential;
  double  ****tsimwppotential;
  double *****tsimjpotential;

  /**************************************************/
  /* determine marker and disease allele spacesizes */
  /**************************************************/
  tspacesize = (int ***) calloc(npedigrees,sizeof(int **));
  for(i=0; i < npedigrees; i++) {
    tspacesize[i] = (int **) calloc(pedigreesize[i],sizeof(int *));
    for(j=0; j < pedigreesize[i]; j++) {
      tspacesize[i][j] = (int *) calloc(nloci,sizeof(int));
      for(k=0; k < nmarkers; k++) {
        if( marker[i][j][k][A] == 0 &&
            marker[i][j][k][B] == 0 ) {
          tspacesize[i][j][k] = nalleles[k] * nalleles[k];
        } else if( marker[i][j][k][A] == marker[i][j][k][B] ) {
          tspacesize[i][j][k] = 1;
        } else {
          mA = marker[i][j][k][A];
	  mB = marker[i][j][k][B];
	  if( !ancestry[i][j][FOUNDER]) {
            findex = ancestry[i][j][FINDEX];
            mindex = ancestry[i][j][MINDEX];
            fmA = marker[i][findex][k][A];
            fmB = marker[i][findex][k][B];
            mmA = marker[i][mindex][k][A];
            mmB = marker[i][mindex][k][B];
            orderhaplotypes(j+1,mA,mB,fmA,fmB,mmA,mmB,&fromF,&fromM);
            if (fromF!=0 && fromM!=0) tspacesize[i][j][k] = 1;
            else tspacesize[i][j][k] = 2;
          } 
          else tspacesize[i][j][k] = 2;
        }
      }
      for(k=nmarkers; k<nloci; k++)
        tspacesize[i][j][k] = 4;
    }
  }

  /**************************************/
  /* create ibd spaces                  */
  /**************************************/
  tibdobs = (int ****) calloc(npedigrees,sizeof(int ***));
  tibdsim = (int ****) calloc(npedigrees,sizeof(int ***));
  for(i=0; i < npedigrees; i++) {
    tibdobs[i] = (int ***) calloc(pedigreesize[i],sizeof(int **));
    tibdsim[i] = (int ***) calloc(pedigreesize[i],sizeof(int **));
    for(j=0; j < pedigreesize[i]; j++) {
      tibdobs[i][j] = (int **) calloc(nloci,sizeof(int *));
      tibdsim[i][j] = (int **) calloc(nloci,sizeof(int *));
      for(k=0; k < nloci; k++) {
        tibdobs[i][j][k] = (int *) calloc(2,sizeof(int));
        tibdsim[i][j][k] = (int *) calloc(2,sizeof(int));
        for(l=0; l < 2; l++) {
          tibdobs[i][j][k][l] = UNKNOWN;
          tibdsim[i][j][k][l] = UNKNOWN;
        }
      }
    }
  }

  /**************************************/
  /* create simulation haplotype spaces */
  /**************************************/

  tsimhap = (int ****) calloc(npedigrees,sizeof(int ***));
  tshap = (int ***) calloc(npedigrees,sizeof(int **));
  thap = (int *) calloc(nmarkers,sizeof(int *));
    for(i=0; i<npedigrees; i++) {
      tsimhap[i] = (int ***) calloc(pedigreesize[i],sizeof(int **));
      tshap[i] = (int **) calloc(pedigreesize[i],sizeof(int *));
      for(j=0; j<pedigreesize[i]; j++) {
        tsimhap[i][j] = (int **) calloc(nloci,sizeof(int *));
	tshap[i][j] = (int *) calloc(nloci,sizeof(int));
        for(k=0; k<nloci; k++)
          tsimhap[i][j][k] = (int *) calloc(2,sizeof(int));
      }
    }

  /*****************************************
  *  create working ibd spaces for Peel()  *
  *****************************************/
  tibdF = (int *) calloc(nloci,sizeof(int));
  tibdM = (int *) calloc(nloci,sizeof(int));

  /**************************************/
  /* create point potential spaces      */
  /**************************************/
  tppotential  = (double ****) calloc(npedigrees,sizeof(double ***));
  twppotential = (double ****) calloc(npedigrees,sizeof(double ***));
  tsimwppotential = (double ****) calloc(npedigrees,sizeof(double ***));
  for(i=0; i < npedigrees; i++) {
    tppotential[i]  = (double ***) calloc(pedigreesize[i],sizeof(double **));
    twppotential[i] = (double ***) calloc(pedigreesize[i],sizeof(double **));
    tsimwppotential[i] = (double ***) calloc(pedigreesize[i],sizeof(double **));
    for(j=0; j < pedigreesize[i]; j++) {
      tppotential[i][j]  = (double **) calloc(nloci,sizeof(double *));
      twppotential[i][j] = (double **) calloc(nloci,sizeof(double *));
      tsimwppotential[i][j] = (double **) calloc(nloci,sizeof(double *));
      for(k=0; k < nloci; k++) {
	tppotential[i][j][k]  = (double *) calloc(tspacesize[i][j][k],
						  sizeof(double));
	twppotential[i][j][k] = (double *) calloc(tspacesize[i][j][k],
						  sizeof(double));
	tsimwppotential[i][j][k] = (double *) calloc(tspacesize[i][j][k],
						  sizeof(double));
      }
    }
  }

  /**************************************/
  /* create joint potential indicators  */
  /**************************************/
  tjpotflag = (int **) calloc(npedigrees,sizeof(int *));
  for(i=0; i < npedigrees; i++)
    tjpotflag[i] = (int *) calloc(pedigreesize[i],sizeof(int));
  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++)
      switch( peeltype[i][j] ) {
      case 1:
      case 3:
	tjpotflag[i][peelinto[i][j]] = TRUE;
	break;
      default:
	break;
      }

  /**************************************/
  /* create joint potential spaces      */
  /**************************************/
  tjpotential = (double *****) calloc(npedigrees,sizeof(double ****));
  tsimjpotential = (double *****) calloc(npedigrees,sizeof(double ****));
  for(i=0; i < npedigrees; i++) {
    tjpotential[i] = (double ****) calloc(pedigreesize[i],sizeof(double ***));
    tsimjpotential[i] =(double ****) calloc(pedigreesize[i],sizeof(double ***));
    for(j=0; j < pedigreesize[i]; j++)
      if( tjpotflag[i][j] ) {
	tjpotential[i][j] = (double ***) calloc(nloci,sizeof(double **));
	tsimjpotential[i][j] = (double ***) calloc(nloci,sizeof(double **));
	for(k=0; k < nloci; k++) {
	  sizeA = tspacesize[i][j][k];
	  sizeB = tspacesize[i][peelinto[i][j]][k];
	  tjpotential[i][j][k] = (double **) calloc(sizeA,sizeof(double *));
	  tsimjpotential[i][j][k] = (double **) calloc(sizeA,sizeof(double *));
	  for(l=0; l < sizeA; l++) {
	    tjpotential[i][j][k][l] = (double *) calloc(sizeB,sizeof(double));
	    tsimjpotential[i][j][k][l] =(double *) calloc(sizeB,sizeof(double));
          }
	}
      }
  }

  *spacesize   = tspacesize;
  *ibdobs      = tibdobs;
  *ibdsim      = tibdsim;
  *ibdF        = tibdF;
  *ibdM        = tibdM;
  *ppotential  = tppotential;
  *wppotential = twppotential;
  *simwppotential = tsimwppotential;
  *jpotflag    = tjpotflag;
  *jpotential  = tjpotential; 
  *simjpotential  = tsimjpotential; 
  *simhap      = tsimhap;
  *shap        = tshap;
  *hap         = thap;
}


void DebugCreateSpaces(int npedigrees,
                       int *pedigreesize,
                       int ***pedigree,
                       int nmarkers,
                       int ****marker,
                       int ***spacesize,
                       int **jpotflag)
{
  char response[2];

  printf("\n\nDebug CreateSpaces()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {
    
    int i;
    int j;
    int k;

    printf("\n\n");
    for(i=0; i < npedigrees; i++) {
      printf("Space Sizes for Pedigree #%d",i+1);
      printf("\n\n");
      printf("PersonID  ");
      for(j=0; j < nmarkers; j++) {
        if( j < 9 ) {
          printf("M0%1dA M0%1dB  SS0%1d  ",j+1,j+1,j+1);
        } else {
          printf("M%2dA M%2dB  SS%2d  ",j+1,j+1,j+1);
        }
      }
      printf("\n");
      printf("--------  ");
      for(j=0; j < nmarkers; j++) {
        printf("---- ----  ----  ");
      }
      printf("\n");
      for(j=0; j < pedigreesize[i]; j++) {
        printf("%8d  ",pedigree[i][j][PID]);
        for(k=0; k < nmarkers; k++) {
          printf("%4d %4d  %4d  ",
                 marker[i][j][k][A],
                 marker[i][j][k][B],
                 spacesize[i][j][k]);
        }
        printf("\n");
      }
    }
    printf("\n jpotflags\n\n");
    for(i=0; i<npedigrees; i++) {
      printf("Pedigree #%d\n\n",i+1);
      printf("PersonID Flag?\n");
      printf("-------- -----\n");
      for(j=0; j<pedigreesize[i]; j++)
        printf("%8d %5d\n",pedigree[i][j][PID],jpotflag[i][j]);
    }
  }
}
