/* ***************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      getancestry.c
   Author:    Mark E. Irwin
   Date:      July 17, 1992
   Purpose:   Determines the relationships and stores for easy access
	      the relationships between members of the pedigrees.

*************************************************************************** */

#include <stdio.h>

#define TRUE  1
#define FALSE 0

#define PID 0
#define FID 1
#define MID 2

#define FOUNDER 0
#define PARENT  1
#define FINDEX  2
#define MINDEX  3
#define SID     4
#define SINDEX  5
#define SEX     6
#define ANCESTRYSIZE 7

#define MALE   1
#define FEMALE 0

void GetAncestry(int npedigrees,
		 int *pedigreesize,
		 int ***pedigree,
		 int ****ancestry)
{
  int    i;
  int    j;
  int    k;
  int    l;
  int ***tancestry;


  tancestry = (int ***) calloc(npedigrees,sizeof(int **));
  for(i=0; i < npedigrees; i++) {
    tancestry[i] = (int **) calloc(pedigreesize[i],sizeof(int *));
    for(j=0; j < pedigreesize[i]; j++) {
      tancestry[i][j] = (int *) calloc(ANCESTRYSIZE,sizeof(int));
    }
  }

  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++)
      if( pedigree[i][j][FID] == 0 &&
	  pedigree[i][j][MID] == 0 ) {
	tancestry[i][j][FOUNDER] = TRUE;
	tancestry[i][j][PARENT]  = TRUE;
	for(k=0; k < pedigreesize[i]; k++) {
	  if( pedigree[i][j][PID] == pedigree[i][k][FID] ) {
	    tancestry[i][j][SEX] = MALE;
	    tancestry[i][j][SID] = pedigree[i][k][MID];
	    break;
	  }
	  if( pedigree[i][j][PID] == pedigree[i][k][MID] ) {
	    tancestry[i][j][SEX] = FEMALE;
	    tancestry[i][j][SID] = pedigree[i][k][FID];
	    break;
	  }
	}
      } else {
	tancestry[i][j][FOUNDER] = FALSE;
	for(k=0; k < pedigreesize[i]; k++) {
	  if( pedigree[i][j][FID] == pedigree[i][k][PID] )
	    tancestry[i][j][FINDEX] = k;
	  if( pedigree[i][j][MID] == pedigree[i][k][PID] )
	    tancestry[i][j][MINDEX] = k;
	  if( pedigree[i][j][PID] == pedigree[i][k][FID] ) {
	    tancestry[i][j][PARENT] = TRUE;
	    tancestry[i][j][SEX] = MALE;
	    tancestry[i][j][SID] = pedigree[i][k][MID];
	  }
	  if( pedigree[i][j][PID] == pedigree[i][k][MID] ) {
	    tancestry[i][j][PARENT] = TRUE;
	    tancestry[i][j][SEX] = FEMALE;
	    tancestry[i][j][SID] = pedigree[i][k][FID];
	  }
	}
      }

  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++)
      if( tancestry[i][j][PARENT] )
	for(k=0; k < pedigreesize[i]; k++)
	  if( tancestry[i][j][SID] == pedigree[i][k][PID] ) {
	    tancestry[i][j][SINDEX] = k;
	    break;
	  }


  *ancestry = tancestry;
}


void DebugGetAncestry(int npedigrees,
		      int *pedigreesize,
		      int ***pedigree,
		      int ***ancestry)
{
  char response[2];

  printf("\n\nDebug GetAncestry()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) exit(0);
  if(strcmp(response, "y") == 0) {

    char *labels[] ={ "NO",
		      "YES",
		      "FM",
		      "M"};

    int i;
    int j;

    printf("\n\n");
    for(i=0; i < npedigrees; i++) {
      printf("\n\n");
      printf("\t*** Ancestry for Pedigree #%d  (%d Persons) ***", 
	     i+1, pedigreesize[i]);
      printf("\n\n");
      printf("PersID Fdr? F Index FatherID M Index");
      printf(" MotherID Parent Sex S Index SpouseID");
      printf("\n");
      printf("------ ---- ------- -------- -------");
      printf(" -------- ------ --- ------- --------");
      for(j=0; j < pedigreesize[i]; j++) {
	if( ancestry[i][j][PARENT] ) {
	  if( ancestry[i][j][FOUNDER] ) {
	    printf("\n");
	    printf("%6d %4s    NONE %8d    NONE %8d %6s %3s %7d %8d",
		   pedigree[i][j][PID],
		   labels[ancestry[i][j][FOUNDER]],
		   pedigree[i][j][FID],
		   pedigree[i][j][MID],
		   labels[ancestry[i][j][PARENT]],
		   labels[2+ancestry[i][j][SEX]],
		   ancestry[i][j][SINDEX],
		   ancestry[i][j][SID]);
	  } else {
	    printf("\n");
	    printf("%6d %4s %7d %8d %7d %8d %6s %3s %7d %8d",
		   pedigree[i][j][PID],
		   labels[ancestry[i][j][FOUNDER]],
		   ancestry[i][j][FINDEX],
		   pedigree[i][j][FID],
		   ancestry[i][j][MINDEX],
		   pedigree[i][j][MID],
		   labels[ancestry[i][j][PARENT]],
		   labels[2+ancestry[i][j][SEX]],
		   ancestry[i][j][SINDEX],
		   ancestry[i][j][SID]);
	  }
	} else {
	  printf("\n");
	  printf("%6d %4s %7d %8d %7d %8d %6s N/A    NONE %8d",
		 pedigree[i][j][PID],
		 labels[ancestry[i][j][FOUNDER]],
		 ancestry[i][j][FINDEX],
		 pedigree[i][j][FID],
		 ancestry[i][j][MINDEX],
		 pedigree[i][j][MID],
		 labels[ancestry[i][j][PARENT]],
		 ancestry[i][j][SID]);
	}
      }
    }
    printf("\n\n");
  }
}

