/* *************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      getibdobs.c
   Author:    Mark E. Irwin
   Date:      July 29, 1992
   Purpose:   Determines the ibd variables, where possible, from
	      input marker and disease data.
   Modied:    January 27, 2002.  orderhaplotypes was modified
	      to give an error message if an impossible genotype
              was given.

************************************************************* */

#include <stdio.h>

#define FOUNDER 0
#define PARENT  1
#define FINDEX  2
#define MINDEX  3
#define SID     4
#define SINDEX  5
#define SEX     6

#define A 0
#define B 1

#define F 0
#define M 1

#define FATHER 0
#define MOTHER 1

#define FROMPARENTSFATHER 0
#define FROMPARENTSMOTHER 1
#define UNKNOWN           2

#define PID 0

void GetIBDObs(int npedigrees,
	       int *pedigreesize,
	       int ***ancestry,
               int nmarkers,
	       int *nalleles,
	       int ****marker,
	       int isdisease,
	       int nloci,
	       int ****ibdobs)
{
  int i;
  int j;
  int k;
  int mA;
  int mB;
  int findex;
  int fmA;
  int fmB;
  int mindex;
  int mmA;
  int mmB;
  int fromF;
  int fromM;
  int PfromF;
  int PfromM;


  void orderhaplotypes();


  for(i=0; i < nmarkers; i++)
    for(j=0; j < npedigrees; j++)
      for(k=0; k < pedigreesize[j]; k++) {
        mA = marker[j][k][i][A];
        mB = marker[j][k][i][B];
        if( !ancestry[j][k][FOUNDER] &&
            mA != 0 && mB != 0) {
          findex = ancestry[j][k][FINDEX];
          mindex = ancestry[j][k][MINDEX];
          fmA = marker[j][findex][i][A];
          fmB = marker[j][findex][i][B];
          mmA = marker[j][mindex][i][A];
          mmB = marker[j][mindex][i][B];
	  orderhaplotypes(k,mA,mB,fmA,fmB,mmA,mmB,&fromF,&fromM);
          if( fromF && fromM ) {
            if( !ancestry[j][findex][FOUNDER] &&
                fmA != fmB ) {
	      orderhaplotypes(k,fmA,fmB,
			      marker[j][ancestry[j][findex][FINDEX]][i][A],
			      marker[j][ancestry[j][findex][FINDEX]][i][B],
                              marker[j][ancestry[j][findex][MINDEX]][i][A],
			      marker[j][ancestry[j][findex][MINDEX]][i][B],
			      &PfromF,&PfromM);
              if( PfromF && PfromM )
                if( fromF == PfromF )
                  ibdobs[j][k][i][FATHER] = FROMPARENTSFATHER;
		else
                  ibdobs[j][k][i][FATHER] = FROMPARENTSMOTHER;
            }
            if( !ancestry[j][mindex][FOUNDER] &&
                mmA != mmB ) {
	      orderhaplotypes(k,mmA,mmB,
			      marker[j][ancestry[j][mindex][FINDEX]][i][A],
			      marker[j][ancestry[j][mindex][FINDEX]][i][B],
			      marker[j][ancestry[j][mindex][MINDEX]][i][A],
			      marker[j][ancestry[j][mindex][MINDEX]][i][B],
			      &PfromF,&PfromM);
              if( PfromF && PfromM )
                if( fromM == PfromF )
                  ibdobs[j][k][i][MOTHER] = FROMPARENTSFATHER;
		else
                  ibdobs[j][k][i][MOTHER] = FROMPARENTSMOTHER;
            }
          }
        }
      }

}


void orderhaplotypes(int k,
		     int mA,
		     int mB,
		     int fmA,
		     int fmB,
		     int mmA,
		     int mmB,
		     int *fromF,
		     int *fromM)
{
/* first make sure that the genotype is feasible */
  if( mA !=0 && mB !=0){
    if( fmA != 0 && fmB != 0)
      if( mA != fmA && mA != fmB &&
	( mB != fmA && mB != fmB) ){
	fprintf(stderr,"member %d\n",k+1);
	nerror("Impossible genotype found!");
      }
    if( mmA != 0 && mmB != 0)
      if( mA != mmA && mA != mmB &&
	( mB != mmA && mB != mmB) ){
	fprintf(stderr,"member %d\n",k+1);
	nerror("Impossible genotype found!");
      }
  }

  if( mA == fmA || mA == fmB ||
     (fmA == 0 && fmB == 0) ) {
    if( mB == fmA || mB == fmB ||
       (fmA == 0 && fmB == 0) ) {
      if( mA == mmA || mA == mmB ||
	 (mmA == 0 && mmB == 0) ) {
	if( mB == mmA || mB == mmB ||
	   (mmA == 0 && mmB == 0) ) {
	  if( mA != mB ) {
	    *fromF = 0;
	    *fromM = 0;
          } else {
            *fromF = mA;
	    *fromM = mB;
          }
	} else {
	  *fromF = mB;
	  *fromM = mA;
	}
      } else {
	*fromF = mA;
	*fromM = mB;
      }
    } else {
      *fromF = mA;
      *fromM = mB;
    }
  } else {
    *fromF = mB;
    *fromM = mA;
  }

}

void DebugGetIBDObs(int npedigrees,
                    int *pedigreesize,
                    int ***pedigree,
                    int nmarkers,
                    int ****ibd)
{ 
  char response[2];

  printf("\n\nDebug GetIBDObs()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) {

    int i;
    int j;
    int k;
    int l;

    printf("\n\n");
    for(i=0; i < npedigrees; i++) {
      printf("Pedigree #%d\n\n",i+1); 
      for(j=0; j < pedigreesize[i]; j++) {
        printf("%5d  ",pedigree[i][j][PID]);
        for(l=0; l<=1; l++) {
          if (l==0) printf("FromF ");
            else printf("       FromM ");
          for(k=0; k < nmarkers; k++) 
            printf("%d  ",ibd[i][j][k][l]);
	printf("\n");
        }
      }
    }
  }

}

void UpdateIBDsim(int npedigrees,
		  int *pedigreesize,
		  int ***ancestry,
		  int locus,
		  int ****simmarker,
		  int ****ibd)
{
  int i,j;
  int mF,mM;
  int fmF,fmM;
  int mmF,mmM;
  int findex,mindex;

  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++) {
      if (!ancestry[i][j][FOUNDER]) {
        mF = simmarker[i][j][locus][F];
        mM = simmarker[i][j][locus][M];
	findex = ancestry[i][j][FINDEX];
	mindex = ancestry[i][j][MINDEX];
	fmF = simmarker[i][findex][locus][F];
	fmM = simmarker[i][findex][locus][M];
	mmF = simmarker[i][mindex][locus][F];
	mmM = simmarker[i][mindex][locus][M];
	if (fmF != fmM)
	  if (fmF == mF) ibd[i][j][locus][F] = FROMPARENTSFATHER;
	    else ibd[i][j][locus][F] = FROMPARENTSMOTHER;
	if (mmF != mmM)
	  if (mmF == mM) ibd[i][j][locus][M] = FROMPARENTSFATHER;
	    else ibd[i][j][locus][M] = FROMPARENTSMOTHER;
      }
    }
}

void CopyIBD(int npedigrees,
	     int *pedigreesize,
	     int nloci,
	     int ****ibd1,
	     int ****ibd2)

{
  int i,j,k;

  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++)
      for(k=0; k<nloci; k++) {
	ibd2[i][j][k][F] = ibd1[i][j][k][F];
	ibd2[i][j][k][M] = ibd1[i][j][k][M];
      }
}

