/* *************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      getprobibdobs.c
   Author:    Mark E. Irwin
   Date:      July 17, 1992
   Purpose:   Calculates P[ibdobs]

************************************************************************* */

#include <stdio.h>

#define F 0
#define M 1

#define PID 0

double GetProbIBDObs(int npedigrees,
                     int *pedigreesize,
                     int ****ibdobs,
		     int *ibdF,
		     int *ibdM,
		     int nloci,
		     int model,
                     double **prob)

{
  int i,j;
  int f,m;

  double dnum;

  void Getibd();
  int unbase();

  dnum = 1.0;
  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++) {
      Getibd(i,j,F,nloci,ibdobs,ibdF);
      Getibd(i,j,M,nloci,ibdobs,ibdM);
      f = unbase(nloci,3,ibdF);
      m = unbase(nloci,3,ibdM);
      dnum *= prob[model][f] * prob[model][m];
    }
  return(dnum);
}

void DebugGetProbIBDObs(int npedigrees,
                        int *pedigreesize,
			int ***pedigree,
                        int ****ibdobs,
                        int *ibdF,
                        int *ibdM,
                        int nloci,
			int model,
                        double **prob)

{
  char response[2];

  printf("\n\nDebug GetProbIBDObs()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) exit(0);
  if( strcmp(response,"n") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {

    int i,j,k;
    int f,m;

    double dnum;

    void Getibd();
    int unbase();

    dnum = 1.0;
    for(i=0; i<npedigrees; i++)
      for(j=0; j<pedigreesize[i]; j++) {
        Getibd(i,j,F,nloci,ibdobs,ibdF);
        Getibd(i,j,M,nloci,ibdobs,ibdM);
        f = unbase(nloci,3,ibdF);
        m = unbase(nloci,3,ibdM);
	dnum *= prob[model][f] * prob[model][m];
	printf("%5d From F ",pedigree[i][j][PID]);
        for(k=0; k<nloci; k++)
          printf("%d",ibdF[k]);
	printf(" P[ibdobsF] = %le\n",prob[model][f]);
        printf("%5d From M ",pedigree[i][j][PID]);
        for(k=0; k<nloci; k++)
          printf("%d",ibdM[k]);
        printf(" P[ibdobsM] = %le\n",prob[model][m]);
      }
    printf("\n\nProb[ibdobs] = %le\n",dnum);
  }
}

double CalcLikRatio(int npedigrees,
		    int *pedigreesize,
		    int ****ibd,
		    int *ibdF,
		    int *ibdM,
		    int nloci,
		    int model,
		    int sub,
		    double **prob,
		    double ***likprob)
{
  int i,j,f,m;
  double dnum;
  double num,den;

  void Getibd();
  int unbase();
  void nerror();

  dnum=1.0;
  num = 1.0;
  den = 1.0;
  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++) {
      Getibd(i,j,F,nloci,ibd,ibdF);
      Getibd(i,j,M,nloci,ibd,ibdM);
      f = unbase(nloci,3,ibdF);
      m = unbase(nloci,3,ibdM);
      /* causes numerical underflow */
      /* num *= (likprob[model][sub][f] * likprob[model][sub][m]); */
      /* den *= (prob[model][m] * prob[model][f]); */
      num = (likprob[model][sub][f] * likprob[model][sub][m]); 
      den = (prob[model][m] * prob[model][f]); 
      if(den > 0.0)
        dnum *= (num/den);
      else nerror("CalclikRatio:  numerical underflow");
    }
  /* if(den > 0.0) dnum = num / den;
    else dnum = 0.0; */
  return(dnum);
}
   
