#include <malloc.h>
#include <stdio.h>
#include <math.h>

void allmat(n,p,u,amat)
/*p is the model size, and n is the number of loci*/
int n,p;
double *u,*amat; /*u is the interval length ad amat is all the matrices needed*/

{
  int i,j,k;
  double up;
  void indimat();

  for (i=0;i<n;i++)
    {
      up = pow (*(u+i) , (double) p);
      /*printf("up=%f\n", up);*/
      indimat(n,p,*(u+i),  amat+i*2*p*p);
    }
}

void indimat(n,p,u,result)   /* Program to calculate the individual matrix */
int n,p;
double *result;
double u;

{ 
  double *sequence;
  double *dvector();
  double singsum();
  double factrl();
  double up,cf;
  double powtemp;   /* added for 0^0 problem  */
  void free_dvector();
  int seqlength;
  int i,j;

  seqlength = 2 * p - 1;
  if (u>60.0) u=50.0;
  cf = exp(-u);
  /*printf("cf=%e\n", cf);*/
  up = pow(u, (double) p);
  sequence = dvector(1,seqlength);

  for (i=p;i<=seqlength;i++)
    { /* Calculate the elemnets of a Toeplitz matrix */
      sequence[i] = singsum(n,p,u,i,up);
      /*printf("%f\n", sequence[i]);*/
    }
 
  if(p> 1)
    {
      for (i=1;i<p;i++)
        sequence[i] = sequence[i+p] + pow(u,(double) i)/factrl(i);
    }

  for (i=0;i<p;i++)
    for (j=0;j<p;j++)
      *(result+i*p+j) = sequence[p-i+j]*cf/((double) 2);

  for (i=p;i<=seqlength;i++)
    {
      if (i==p) powtemp = 1.0;
      else powtemp = pow(u,(double)(i-p));
          /* modification for 0^0 problem  */
      sequence[i] = sequence[i] + 2.0 * powtemp / factrl(i-p);
    }

  for(i=0;i<p;i++)
    for(j=0;j<p;j++)
      *(result+p*p+i*p+j) = sequence[p-i+j]*cf/2.0;

  free_dvector(sequence,1,2*p-1);
}

/* This is the function used to calculate the sum of the form
u^i0/[(i0)]! + u^(i0+p)/[(i0+p)]! + ........
x equals u^p which is used here to reduce the calculation intensity */

/* This should be used together with nrerror.c, gammaln.c and factrl.c */

double singsum(n,p,u,i0,x)
double u,x;
int i0,n,p;

{
  double s0,s1,x0;
  double epsilon;
  int i,n1;

  double factrl();

  epsilon = 1e-38;

  s0 = 0.0;
  s1 = 0.0;
  if (x == 0.0) x0 = 1.0;   /* to eliminate div by 0 problem  */
    else x0 = pow(u,(double) i0)/x;
  i = 0;

  do {
       s0 = s1;
       n1 = p * i + i0;
       s1 = s0 + x0 * x / factrl(n1);
       i++;
       x0 = x0 * x;
     } while (fabs(s0-s1) > epsilon);

  return(s1);
}

/* the matrix product function */

void matprod(x,y,xy,l,m,n) /* x is the pointer to the X matrix, y to the Y
			    matrix, and xy to their product. X is an l*m matrix,
			    Y is an m*n matrix and XY is an l*n matrix */
double *x,*y,*xy;
int l,m,n;

{ 
   int i,j,k;

   for (i=0;i<l;i++) 
      for (j=0;j<n;j++)
        {
	  *(xy+i*n+j) = 0.0;
          for (k=0;k<m;k++)
	    *(xy+i*n+j) += (*(x+i*m+k)) * (*(y+k*n+j));
        }
}


double gammln(xx)
double xx;
{
	double x,tmp,ser;
	static double cof[6]={76.18009173,-86.50532033,24.01409822,
		-1.231739516,0.120858003e-2,-0.536382e-5};
	int j;

	x=xx-1.0;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.0;
	for (j=0;j<=5;j++) {
		x += 1.0;
		ser += cof[j]/x;
	}
	return(-tmp+log(2.50662827465*ser));
}

double factrl(n)
int n;
{
	static int ntop=4;
	static double a[33]={1.0,1.0,2.0,6.0,24.0};
	int j;
	double gammln();
	void nrerror();

	/*printf("%d\n",n);*/
	if (n < 0) nrerror("Negative factorial in routine FACTRL");
	if (n > 32) return(exp(gammln(n+1.0)));
	while (ntop<n) {
		j=ntop++;
		a[ntop]=a[j]*ntop;
	}
	return(a[n]);
}
