/* *********************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      map.c 
   Author:    Zachary Skrivanek 
   Date:      April 15, 1999 
   Modified:  August 29, 1999 converts distances to recomb fracs 
   Purpose:   Converts recombination fractions to genetic distances 
   Modified:  Nov 21, 2001 to convert to cM instead of M

********************************************************************* */

#include <stdio.h>
#include <math.h>

#define  to_cmorgan      0
#define  to_recomb_frac 1



/* maps 2 matrices of recomb values */
void MapMatrix(int     nloci,
	       int     nmodels,
	       double  **theta, 
	       int     *numlik,
               double  ***liktheta,
	       int     isdisease,
	       int     which)

{
  int     i,j,k;

  void    MapVector(int,double *,int);

  for(i=0; i<nmodels; i++) 
  {

    if(isdisease && i==0)
      MapVector(nloci-2,&theta[i][0],which);
    else MapVector(nloci-1,&theta[i][0],which);
    if(isdisease && which==to_recomb_frac && i==0)
      theta[i][nloci-2]=0.5;
    for(k=0; k<numlik[i]; k++)
    {
      if(isdisease && i==0)
        MapVector(nloci-2,&liktheta[i][k][0],which);
      else MapVector(nloci-1,&liktheta[i][k][0],which);
      if(isdisease && which==to_recomb_frac && i==0)
        liktheta[i][k][nloci-2]=0.5;
    } 
  }
}
 

void DebugMapMatrix(int     nloci,
                    int     nmodels,
                    double  **theta,
                    int     *numlik,
                    double  ***liktheta,
		    int     isdisease,
		    int     which)
{
  char    response[2];

  printf("\n\nDebug MapMatrix ()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0)
  {
    int i,j,k;


    printf("\n\n");
    for(i=0; i<nmodels; i++) 
    {  
      if(which==to_cmorgan) 
      {
	printf("\nGenetic Distances for Model %d\n",i);
        printf(" Loci    Genetic Distance\n");
      } 
      else if(which==to_recomb_frac)
      {
	printf("\nRecomb Fractions for Model %d\n",i);
        printf(" Loci    Recomb Fraction\n");
      }
      for(j=0; j<(nloci-1-(i==0 && isdisease && which==to_cmorgan)); j++)
          printf("%2d %2d     %le \n",j+1,j+2,theta[i][j]);
      for(k=0; k<numlik[i]; k++)   
      {
        if(which==to_cmorgan) 
        {
          printf("\nGenetic Distances for model %d, submodel %d are:\n",i,k);
          printf(" Loci    Genetic Distance\n");
        } 
        else if(which==to_recomb_frac) 
        {
          printf("\nRecomb Fractions for model %d, submodel %d are:\n",i,k);
          printf(" Loci    Recomb Fractions\n");
        } 
        for(j=0; j<(nloci-1-(i==0 && isdisease && which==to_cmorgan)); j++)
          printf("%2d %2d     %le \n",j+1,j+2,liktheta[i][k][j]);  
      }
    }
  }
}
 
/* maps a vector of recombination values */
void MapVector(int     nloci,
               double  *theta,
	       int     which)
{
  int i;


  if(which==to_cmorgan)
  for(i=0; i<nloci; i++)
    theta[i]= -100*log(1.0 - 2.0 * theta[i])/2.0; 
  else if(which==to_recomb_frac)
  for(i=0; i<nloci; i++)
    theta[i]= (1-exp(-2.0 * theta[i]/100.0))/2.0;
  else
  {
    fprintf(stderr,"'which' value %d not valid\n",which);
    return;
  }
}

void MapDistance(double  *theta,
	         int     which)
{
  int i;

  if(which==to_cmorgan)
  *theta = -100*log(1.0 - 2.0 * *theta)/2.0; 
  else if(which==to_recomb_frac)
  *theta = (1-exp(-2.0 * *theta/100.0))/2.0;
  else
  {
    fprintf(stderr,"'which' value %d not valid\n",which);
    return;
  }
}
