/* ***************************************************************************


   Program:   mlla (multi-locus linkage analysis)
   File:      peel.c
   Author:    Mark E. Irwin
   Date:      July 17, 1992
   Purpose:   Does a single locus peel conditional on observed ibds of
	      other loci.
   Modified:  September 1, 2002.  made prob a one dim array instead of two 
	      dimensional array

*************************************************************************** */

#include <stdio.h>

#define PID 0
#define F 0
#define M 1
#define SEX 6

 
void Peel(int npedigrees,
	  int *pedigreesize,
	  int ***ancestry,
	  int **peelorder,
	  int **peeltype,
	  int **peelinto,
	  int nloci,
	  int loci,
	  int ***spacesize,
	  int **jpotflag,
          int *****haplabel,
	  int ****ibd,
	  int *ibdF,
	  int *ibdM,
	  double ****ppotential,
	  double *****jpotential,
	  double ****wppotential,
	  double *prob)

{
  int foi;
  int ffoi;
  int i;
  int j;
  int k;
  int l;
  int m;
  int n;
  int ii;
  int jj;
  int lF,lM;
  int mF,mM;
  int nF,nM;

  double cpF,cpM;

  void Getibd();
  double condprob();
  double dsum;

 /**********************************************************************
  * initialize the working point potentials
  **********************************************************************/
  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++)
      for(k=0; k < spacesize[i][j][loci]; k++)
	wppotential[i][j][loci][k] = ppotential[i][j][loci][k];

 /**********************************************************************
  * initialize the joint potentials to 1
  **********************************************************************/
  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++)
      if( jpotflag[i][j] ) {
	for(k=0; k < spacesize[i][j][loci]; k++)
	  for(l=0; l < spacesize[i][peelinto[i][j]][loci]; l++)
	    jpotential[i][j][loci][k][l] = 1.0;
      }

 /**********************************************************************
  * peel
  **********************************************************************/
  for(i=0; i < npedigrees; i++) {
    for(j=1; j < pedigreesize[i]; j++) { /* root isn't peeled */
      for(k=0; k < pedigreesize[i]; k++) 
	if( peelorder[i][k] == j ) {
	  switch( peeltype[i][k] ) {
	  case 1:
	    foi  = peelinto[i][k];
	    ffoi = peelinto[i][foi];
            Getibd(i,k,F,nloci,ibd,ibdF);
            Getibd(i,k,M,nloci,ibd,ibdM);
	    for(l=0; l < spacesize[i][foi][loci];  l++)
	      for(m=0; m < spacesize[i][ffoi][loci]; m++) {
		dsum = 0.0;
                lF = haplabel[i][foi][loci][l][F];
                lM = haplabel[i][foi][loci][l][M];
                mF = haplabel[i][ffoi][loci][m][F];
		mM = haplabel[i][ffoi][loci][m][M];
		for(n=0; n < spacesize[i][k][loci]; n++) {
                  nF = haplabel[i][k][loci][n][F];
                  nM = haplabel[i][k][loci][n][M];
		  if( ancestry[i][foi][SEX] ) {
		    cpF = condprob(nF,lF,lM,nloci,loci,ibdF,prob);
		    cpM = condprob(nM,mF,mM,nloci,loci,ibdM,prob);
		    dsum += wppotential[i][k][loci][n] * cpF * cpM;
		  } else {
		    cpF = condprob(nF,mF,mM,nloci,loci,ibdF,prob);
		    cpM = condprob(nM,lF,lM,nloci,loci,ibdM,prob);
		    dsum += wppotential[i][k][loci][n] * cpF * cpM;
                  }
		}
		jpotential[i][foi][loci][l][m] *= dsum;
	      }
	    break;
	  case 2:
	    foi = peelinto[i][k];
	    for(l=0; l < spacesize[i][foi][loci]; l++) {
	      dsum = 0.0;
	      for(m=0; m < spacesize[i][k][loci]; m++) {
		jpotential[i][k][loci][m][l] *= wppotential[i][k][loci][m];
		dsum += jpotential[i][k][loci][m][l];
	      }
	      wppotential[i][foi][loci][l] *= dsum;
	    }
	    break;
	  case 3:
	    foi  = peelinto[i][k];
	    ffoi = peelinto[i][foi];
	    if( jpotflag[i][k] ) {
	      for(l=0; l < spacesize[i][foi][loci]; l++)
		for(m=0; m < spacesize[i][k][loci]; m++)
		  jpotential[i][k][loci][m][l] *= wppotential[i][k][loci][m];
	      for(l=0; l < spacesize[i][ffoi][loci]; l++)
		for(m=0; m < spacesize[i][foi][loci]; m++) {
		  dsum = 0.0;
		  lF = haplabel[i][ffoi][loci][l][F];
		  lM = haplabel[i][ffoi][loci][l][M];
		  mF = haplabel[i][foi][loci][m][F];
		  mM = haplabel[i][foi][loci][m][M];
		  Getibd(i,ffoi,F,nloci,ibd,ibdF);
		  Getibd(i,ffoi,M,nloci,ibd,ibdM);
		  for(n=0; n < spacesize[i][k][loci]; n++) {
		    nF = haplabel[i][k][loci][n][F];
                    nM = haplabel[i][k][loci][n][M];
		    if( ancestry[i][foi][SEX] ) {
                      cpF = condprob(lF,mF,mM,nloci,loci,ibdF,prob);
		      cpM = condprob(lM,nF,nM,nloci,loci,ibdM,prob);
		      dsum += jpotential[i][k][loci][n][m] * cpF * cpM;
		    } else {
		      cpF = condprob(lF,nF,nM,nloci,loci,ibdF,prob);
		      cpM = condprob(lM,mF,mM,nloci,loci,ibdM,prob);
                      dsum += jpotential[i][k][loci][n][m] * cpF * cpM;
		    }
		  }
		  jpotential[i][foi][loci][m][l] *= dsum;
		}
	    } else {
	      for(l=0; l < spacesize[i][ffoi][loci]; l++)
		for(m=0; m < spacesize[i][foi][loci]; m++) {
		  dsum = 0.0;
                  lF = haplabel[i][ffoi][loci][l][F];
                  lM = haplabel[i][ffoi][loci][l][M];
                  mF = haplabel[i][foi][loci][m][F];
                  mM = haplabel[i][foi][loci][m][M];
		  Getibd(i,ffoi,F,nloci,ibd,ibdF);
		  Getibd(i,ffoi,M,nloci,ibd,ibdM);
		  for(n=0; n < spacesize[i][k][loci]; n++) {
                    nF = haplabel[i][k][loci][n][F];
                    nM = haplabel[i][k][loci][n][M];
                    if( ancestry[i][foi][SEX] ) {
		      cpF = condprob(lF,mF,mM,nloci,loci,ibdF,prob);
                      cpM = condprob(lM,nF,nM,nloci,loci,ibdM,prob);
		      dsum += wppotential[i][k][loci][n] * cpF * cpM;
		    } else {
		      cpF = condprob(lF,nF,nM,nloci,loci,ibdF,prob);
                      cpM = condprob(lM,mF,mM,nloci,loci,ibdM,prob);
		      dsum += wppotential[i][k][loci][n] * cpF * cpM;
                    }
		  }
		  jpotential[i][foi][loci][m][l] *= dsum;
		}
	    }
	    break;
	  case 4:
	    foi = peelinto[i][k];
	    for(l=0; l < spacesize[i][foi][loci]; l++) {
	      dsum = 0.0;
	      for(m=0; m < spacesize[i][k][loci]; m++) {
		jpotential[i][k][loci][m][l] *= wppotential[i][k][loci][m];
		dsum += jpotential[i][k][loci][m][l];
	      }
	      wppotential[i][foi][loci][l] *= dsum;
	    }
	    break;
	  }
        }
    }
  }

 /**********************************************************************/

}

void Getibd(int i,
	    int j,
	    int k,
	    int nloci,
	    int ****ibd,
	    int *ibdk)
	    
{
  int m;

  for(m=0; m<nloci; m++)
    ibdk[m]=ibd[i][j][m][k];
}

void DebugPeel(int npedigrees,
	       int *pedigreesize,
	       int ***pedigree,
	       int loci,
	       int **peelinto,
	       int **jpotflag,
	       int ***spacesize,
	       int *****haplabel,
	       double ****wppotential,
	       double *****jpotential)
{
  char response[2];

  printf("\n\nDebug Peel ()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {

    int i;
    int j;
    int k;
    int l;

    printf("\n\n");
    printf("wppotentials\n\n");
    for(i=0; i<npedigrees; i++) {
      printf("       Pedigree #%d\n\n",i+1);
      printf("PersonID  FromF FromM h(genotype)\n");
      printf("--------  ----- ----- -----------\n");
      for(j=0; j < pedigreesize[i]; j++) {
        for(k=0; k<spacesize[i][j][loci]; k++)
          printf("%8d %5d %5d %le\n",pedigree[i][j][PID],
			haplabel[i][j][loci][k][F],haplabel[i][j][loci][k][M],
			wppotential[i][j][loci][k]);
        printf("\n\n");
      }
    printf("\n\n");
    }

    printf("jpotentials\n\n");
    for(i=0; i<npedigrees; i++) {
      printf("       Pedigree #%d\n\n",i+1);
      printf("PersonID  FromF FromM PeelInto ID FromF FromM h(genotype)\n");
      printf("--------  ----- ----- ----------- ----- ----- -----------\n");
      for(j=0; j < pedigreesize[i]; j++)
        if(jpotflag[i][j]) {
          for(k=0; k<spacesize[i][j][loci]; k++)
            for(l=0; l<spacesize[i][peelinto[i][j]][loci]; l++)
              printf("%8d %5d %5d %8d %5d %5d %le\n",
			pedigree[i][j][PID],
			haplabel[i][j][loci][k][F],haplabel[i][j][loci][k][M],
			pedigree[i][peelinto[i][j]][PID],
                        haplabel[i][peelinto[i][j]][loci][l][F],
                        haplabel[i][peelinto[i][j]][loci][l][M],
                        jpotential[i][j][loci][k][l]);
          printf("\n\n");
        }
    printf("\n\n");
    }
  fflush(stdout);
  }
}
