/* ***************************************************************************

   Program:   mlla (multi-locus linkage analysis)
   File:      readblank.c
   Author:    Mark E. Irwin
   Date:      July 17, 1992
   Purpose:   Reads in the ids of people who's ibds should not be
	      simulated for each locus.

*************************************************************************** */

#include <stdio.h>

#define PID 0
#define FALSE 0
#define TRUE 1

void ReadBlank(int npedigrees,
	       int *pedigreesize,
	       int ***pedigree,
	       int nmarkers,
	       int isdisease,
	       int nloci,
	       int ****blank)

{
  int i,j,k,l;
  int numtoread,pid;
  int ***tblank;
  FILE *fp;
  FILE *fopen();
  char    filename[80];

  tblank = (int ***) calloc(npedigrees,sizeof(int **));
  for(i=0; i<npedigrees; i++) {
    tblank[i] = (int **) calloc(pedigreesize[i],sizeof(int *));
    for(j=0; j<pedigreesize[i]; j++)
      tblank[i][j] = (int *) calloc(nloci,sizeof(int));
  }

  for(i=0; i<npedigrees; i++)
    for(j=0; j<pedigreesize[i]; j++)
      for(k=0; k<nloci; k++)
        tblank[i][j][k] = FALSE;

  for(k=0; k<nmarkers; k++) {
    sprintf(filename,"blank%d.dat",k+1);
    if((fp=fopen(filename,"r")) == NULL)
      nerror("blank file could not be opened for reading");

    for(i=0; i<npedigrees; i++) {
      fscanf(fp,"%d",&numtoread);
      for(l=0; l<numtoread; l++) {
        fscanf(fp,"%d",&pid);
        for(j=0; j<pedigreesize[i]; j++) 
          if(pedigree[i][j][PID] == pid) {
            tblank[i][j][k] = TRUE;
            break;
          }
        if(j == pedigreesize[i])
          nerror("blank file improperly formatted");
      }
    }
    fclose(fp);
  }
  if(isdisease) {
    if((fp = fopen("blankdis.dat","r")) == NULL)
      nerror("blankdis.dat could not be opened for reading");
    for(i=0; i<npedigrees; i++) {
      fscanf(fp,"%d",&numtoread);
      for(l=0; l<numtoread; l++) {
        fscanf(fp,"%d",&pid);
        for(j=0; j<pedigreesize[i]; j++)
          if(pedigree[i][j][PID] == pid) {
	    tblank[i][j][nloci-1] = TRUE;
	    break;
          }
        if(j == pedigreesize[i])
	  nerror("blankdis.dat could not be opened for reading");
      }
    }
    fclose(fp);
  }
  *blank = tblank;
}

void DebugReadBlank(int npedigrees,
		    int *pedigreesize,
		    int ***pedigree,
		    int nloci,
		    int ***blank)
{
  char response[2];

  printf("\n\nDebug ReadBlank()? => ");
  scanf("%s",response);
  if( strcmp(response,"q") == 0 ) return;
  if( strcmp(response,"y") == 0 ) {

    int i,j,k;

    printf("\n\n");
    for(k=0; k<nloci; k++) {
      printf("People not to have ibds simulated for locus %d\n",k+1);
      printf("\nPedigree PersonID SimStatus\n");
      printf("------- -------- ---------\n\n");
      for(i=0; i<npedigrees; i++) {
	for(j=0; j<pedigreesize[i]; j++)
          printf("%6d   %6d   %6d\n",i+1,pedigree[i][j][PID],
						blank[i][j][k]);
        printf("\n");
      }
    }
  }
}
