/* **********************************************************************

    Program:   mlla(multi-locus linkage analysis)
    File:      readrecombfracs.c
    Author:    Mark E. Irwin
    Date:      Sept 29, 1992
    Purpose:   Reads in the recombination fractions for a
	       non-interference model.

************************************************************************* */
#include <stdio.h>

void ReadRecombFracs(int nloci,
		     int nmodels,
		     int ***locusorder,
		     double ***theta,
		     int **numlik,
		     double ****liktheta)

{
  FILE *fp;
  FILE *fopen();

  int i,j,k;
  int **tlocusorder;
  double **ttheta;
  int *tnumlik;
  double ***tliktheta;

  if( (fp = fopen("theta.dat","r")) == NULL)
     nerror("theta.dat not available for reading");

  tlocusorder = (int **) calloc(nmodels,sizeof(int *));
  tnumlik = (int *) calloc(nmodels,sizeof(int));
  ttheta = (double **) calloc(nmodels,sizeof(double *));
  tliktheta = (double ***) calloc(nmodels,sizeof(double **));
  for(i=0; i<nmodels; i++) {
    ttheta[i] = (double *) calloc(nloci-1,sizeof(double));
    tlocusorder[i] = (int *) calloc(nloci,sizeof(int));
  }

  for(i=0; i<nmodels; i++) {
    for(j=0; j<nloci; j++)
      fscanf(fp,"%d",&tlocusorder[i][j]);
    for(j=0; j<nloci-1; j++)
      fscanf(fp,"%le ",&ttheta[i][j]);
    fscanf(fp,"%d",&tnumlik[i]);
    tliktheta[i] = (double **) calloc(tnumlik[i],sizeof(double *));
    for(j=0; j<tnumlik[i]; j++) {
      tliktheta[i][j] = (double *) calloc(nloci-1,sizeof(double));
      for(k=0; k<nloci-1; k++)
        fscanf(fp,"%le",&tliktheta[i][j][k]);
    }
  }

  *locusorder = tlocusorder;
  *theta = ttheta;
  *numlik = tnumlik;
  *liktheta = tliktheta;
}


void DebugReadRecombFracs(int nloci,
			  int nmodels,
			  int **locusorder,
			  double **theta,
			  int *numlik,
			  double ***liktheta)
{
  char response[2];

  printf("\n\nDebug ReadRecombFracs ()? =>");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) {

    int i,j,k;

    printf("\n\n");
    for(i=0; i<nmodels; i++) {
      printf("The locus order for model %d is:",i);
      for(j=0; j<nloci; j++)
        printf(" %d",locusorder[i][j]);
      printf("\n\n");
      printf("Recombination Probabilities for Model %d\n\n",i);
      printf(" Loci Prob[Recombination]\n");
      for(j=0; j<nloci-1; j++)
        printf("%2d %2d    %7.5lf \n",j+1,j+2,theta[i][j]);
      for(k=0; k<numlik[i]; k++) {
        printf("\nRecombination Probabilities for model %d, submodel %d are:\n"
								,i,k);
        printf(" Loci Prob[Recombination]\n");
        for(j=0; j<nloci-1; j++)
          printf("%2d %2d    %7.5lf \n",j+1,j+2,liktheta[i][k][j]);
      }
    }
  }
}

