/* Modified September 2, 2002 to make prob 1-dim array instead of 2 */

#include <stdio.h>

#define PID 0
#define F 0
#define M 1
#define SEX 6

void Simulate(int npedigrees,
	      int *pedigreesize,
	      int ***ancestry,
	      int ***spacesize,
	      int **jpotflag,
	      int **peelorder,
	      int **peeltype,
	      int **peelinto,
	      int *****haplabel,
	      int nloci,
	      int locus,
	      int ****ibd,
	      int *ibdF,
	      int *ibdM,
	      double ****wppotential,
	      double ****simwppotential,
	      double *****jpotential,
	      double *****simjpotential,
	      int ****simhap,
	      int ***shap,
	      double *prob)
{

  int i;
  int j;
  int k;
  int l;
  int foi;
  int ffoi;
  int hapf;
  int hapff;
  int lF,lM;
  int mF,mM;
  int nF,nM;

  double dtemp,drnd;
  double cpF,cpM;
  double rnd();
  double condprob();

  void Getibd();
  for(i=0; i < npedigrees; i++)
    for(j=0; j < pedigreesize[i]; j++) {
      for(k=0; k < spacesize[i][j][locus]; k++)
	simwppotential[i][j][locus][k] = wppotential[i][j][locus][k];
      if( jpotflag[i][j] ) {
	foi = peelinto[i][j];
	for(k=0; k < spacesize[i][j][locus]; k++)
	  for(l=0; l < spacesize[i][foi][locus]; l++)
	    simjpotential[i][j][locus][k][l] = jpotential[i][j][locus][k][l];
      }
    }


  for(i=0; i < npedigrees; i++) {

    /*******************************************************************
     * simulate from the root distribution
     *******************************************************************/
    for(j=0; j < pedigreesize[i]; j++)
      if ( peelorder[i][j] == pedigreesize[i] ) {
	for(k=1; k < spacesize[i][j][locus]; k++)
	  simwppotential[i][j][locus][k] += simwppotential[i][j][locus][k-1];
        drnd = rnd();
	dtemp = drnd * simwppotential[i][j][locus][spacesize[i][j][locus]-1];
	for(k=0; k < spacesize[i][j][locus]-1; k++) 
	  if( dtemp <= simwppotential[i][j][locus][k] )
	    break;
	shap[i][j][locus] = k;
        simhap[i][j][locus][F] = haplabel[i][j][locus][k][F];
        simhap[i][j][locus][M] = haplabel[i][j][locus][k][M];
      }

   /**********************************************************************
    * simulate the remaining in reverse peeling order
    **********************************************************************/
    for(j=pedigreesize[i]-1; j > 0; j--) {
      for(k=0; k < pedigreesize[i]; k++)
	if( peelorder[i][k] == j )
	  switch( peeltype[i][k] ) {
	  case 1:
	    foi  = peelinto[i][k];
	    ffoi = peelinto[i][foi];
	    hapf = shap[i][foi][locus];
	    hapff = shap[i][ffoi][locus];
	    mF = haplabel[i][foi][locus][hapf][F];
	    mM = haplabel[i][foi][locus][hapf][M];
	    nF = haplabel[i][ffoi][locus][hapff][F];
	    nM = haplabel[i][ffoi][locus][hapff][M];
	    Getibd(i,k,F,nloci,ibd,ibdF);
	    Getibd(i,k,M,nloci,ibd,ibdM);
	    for(l=0; l < spacesize[i][k][locus]; l++) {
	      lF = haplabel[i][k][locus][l][F];
	      lM = haplabel[i][k][locus][l][M];
              if (ancestry[i][foi][SEX]) {
	        cpF = condprob(lF,mF,mM,nloci,locus,ibdF,prob);
		cpM = condprob(lM,nF,nM,nloci,locus,ibdM,prob);
		simwppotential[i][k][locus][l] *= (cpF * cpM); 
	      } else {
	        cpF = condprob(lF,nF,nM,nloci,locus,ibdF,prob);
		cpM = condprob(lM,mF,mM,nloci,locus,ibdM,prob);
		simwppotential[i][k][locus][l] *= (cpF * cpM); 
              }
	    }
	    for(l=1; l < spacesize[i][k][locus]; l++)
	      simwppotential[i][k][locus][l] +=simwppotential[i][k][locus][l-1];
            drnd = rnd();
	    dtemp = drnd*simwppotential[i][k][locus][spacesize[i][k][locus]-1];
	    for(l=0; l < spacesize[i][k][locus]; l++)
	      if( dtemp <= simwppotential[i][k][locus][l])
		break;
	    shap[i][k][locus] = l;
	    simhap[i][k][locus][F] = haplabel[i][k][locus][l][F];
            simhap[i][k][locus][M] = haplabel[i][k][locus][l][M];
	    break;
	  case 2:
	    foi = peelinto[i][k];
	    hapf = shap[i][foi][locus];
	    for(l=1; l < spacesize[i][k][locus]; l++)
	      simjpotential[i][k][locus][l][hapf] +=
					 simjpotential[i][k][locus][l-1][hapf];
            drnd = rnd();
	    dtemp = drnd*simjpotential[i][k][locus][spacesize[i][k][locus]-1][hapf];
	    for(l=0; l < spacesize[i][k][locus]; l++)
	      if( dtemp <= simjpotential[i][k][locus][l][hapf])
		break;
	    shap[i][k][locus] = l;
            simhap[i][k][locus][F] = haplabel[i][k][locus][l][F];
            simhap[i][k][locus][M] = haplabel[i][k][locus][l][M];
	    break;
	  case 3:
	    foi  = peelinto[i][k];
	    ffoi = peelinto[i][foi];
	    hapf = shap[i][foi][locus];
	    hapff = shap[i][ffoi][locus];
	    mF = haplabel[i][foi][locus][hapf][F];
	    mM = haplabel[i][foi][locus][hapf][M];
	    nF = haplabel[i][ffoi][locus][hapff][F];
	    nM = haplabel[i][ffoi][locus][hapff][M];
	    Getibd(i,ffoi,F,nloci,ibd,ibdF);
	    Getibd(i,ffoi,M,nloci,ibd,ibdM);
	    if( jpotflag[i][k] ) {
	      for(l=0; l < spacesize[i][k][locus]; l++) {
	        lF = haplabel[i][k][locus][l][F];
	        lM = haplabel[i][k][locus][l][M];
                if (ancestry[i][foi][SEX]) {
	          cpF = condprob(nF,mF,mM,nloci,locus,ibdF,prob);
		  cpM = condprob(nM,lF,lM,nloci,locus,ibdM,prob);
		  simjpotential[i][k][locus][l][hapf] *= (cpF * cpM); 
	        } else {
	          cpF = condprob(nF,lF,lM,nloci,locus,ibdF,prob);
		  cpM = condprob(nM,mF,mM,nloci,locus,ibdM,prob);
		  simjpotential[i][k][locus][l][hapf] *= (cpF * cpM); 
                }
	      }
	      for(l=1; l < spacesize[i][k][locus]; l++)
		simjpotential[i][k][locus][l][hapf] +=
				 simjpotential[i][k][locus][l-1][hapf];
              drnd = rnd();
	      dtemp = drnd *
		    simjpotential[i][k][locus][spacesize[i][k][locus]-1][hapf];
	      for(l=0; l < spacesize[i][k][locus]; l++)
		if( dtemp <= simjpotential[i][k][locus][l][hapf])
		  break;
	      shap[i][k][locus] = l;
              simhap[i][k][locus][F] = haplabel[i][k][locus][l][F];
              simhap[i][k][locus][M] = haplabel[i][k][locus][l][M];
	    } else {
	      for(l=0; l < spacesize[i][k][locus]; l++) {
	        lF = haplabel[i][k][locus][l][F];
	        lM = haplabel[i][k][locus][l][M];
                if (ancestry[i][foi][SEX]) {
	          cpF = condprob(nF,mF,mM,nloci,locus,ibdF,prob);
		  cpM = condprob(nM,lF,lM,nloci,locus,ibdM,prob);
		  simwppotential[i][k][locus][l] *= (cpF * cpM); 
	        } else {
	          cpF = condprob(nF,lF,lM,nloci,locus,ibdF,prob);
		  cpM = condprob(nM,mF,mM,nloci,locus,ibdM,prob);
		  simwppotential[i][k][locus][l] *= (cpF * cpM); 
                }
	      }
	      for(l=1; l < spacesize[i][k][locus]; l++) 
		simwppotential[i][k][locus][l] +=
					simwppotential[i][k][locus][l-1];
	      drnd = rnd();
	      dtemp = drnd *
			simwppotential[i][k][locus][spacesize[i][k][locus]-1];
	      for(l=0; l < spacesize[i][k][locus]; l++)
		if( dtemp <= simwppotential[i][k][locus][l])
		  break;
	      shap[i][k][locus] = l;
              simhap[i][k][locus][F] = haplabel[i][k][locus][l][F];
              simhap[i][k][locus][M] = haplabel[i][k][locus][l][M];
	    }
	    break;
	  case 4:
	    foi = peelinto[i][k];
	    hapf = shap[i][foi][locus];
	    for(l=1; l < spacesize[i][k][locus]; l++)
	      simjpotential[i][k][locus][l][hapf] +=
					simjpotential[i][k][locus][l-1][hapf];
	    drnd = rnd();
	    dtemp = drnd *
	 	    simjpotential[i][k][locus][spacesize[i][k][locus]-1][hapf];
	    for(l=0; l < spacesize[i][k][locus]; l++)
	      if( dtemp <= simjpotential[i][k][locus][l][hapf])
		break;
	    shap[i][k][locus] = l;
            simhap[i][k][locus][F] = haplabel[i][k][locus][l][F];
            simhap[i][k][locus][M] = haplabel[i][k][locus][l][M];
	    break;
	  }
    }
  }

}


void DebugSimulate(int npedigrees,
		   int *pedigreesize,
		   int ***pedigree,
		   int ****simhap,
		   int locus)
{
  char response[2];

  printf("\n\nDebug Simulate()? => ");
  scanf("%s", response);
  if(strcmp(response, "q") == 0) return;
  if(strcmp(response, "y") == 0) {

    int i;
    int j;

    for(i=0; i < npedigrees; i++) {
      printf("\n\n");
      printf("\t*** Pedigree #%d  (%d Persons) ***", i+1, pedigreesize[i]);
      printf("\n\n");
      printf("PersonID  From F  From M");
      printf("\n");
      printf("--------  ------  ------");
      for(j=0; j < pedigreesize[i]; j++) {
	printf("\n");
	printf("%8d  %6d  %6d",
	       pedigree[i][j][PID],
	       simhap[i][j][locus][F],
	       simhap[i][j][locus][M]);
      }
    }
    printf("\n\n");
  }
}
