#include <malloc.h>
#include <stdio.h> /* FILE */
#include <math.h>
#include <stdlib.h> /* exit() */

#define PI 3.1415926535897932384626433832795028841972

void allmat(n,p,u,amat)
/*p is the model size, and n is the number of loci*/
int n,p;
double *u,*amat; /*u is the interval length ad amat is all the matrices needed*/

{
  int i,j,k;
  double up;
  void indimat();

  for (i=0;i<n;i++)
    {
      up = pow (*(u+i) , (double) p);
      /*printf("up=%f\n", up);*/
      indimat(n,p,*(u+i),  amat+i*2*p*p);
    }
}

void indimat(n,p,u,result)   /* Program to calculate the individual matrix */
int n,p;
double *result;
double u;

{ 
  double *sequence;
  double *dvector();
  double singsum();
  double factrl();
  double up,cf;
  double powtemp;   /* added for 0^0 problem  */
  void free_dvector();
  int seqlength;
  int i,j;

  seqlength = 2 * p - 1;
  if (u>60.0) u=50.0;
  cf = exp(-u);
  /*printf("cf=%e\n", cf);*/
  up = pow(u, (double) p);
  sequence = dvector(1,seqlength);

  for (i=p;i<=seqlength;i++)
    { /* Calculate the elemnets of a Toeplitz matrix */
      sequence[i] = singsum(n,p,u,i,up);
      /*printf("%f\n", sequence[i]);*/
    }
 
  if(p> 1)
    {
      for (i=1;i<p;i++)
        sequence[i] = sequence[i+p] + pow(u,(double) i)/factrl(i);
    }

  for (i=0;i<p;i++)
    for (j=0;j<p;j++)
      *(result+i*p+j) = sequence[p-i+j]*cf/((double) 2);

  for (i=p;i<=seqlength;i++)
    {
      if (i==p) powtemp = 1.0;
      else powtemp = pow(u,(double)(i-p));
          /* modification for 0^0 problem  */
      sequence[i] = sequence[i] + 2.0 * powtemp / factrl(i-p);
    }

  for(i=0;i<p;i++)
    for(j=0;j<p;j++)
      *(result+p*p+i*p+j) = sequence[p-i+j]*cf/2.0;

  free_dvector(sequence,1,2*p-1);
}

/* This is the function used to calculate the sum of the form
u^i0/[(i0)]! + u^(i0+p)/[(i0+p)]! + ........
x equals u^p which is used here to reduce the calculation intensity */

/* This should be used together with nrerror.c, gammaln.c and factrl.c */

double singsum(n,p,u,i0,x)
double u,x;
int i0,n,p;

{
  double s0,s1,x0;
  double epsilon;
  int i,n1;

  double factrl();

  epsilon = 1e-38;

  s0 = 0.0;
  s1 = 0.0;
  if (x == 0.0) x0 = 1.0;   /* to eliminate div by 0 problem  */
    else x0 = pow(u,(double) i0)/x;
  i = 0;

  do {
       s0 = s1;
       n1 = p * i + i0;
       s1 = s0 + x0 * x / factrl(n1);
       i++;
       x0 = x0 * x;
     } while (fabs(s0-s1) > epsilon);

  return(s1);
}

/* the matrix product function */

void matprod(x,y,xy,l,m,n) /* x is the pointer to the X matrix, y to the Y
			    matrix, and xy to their product. X is an l*m matrix,
			    Y is an m*n matrix and XY is an l*n matrix */
double *x,*y,*xy;
int l,m,n;

{ 
   int i,j,k;

   for (i=0;i<l;i++) 
      for (j=0;j<n;j++)
        {
	  *(xy+i*n+j) = 0.0;
          for (k=0;k<m;k++)
	    *(xy+i*n+j) += (*(x+i*m+k)) * (*(y+k*n+j));
        }
}


double gammln(xx)
double xx;
{
	double x,tmp,ser;
	static double cof[6]={76.18009173,-86.50532033,24.01409822,
		-1.231739516,0.120858003e-2,-0.536382e-5};
	int j;

	x=xx-1.0;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.0;
	for (j=0;j<=5;j++) {
		x += 1.0;
		ser += cof[j]/x;
	}
	return(-tmp+log(2.50662827465*ser));
}

double factrl(n)
int n;
{
	static int ntop=4;
	static double a[33]={1.0,1.0,2.0,6.0,24.0};
	int j;
	double gammln();
	void nrerror();

	/*printf("%d\n",n);*/
	if (n < 0) nrerror("Negative factorial in routine FACTRL");
	if (n > 32) return(exp(gammln(n+1.0)));
	while (ntop<n) {
		j=ntop++;
		a[ntop]=a[j]*ntop;
	}
	return(a[n]);
}

void basen(int nloci, int base, int count, int *cbase)
{
  int temp,i;

  temp = count;
  for (i=0; i<nloci; i++)
  {
    cbase[i] = temp%base;
    temp = (temp-cbase[i])/base;
  }
}

int unbase(int nloci, int base, int *z)
{
  int l,m;

  m = z[nloci-1];
  for(l=nloci-2;l>=0;l--)
    m = base*m + z[l];
  return(m);
}

int power(int base, int expon)
{
  int i;
  int temp;

  temp=1;
  for(i=0; i<expon; i++)
    temp*=base;

  return(temp);
}


void nerror( char *error_text )
{

  fprintf( stderr, "Run-time error...\n" );
  fprintf( stderr, "%s\n", error_text );
  fprintf( stderr, "...now exiting to system...\n" );

  exit( 1 );
}

FILE *gfopen(char *fn, char *mode)
{
  FILE  *fp;

  if((fp = fopen(fn, mode)) == NULL) {
    fprintf(stderr, "Cannot open %s - bye!\n", fn);
    exit(1);
  }
  return fp;
}


int firstm(int nloci, int m, int *cbase)
{
  int min,i;

  min=nloci; i=0;
  while ((min==nloci) && (i<nloci))
  {
    if (cbase[i]==m) min = i;
    i++;
  }
  return(min);
}

/*
void tostorageorder(int nloci,
                    int *order,
                    int *c2,
                    int *s2)

{
  int i;

  for(i=0; i<nloci; i++)
    s2[order[i]-1] = c2[i];
}
*/

void swap(int *x, int *y){
  int    tmp1,tmp2;

  tmp1=*x;
  tmp2=*y;
  *x=tmp2;
  *y=tmp1;
}

void sort(int *x, int *order, int n){
/* order x in descending order by array order of length n */
/* but don't change the array 'order' ! */

  int    *torder;
  int    i,j;

  torder = (int *) calloc(n, sizeof(int));
  for(i=0; i<n; i++)
    torder[i]=order[i];

  for(i=0; i<n-1; i++)
    for(j=n-1; j>i; j--)
      if(torder[j-1] < torder[j]){
        swap(&x[j],&x[j-1]);
        swap(&torder[j],&torder[j-1]);
      }
  free(torder);
}




void tolocusorder(int nloci,
                  int *order,
                  int *s2,
                  int *c2)

{
  int i;

  for(i=0; i<nloci; i++)
    c2[i] = s2[order[i]-1];
}

double mean(double *x, int n)
{
  int    i;
  double sum=0.0;

  for(i=0; i<n; i++)
    sum += x[i];

  return(sum/((double) n));
}

double var(double *x, int n)
{

  int     i;
  double  sum=0.0;
  double  mean(double*, int);
  double  pow();

  for(i=0; i<n; i++)
    sum += pow(x[i],2.0);

  return((sum - pow(mean(x, n),2.0) * (double) n)/((double) n - 1));

}

double cov(double *x, double *y, int n)
{

  int     i;
  double  sum=0.0;
  double  mean(double*, int);

  for(i=0; i<n; i++)
    sum += x[i]*y[i];

  return((sum - mean(x, n) * mean(y,n) * (double) n)/((double) n - 1));

}

double max(double x, double y) {
  if(x > y) return(x);
  else return(y);
}

double abs_value(double x){
  if(x>0.0) return(x);
  else return(-1*x);
}

double norm_pdf(double x, double mu, double sigma_2){
  return(exp(-1*pow(x-mu,2.0)/(2*sigma_2))/sqrt(2*PI*sigma_2));
}

/********************************************

              Seed Functions

********************************************/

/* writes random seed to 'seed' file */

void PlantSeed()
{
  FILE   *fp, *fopen();
  int    fclose();
  
  fp = fopen("seed","w");
  
  fprintf(fp, "seed %d\n",(int) lrand48() );
  fclose(fp);
}
