/* *********************************************************************

   Program:   SIMPLE 
   File:      utils.c
   Author:    Mark E. Irwin
   Date:      July 17, 1992
   Purpose:   Utilities for base to base transformations and identifying
	      patterns in the expansions.
   Modified:  1/30/02-  included runif(), this is a redundant use
              of rnd() which I should eliminate.

********************************************************************* */

#include <stdio.h>
#include <math.h> /* multi_base */
#include <stdlib.h> /* multi_base */
#include "simple.h"

void basen(int nloci, int base, int num, int *cbase)
{
  int temp,i;

  temp = num;
  for (i=0; i<nloci; i++)
  {
    cbase[i] = temp%base;
    temp = (temp-cbase[i])/base;
  }
}

int firstm(int nloci, int m, int *cbase)
{
  int min,i;

  min=nloci; i=0;
  while ((min==nloci) && (i<nloci))
  {
    if (cbase[i]==m) min = i;
    i++;
  }
  return(min);
}

int unbase(int nloci, int base, int *z)
{
  int l,m;

  m = z[nloci-1];
  for(l=nloci-2;l>=0;l--)
    m = base*m + z[l];
  return(m);
}

int power(int base, int expon)
{
  int i;
  int temp;

  temp=1;
  for(i=0; i<expon; i++)
    temp*=base;

  return(temp);
}



/********************************************

              Seed Functions

********************************************/

/* writes random seed to 'seed' file */

void PlantSeed()
{
  FILE   *fp, *fopen();
  int    fclose();
  
  fp = fopen("seed","w");
  
  fprintf(fp, "seed %d\n",(int) lrand48() );
  fclose(fp);
}

void swap(int *x, int *y){
  int    tmp1,tmp2;

  tmp1=*x;
  tmp2=*y;
  *x=tmp2;
  *y=tmp1;
}

void sort(int *x, int *order, int n){
/* order x in descending order by array order of length n */
/* but don't change the array 'order' ! */

  int    *torder;
  int    i,j;

  torder = (int *) calloc(n, sizeof(int));
  for(i=0; i<n; i++)
    torder[i]=order[i];

  for(i=0; i<n-1; i++)
    for(j=n-1; j>i; j--)
      if(torder[j-1] < torder[j]){
	swap(&x[j],&x[j-1]);
	swap(&torder[j],&torder[j-1]);
      }
  free(torder);
}

FILE *gfopen(char *fn, char *mode)
{
  FILE  *fp;

  if((fp = fopen(fn, mode)) == NULL) {
    fprintf(stderr, "Cannot open %s - bye!\n", fn);
    exit(1);
  }
  return fp;
}

/****************************************************************/
/*  error handling routine which exits to system with a message */
/****************************************************************/

void nerror( char *error_text )
{

  fprintf( stderr, "Run-time error...\n" );
  fprintf( stderr, "%s\n", error_text );
  fprintf( stderr, "...now exiting to system...\n" );

  exit( 1 );
}

void multi_base(int n, int *base, int num, int *cbase)
{
  int   temp,i,j,tbase;
  div_t d;

  tbase=1;
  for(j=0; j<n; j++)
    tbase *= base[j];

  temp = num;
  for (i=n-1; i>=0; i--)
  {
    tbase /= base[i];
    d=div(temp, tbase);
    cbase[i] = d.quot;
    temp = d.rem;
  }
}


int unmulti_base(int n, int *base, int *z)
{
  int i,m;

  m = z[n-1];
  for(i=n-2; i>=0; i--)
    m = base[i]*m + z[i];
  return(m);
}

/* take subset of whaps (indexed by set) */
/* and put into whaps2 */
void  subset(int     *whaps,
	     int     n,
	     int     *set,
	     int     *whaps2)
{

  int  i;

  for(i=0; i < n; i++)
     whaps2[i]= whaps[set[i]];
}


/*******************************************************

random uniform number generation with drand48 not rand()

********************************************************/


double runif( void )
{
  double rndnumber;

  double drand48();


  rndnumber = drand48();
  return (rndnumber);

}
